/// \file Bat.cpp Source file for the Bat class

#include <sstream>

#include "SDL_gfxPrimitives.h"
#include "SDL_mixer.h"

#include "Bat.hpp"
#include "LevelWriter.hpp"

SDL_Surface *Bat::batImage;
SDL_Surface *Bat::batAsleep;
Mix_Chunk *Bat::batFeedingSound;

unsigned int Bat::batcounter = 0;

Bat::Bat(Level *l, Vec2f p) :
	GameObject(l, p, BAT_DEFAULT_MAX_SPEED, BAT_DEFAULT_RADIUS),
	waypoints(1, p),
	wp_index(0),
	asleep(true),
	anim_timer(0)
{
	if (batcounter++ == 0) {
		batImage = LoadSprite(PKGDATADIR"Artwork/creatures/bat/bat.png",
							  CREATURE_ROTATIONS,
							  level->GetTileWidth(),
							  level->GetTileHeight());
		batAsleep = LoadSprite(PKGDATADIR"Artwork/creatures/bat/bat_asleep.png",
							   CREATURE_ROTATIONS,
							   level->GetTileWidth(),
							   level->GetTileHeight());
		batFeedingSound = LoadSound(PKGDATADIR"Sounds/BatAttack.ogg");
	}

	image = batImage;
	SetOrientation(M_PIf*1.5f);
}

Bat::~Bat()
{
	if (--batcounter == 0) {
		SDL_FreeSurface(batImage);
		SDL_FreeSurface(batAsleep);
		Mix_FreeChunk(batFeedingSound);
	}
}

void Bat::Run()
{
	if (disabled)
		return;

	//determine if I change state
	asleep = level->IsItDay();
	if (!asleep && (waypoints.at(wp_index) - position).Length() < max_speed)
		wp_index = (wp_index + 1)%waypoints.size();

	//Set goal based on my state
	if (asleep) {
		velocity = start_position - position;
		wp_index = 0;
	} else
		velocity = waypoints.at(wp_index) - position;

	if (velocity.Length() > max_speed)
		velocity *= max_speed/velocity.Length();

	position += velocity;

	UpdateImage();

}

void Bat::Reset()
{
	this->GameObject::Reset();

	max_speed = BAT_DEFAULT_MAX_SPEED;
	radius = BAT_DEFAULT_RADIUS;

	wp_index = 0;
	asleep = true;

	image = batImage;
}

void Bat::UpdateImage()
{
	if (asleep && velocity == Vec2f(0,0)) {
		anim_index = 0;
		image = batAsleep;
	} else {
		anim_timer = (anim_timer + 1) % BAT_ANIMATION_SPEED;
		image = batImage;
		if (anim_timer == 0)
			anim_index = (anim_index + 1) % 4;
	}
}

std::string Bat::GetXMLRepresentation(const std::string& indent) const
{
	std::ostringstream xml;
	xml << indent << "<creature x=\"" << level->GetTilePos(position).x + 0.5f
		<< "\" y=\"" << level->GetTilePos(position).y + 0.5f
		<< "\" type=\"bat\"";

	if (HasColor())
		xml << " color=\"" << GetColor().name() << "\"";

	std::string wptstring = LevelWriter::waypointsToXML(indent+"  ", position,
														waypoints.begin(),
														waypoints.end());
	if (wptstring.size() == 0)
		xml << "/>\n";
	else
		xml << ">\n" << wptstring << indent << "</creature>\n";

	return xml.str();
}

bool Bat::IsPatroller() {
	return true;
}

void Bat::DrawWaypoints(SDL_Surface *screen, ColorRGBA color)
{
	Vec2f first = waypoints.front();
	Vec2f prev;
	for (std::vector<Vec2f>::iterator i = waypoints.begin();
		 i != waypoints.end(); i++) {
		if (*i != first)
			aalineRGBA(screen, prev.x, prev.y, (*i).x, (*i).y,
					   color.r, color.g, color.b, color.a);

		prev = *i;
	}
	aalineRGBA(screen, prev.x, prev.y, first.x, first.y,
			   color.r, color.g, color.b, color.a);
}

void Bat::ResetWaypoints(Vec2f pos){
	waypoints.clear();
	waypoints.push_back(pos);
}
// vim: set ts=4 sts=4 sw=4 noet:
