/// \file Bat.hpp Header file for the Bat class

/// \brief Include guard for Bat.hpp
#ifndef INCLUDED_BAT_H
#define INCLUDED_BAT_H 1

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif

#include "GameObject.hpp"
#include "Utility.hpp"
#include "Patroller.hpp"

/**
 * \brief Class representing a swarm of bats.
 */
class Bat : public GameObject, public Patroller {
public:
	/** \brief Constructs a Bat in Level l at position p
	* \param l a pointer to the Level this Bat exists in
	* \param p the Vec2f start_position of this Bat
	* \param c the Color of this Bat
	*/
	Bat(Level *l, Vec2f p);

	/** \brief Destructs a Bat */
	~Bat();

	/** \brief Runs one cycle of this Bat.
	*/
	void Run(void);

	/** \brief Resets this Bat to its original state in its Level.
	*/
	void Reset(void);

	inline void AddWaypoint(const Vec2f waypoint, bool duringday = true,
							bool duringnight = true) {waypoints.push_back(waypoint);};

	//GET methods

	std::string GetXMLRepresentation(const std::string& indent) const;

	/** \brief Gets the GameObjectType of this Bat.
	 * \return the GameObjectType of this Bat
	 */
	inline GameObjectType GetType(void) const { return GAME_OBJECT_BAT; }

	void PlayCollisionSound() const;

	/** \brief draws the creature's waypoints
	 *
	 * Draws the creature's waypoints as dots and path between them as lines
	 */
	void DrawWaypoints(SDL_Surface *screen, ColorRGBA color);

	/** \brief resets creature's waypoints
	 *
	 * Clears all the creature's waypoints except for the origin
	 */
	void ResetWaypoints(Vec2f pos);

	bool IsPatroller();

private:
	void UpdateImage(void);			///< Updates the image of this Bat

	static SDL_Surface *batImage;				///< The image of a Bat
	static SDL_Surface *batAsleep;
	static Mix_Chunk *batFeedingSound;			///< Sound of the Bat eating
	static unsigned int batcounter;	///< Number of Bats created

	std::vector<Vec2f> waypoints;	///< The waypoints of this Bat
	int wp_index;					///< The current index of this Bat's waypoints

	bool asleep;					///< True if this bat is asleep, false otherwise

	unsigned int anim_timer;		///< The timer until this Bat changes to its next frame.

	Vec2f pt1, pt2, pt3, pt4;
};

inline void Bat::PlayCollisionSound() const
{
	SoundManager::PlayChunk(batFeedingSound);
}

#include "GameObject.hpp"
#include "Utility.hpp"

#endif /* INCLUDED_BAT_H */
// vim: set ts=4 sts=4 sw=4 noet:
