/// \file Bee.cpp Source file for the Bee class

#include <iostream>
#include <sstream>

#include "SDL_gfxPrimitives.h"

#include "ColorRGBA.hpp"
#include "Bee.hpp"
#include "LevelWriter.hpp"

SDL_Surface *Bee::bee_black;
SDL_Surface *Bee::bee_blue;
SDL_Surface *Bee::bee_brown;
SDL_Surface *Bee::bee_green;
SDL_Surface *Bee::bee_orange;
SDL_Surface *Bee::bee_red;
SDL_Surface *Bee::bee_violet;
SDL_Surface *Bee::bee_white;
SDL_Surface *Bee::bee_yellow;
Mix_Chunk *Bee::bee_flutter;

unsigned int Bee::beecounter = 0;

Bee::Bee(Level *l, Vec2f p, ColorRGBA c) :
	GameObject(l, p, c, BEE_DEFAULT_MAX_SPEED, BEE_DEFAULT_RADIUS),
	waypoints(1, p),
	wp_index(0),
	anchorpoint(p),
	state(BEE_STATE_PATHING),
	anim_timer(0)
{
	if (beecounter++ == 0) {
		bee_black = LoadSprite(PKGDATADIR"Artwork/creatures/bee/bee-black.png",
							   CREATURE_ROTATIONS,
							   level->GetTileWidth(),
							   level->GetTileHeight());
		bee_blue = LoadSprite(PKGDATADIR"Artwork/creatures/bee/bee-blue.png",
							  CREATURE_ROTATIONS,
							  level->GetTileWidth(),
							  level->GetTileHeight());
		bee_brown = LoadSprite(PKGDATADIR"Artwork/creatures/bee/bee-brown.png",
							   CREATURE_ROTATIONS,
							   level->GetTileWidth(),
							   level->GetTileHeight());
		bee_green = LoadSprite(PKGDATADIR"Artwork/creatures/bee/bee-green.png",
							   CREATURE_ROTATIONS,
							   level->GetTileWidth(),
							   level->GetTileHeight());
		bee_orange = LoadSprite(PKGDATADIR"Artwork/creatures/bee/bee-orange.png",
								CREATURE_ROTATIONS,
								level->GetTileWidth(),
								level->GetTileHeight());
		bee_red = LoadSprite(PKGDATADIR"Artwork/creatures/bee/bee-red.png",
							 CREATURE_ROTATIONS,
							 level->GetTileWidth(),
							 level->GetTileHeight());
		bee_violet = LoadSprite(PKGDATADIR"Artwork/creatures/bee/bee-violet.png",
								CREATURE_ROTATIONS,
								level->GetTileWidth(),
								level->GetTileHeight());
		bee_white = LoadSprite(PKGDATADIR"Artwork/creatures/bee/bee-white.png",
							   CREATURE_ROTATIONS,
							   level->GetTileWidth(),
							   level->GetTileHeight());
		bee_yellow = LoadSprite(PKGDATADIR"Artwork/creatures/bee/bee-yellow.png",
								CREATURE_ROTATIONS,
								level->GetTileWidth(),
								level->GetTileHeight());

		bee_flutter = LoadSound(PKGDATADIR"Sounds/BeeAttack.ogg");
	}

	SetOrientation(M_PIf*1.5f);
	UpdateImage();
}

Bee::~Bee()
{
	if (--beecounter == 0) {
		SDL_FreeSurface(bee_black);
		SDL_FreeSurface(bee_blue);
		SDL_FreeSurface(bee_brown);
		SDL_FreeSurface(bee_green);
		SDL_FreeSurface(bee_orange);
		SDL_FreeSurface(bee_red);
		SDL_FreeSurface(bee_violet);
		SDL_FreeSurface(bee_white);
		SDL_FreeSurface(bee_yellow);
		Mix_FreeChunk(bee_flutter);
	}
}

void Bee::Run()
{
	if (disabled)
		return;

	Charlie *c = level->GetCharlie();

	//Decide if my state/wp_index has changed
	switch(state) {
		case BEE_STATE_AGGRO:
			if (c->GetColor() != GetColor())
			{
				state = BEE_STATE_DEAGGRO;
				max_speed = BEE_DEFAULT_MAX_SPEED;
			}
			break;
		case BEE_STATE_PATHING:
			if(c->GetColor() == GetColor())
			{
				state = BEE_STATE_AGGRO;
				max_speed = BEE_AGGRO_MAX_SPEED;
				anchorpoint = position;
				SoundManager::PlayChunk(bee_flutter);
			}
			else if(waypoints.at(wp_index).x - max_speed - 1 <= position.x && position.x <= waypoints.at(wp_index).x + max_speed + 1
				 && waypoints.at(wp_index).y - max_speed - 1 <= position.y && position.y <= waypoints.at(wp_index).y + max_speed + 1)
				wp_index = (wp_index + 1)%waypoints.size();
			break;
		case BEE_STATE_DEAGGRO:
			if (c->GetColor() == GetColor())
			{
				state = BEE_STATE_AGGRO;
				max_speed = BEE_AGGRO_MAX_SPEED;
			}
			else if(anchorpoint.x - max_speed - 1 <= position.x && position.x <= anchorpoint.x + max_speed + 1
			&& anchorpoint.y - max_speed - 1 <= position.y && position.y <= anchorpoint.y + max_speed + 1)
				state = BEE_STATE_PATHING;
			break;
#ifdef DEBUG
		default:
			std::cerr << "Unexpected Bee AI state " << state << "\n";
			break;
#endif
	}

	Vec2f goal;

	//Set goal based on my state
	switch(state) {
		case BEE_STATE_AGGRO: goal = c->GetPosition(); break;
		case BEE_STATE_PATHING: goal = waypoints.at(wp_index); break;
		case BEE_STATE_DEAGGRO: goal = anchorpoint; break;
#ifdef DEBUG
		default:
			std::cerr << "Unexpected Bee AI state " << state << "\n";
			break;
#endif
	}

	velocity = goal - position;
	if (velocity.Length() > max_speed) {
		velocity *= max_speed/velocity.Length();
		velocity += velocity.PerpVec() * GetOscillatingValue(-1.0f, 1.0f);
	}
	position += velocity;

	anim_timer = (anim_timer + 1)%BEE_ANIMATION_SPEED;
	if(anim_timer == 0)
		anim_index = (anim_index + 1)%4;

}

void Bee::Reset()
{
	this->GameObject::Reset();

	max_speed = BEE_DEFAULT_MAX_SPEED;
	radius = BEE_DEFAULT_RADIUS;

	wp_index = 0;
	anchorpoint = start_position;
	state = BEE_STATE_PATHING;
	anim_index = 0;
	anim_timer = 0;

	UpdateImage();
}

std::string Bee::GetXMLRepresentation(const std::string& indent) const
{
	std::ostringstream xml;
	xml << indent << "<creature x=\"" << level->GetTilePos(position).x + 0.5f
		<< "\" y=\"" << level->GetTilePos(position).y + 0.5f
		<< "\" type=\"bee\"";

	if (HasColor())
		xml << " color=\"" << GetColor().name() << "\"";
	
	std::string wptstring = LevelWriter::waypointsToXML(indent+"  ", position,
														waypoints.begin(),
														waypoints.end());
	if (wptstring.size() == 0)
		xml << "/>\n";
	else
		xml << ">\n" << wptstring << indent << "</creature>\n";

	return xml.str();
}

void Bee::UpdateImage()
{
	ColorRGBA myColor = GetColor();

	if(myColor == COLOR_BLACK)
		image = bee_black;
	else if(myColor == COLOR_BLUE)
		image = bee_blue;
	else if(myColor == COLOR_BROWN)
		image = bee_brown;
	else if(myColor == COLOR_GREEN)
		image = bee_green;
	else if(myColor == COLOR_ORANGE)
		image = bee_orange;
	else if(myColor == COLOR_RED)
		image = bee_red;
	else if(myColor == COLOR_VIOLET)
		image = bee_violet;
	else if(myColor == COLOR_WHITE)
		image = bee_white;
	else
		image = bee_yellow;
}

bool Bee::IsPatroller(){
	return true;
}

void Bee::DrawWaypoints(SDL_Surface *screen, ColorRGBA color)
{
	Vec2f first = waypoints.front();
	Vec2f prev;
	for (std::vector<Vec2f>::iterator i = waypoints.begin();
		 i != waypoints.end(); i++) {
		if (*i != first)
			aalineRGBA(screen, prev.x, prev.y, (*i).x, (*i).y,
					   color.r, color.g, color.b, color.a);

		prev = *i;
	}
	aalineRGBA(screen, prev.x, prev.y, first.x, first.y,
			   color.r, color.g, color.b, color.a);
}

void Bee::ResetWaypoints(Vec2f pos){
	waypoints.clear();
	waypoints.push_back(pos);
}

// vim: set ts=4 sts=4 sw=4 noet:
