/// \file Bee.hpp Header file for the Bee class

/// \brief Include guard for Bee.hpp
#ifndef INCLUDED_BEE_H
#define INCLUDED_BEE_H 1

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif

#include "GameObject.hpp"
#include "Utility.hpp"
#include "Patroller.hpp"

/**
 * \brief Class representing a swarm of bees.
 */
class Bee : public GameObject, public Patroller {
public:
	/** \brief Constructs a Bee in Level l at position p that likes Color c.
	* \param l a pointer to the Level this Bee exists in
	* \param p the Vec2f start_position of this Bee
	* \param c the Color of this Bee
	*/
	Bee(Level *l, Vec2f p, ColorRGBA c);

	/** \brief Destructs a Bee */
	~Bee();

	/** \brief Runs one cycle of this Bee.
	*/
	void Run(void);

	/** \brief Resets this Bee to its original state in its Level.
	*/
	void Reset(void);

	inline void AddWaypoint(const Vec2f waypoint, bool duringday = true,
							bool duringnight = true) {waypoints.push_back(waypoint);};

	//GET methods

	std::string GetXMLRepresentation(const std::string& indent) const;

	/** \brief Gets the GameObjectType of this Bee.
	 * \return the GameObjectType of this Bee
	 */
	inline GameObjectType GetType(void) const { return GAME_OBJECT_BEE; }

		/** \brief draws the creature's waypoints
	 *
	 * Draws the creature's waypoints as dots and path between them as lines
	 */
	void DrawWaypoints(SDL_Surface *screen, ColorRGBA color);

	/** \brief resets creature's waypoints
	 *
	 * Clears all the creature's waypoints except for the origin
	 */
	void ResetWaypoints(Vec2f pos);

	bool IsPatroller();

private:
	/** \brief Updates the image of this Bee.
	*/
	void UpdateImage();

	static SDL_Surface *bee_black;
	static SDL_Surface *bee_blue;
	static SDL_Surface *bee_brown;
	static SDL_Surface *bee_green;
	static SDL_Surface *bee_orange;
	static SDL_Surface *bee_red;
	static SDL_Surface *bee_violet;
	static SDL_Surface *bee_white;
	static SDL_Surface *bee_yellow;
	static Mix_Chunk *bee_flutter;

	static unsigned int beecounter;	///< Number of Bees created

	std::vector<Vec2f> waypoints;	///< The waypoints of this Bee
	int wp_index;					///< The current index of this Bee's waypoints
	Vec2f anchorpoint;				///< The anchorpoint of this Bee; set when it aggros, returns here when it deaggros

	enum BeeState {					///< Describes the mental state of a Bee
		BEE_STATE_AGGRO,
		BEE_STATE_DEAGGRO,
		BEE_STATE_PATHING
	};

	BeeState state;					///< The mental state of this Bee
	unsigned int anim_timer;		///< Time between each frame of this Bee's animation.
	Vec2f pt1, pt2, pt3, pt4;
};

#include "GameObject.hpp"
#include "Utility.hpp"

#endif /* INCLUDED_BEE_H */
// vim: set ts=4 sts=4 sw=4 noet:
