/// \file Centipede.cpp Source file for the Centipede class

#include <sstream>

#include "SDL_gfxPrimitives.h"

#include "Centipede.hpp"
#include "LevelWriter.hpp"

SDL_Surface *Centipede::centipedeImage;
SDL_Surface *Centipede::rolledUpImage;
Mix_Chunk *Centipede::centipedeBump;
unsigned int Centipede::centipedecounter = 0;

Centipede::Centipede(Level *l, Vec2f p) :
	GameObject(l, p, CENTIPEDE_DEFAULT_MAX_SPEED, CENTIPEDE_DEFAULT_RADIUS),
	anim_timer(0),
	rolled_up(false),
	waypoints(1,p),
	wp_index(0)
{
	if (centipedecounter++ == 0) {
		centipedeImage = LoadSprite(PKGDATADIR"Artwork/creatures/rolly_polly/rolly_polly.png",
									CREATURE_ROTATIONS,
									level->GetTileWidth(),
									level->GetTileHeight());
		rolledUpImage = LoadSprite(PKGDATADIR"Artwork/creatures/rolly_polly/rolly_polly_rolledup.png",
								   CREATURE_ROTATIONS,
								   level->GetTileWidth(),
								   level->GetTileHeight());
		centipedeBump = LoadSound(PKGDATADIR"Sounds/CentipedePushed.ogg");
	}

	image = centipedeImage;
}

Centipede::~Centipede()
{
	if (--centipedecounter == 0) {
		SDL_FreeSurface(centipedeImage);
		SDL_FreeSurface(rolledUpImage);
		Mix_FreeChunk(centipedeBump);
	}
}

void Centipede::Run(void)
{
	if (disabled)
		return;

	if(level->IsItDay())
	{
		rolled_up = false;
		image = centipedeImage;
	}
	else
	{
		rolled_up = true;
		image = rolledUpImage;
		return;
	}

	if ((waypoints.at(wp_index) - position).Length() < max_speed)
		wp_index = (wp_index + 1)%waypoints.size();

	//Move
	anim_timer = (anim_timer + 1) % CENTIPEDE_ANIMATION_SPEED;
		
	if(anim_timer == 0)
		anim_index = (anim_index + 1) % 2;

	velocity = level->GetShortestPath(GetPosition(), waypoints.at(wp_index), max_speed);
	position += velocity;

}

void Centipede::Reset(void)
{
	this->GameObject::Reset();

	image = rolledUpImage;

	max_speed = CENTIPEDE_DEFAULT_MAX_SPEED;
	radius = CENTIPEDE_DEFAULT_RADIUS;

	rolled_up = true;
}

void Centipede::SetRolledUp(bool ru)
{
	rolled_up = ru;

	if(ru)
		image = rolledUpImage;
	else
		image = centipedeImage;
}

std::string Centipede::GetXMLRepresentation(const std::string& indent) const
{
	std::ostringstream xml;
	xml << indent << "<creature x=\"" << level->GetTilePos(position).x + 0.5f
		<< "\" y=\"" << level->GetTilePos(position).y + 0.5f
		<< "\" type=\"centipede\"";

	if (HasColor())
		xml << " color=\"" << GetColor().name() << "\"";
	
	std::string wptstring = LevelWriter::waypointsToXML(indent+"  ", position,
														waypoints.begin(),
														waypoints.end());
	if (wptstring.size() == 0)
		xml << "/>\n";
	else
		xml << ">\n" << wptstring << indent << "</creature>\n";

	return xml.str();
}

bool Centipede::IsPatroller(){
	return true;
}

void Centipede::DrawWaypoints(SDL_Surface *screen, ColorRGBA color)
{
	Vec2f first = waypoints.front();
	Vec2f prev;
	for (std::vector<Vec2f>::iterator i = waypoints.begin();
		 i != waypoints.end(); i++) {
		if (*i != first)
			aalineRGBA(screen, prev.x, prev.y, (*i).x, (*i).y,
					   color.r, color.g, color.b, color.a);

		prev = *i;
	}
	aalineRGBA(screen, prev.x, prev.y, first.x, first.y,
			   color.r, color.g, color.b, color.a);
}

void Centipede::ResetWaypoints(Vec2f pos){
	waypoints.clear();
	waypoints.push_back(pos);
}
// vim: set ts=4 sts=4 sw=4 noet:
