/// \file Centipede.hpp Header file for the Centipede class

/// Include guard for Centipede.hpp
#ifndef INCLUDED_CENTIPEDE_H
#define INCLUDED_CENTIPEDE_H 1

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif

#include "SoundManager.hpp"
#include "GameObject.hpp"
#include "Patroller.hpp"
#include "Saveable.hpp"

/**
 * \brief Class representing a centipede.
 */
class Centipede : public GameObject, public Patroller {
public:
	/** \brief Constructs a Centipede in Level l at position p.
	* \param l a pointer to the Level this Centipede exists in
	* \param p the Vec2f start_position of this Centipede
	*/
	Centipede(Level *l, Vec2f p);

	/** \brief Destructs a Centipede */
	~Centipede();

	/** \brief Runs one cycle of this Centipede.
	*/
	void Run(void);

	/** \brief Resets this Centipede to its original state in its Level.
	*/
	void Reset(void);

	//SET methods

	/** \brief Sets whether or not this Centipede is rolled up.
	 * \param ru whether or not this Centipede is rolled up
	 */
	void SetRolledUp(bool ru);

	/** \brief Adds a waypoint to this Centipede.
	 * 
	 * Adds a waypoint to this Centipede. While awake, the Centipede will
	 * attempt to visit all its waypoints in the order they were added.
	 * When it completes the circuit, it will begin again.
	 * \param wp the Vec2f waypoint to add
	 * \param duringday Should the waypoint be visited during the day?
	 * \param duringnight Should the waypoint be visited at night?
	 */
	inline void AddWaypoint(const Vec2f wp,
							bool duringday = true, bool duringnight = true) 
		{ waypoints.push_back(wp); }

	//GET methods
	std::string GetXMLRepresentation(const std::string& indent) const;

	/** \brief Gets whether or not this Centipede is rolled up.
	 * \return whether or not this Centipede is rolled up
	 */
	inline bool GetRolledUp(void) const { return rolled_up; }

	/** \brief Gets the GameObjectType of this Centipede.
	 * \return the GameObjectType of this Centipede
	 */
	inline GameObject::GameObjectType GetType(void) const { return GAME_OBJECT_CENTIPEDE; }

	void PlayCollisionSound() const;

	/** \brief draws the creature's waypoints
	 *
	 * Draws the creature's waypoints as dots and path between them as lines
	 */
	void DrawWaypoints(SDL_Surface *screen, ColorRGBA color);

	/** \brief resets creature's waypoints
	 *
	 * Clears all the creature's waypoints except for the origin
	 */
	void ResetWaypoints(Vec2f pos);

	bool IsPatroller();

private:
	static SDL_Surface *centipedeImage;		///< The default image of a Centipede
	static SDL_Surface *rolledUpImage;		///< The rolled-up image of a Centipede
	static Mix_Chunk *centipedeBump;		///< The sound of the Centipede being pushed
	static unsigned int centipedecounter;	///< Number of Centipedes

	unsigned int anim_timer;		///< The timer until this Centipede changes to its next frame.

	bool rolled_up; ///< Whether or not this Centipede is rolled up.
	std::vector<Vec2f> waypoints;	///< The waypoints of this Centipede
	int wp_index;					///< The current index of this Centipede's waypoints
	Vec2f pt1, pt2, pt3, pt4;
};

inline void Centipede::PlayCollisionSound() const
{
	SoundManager::PlayChunk(centipedeBump);
}

#endif /* INCLUDED_CENTIPEDE_H */
// vim: set ts=4 sts=4 sw=4 noet:
