/// \file Charlie.hpp Header file for the Charlie class

/// Include guard for Charlie.hpp
#ifndef INCLUDED_CHARLIE_H
#define INCLUDED_CHARLIE_H 1

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif

#include "GameObject.hpp"

/**
 * \brief Class representing Charlie the Chameleon.
 */
class Charlie : public GameObject {
public:
	/** \brief Constructs a Charlie in Level l at position p.
	* \param l a pointer to the Level this Charlie exists in
	* \param p Vec2f start_position of this Charlie
	* \param c Initial color of this Charlie
	*/
	Charlie(Level *l, Vec2f p, ColorRGBA c = ColorRGBA(0,255,0));

	/** \brief Destructs this Charlie */
	~Charlie();

	/** \brief Runs one cycle of this Charlie.
	*/
	void Run(void);

	/** \brief Report a keyboard event (key press or release)
	 * \param event The event to process
	 */
	void KeyboardEvent(SDL_KeyboardEvent *event);

	/** \brief Report a mouse motion event
	 * \param event The event to process
	 */
	void MouseMotionEvent(SDL_MouseMotionEvent *event);

	/** \brief Report a mouse button event (button press or release)
	 * \param event The event to process
	 */
	void MouseButtonEvent(SDL_MouseButtonEvent *event);

	/** \brief Resets this Charlie to its original state in its Level.
	*/
	void Reset(void);

	// SET methods

	/** \brief Sets the Color of the Flower this Charlie is standing on.
     *
	 * Sets the Color of the Flower this Charlie is standing on. By convention,
	 * if Charlie is not standing on a Flower this should be COLOR_GREEN.
	 * \param fc the Color to set
	 */
	inline void SetFlowerColor(ColorRGBA fc) { flowerColor = fc; }

	// GET methods
	std::string GetXMLRepresentation(const std::string& indent) const;

	/** \brief Gets the Color of the Flower this Charlie is standing on, or COLOR_GREEN.
	 *
	 * Gets the Color of the Flower this Charlie is standing on. By convention,
	 * if Charlie is not standing on a Flower this should be COLOR_GREEN.
	 * \return the Color of the Flower this Charlie is standing on, or COLOR_GREEN
	 */
	inline ColorRGBA GetFlowerColor(void) const { return flowerColor; }

	/** \brief Gets whether or not Charlie is invisible.
	 *
	 * Gets whether or not this Charlie is invisible. A Charlie is defined to be invisible
	 * if his color matches the color of the Tile he is standing on.
	 * \return whether or not Charlie is invisible
	 */
	bool GetInvisible(void);

	/** \brief Gets the GameObjectType of this Charlie.
	 * \return the GameObjectType of this Charlie
	*/
	inline GameObjectType GetType(void) const { return GAME_OBJECT_CHARLIE; }
private:
	/** \brief Returns the Image this Charlie would be if it was Color c
	*/
	SDL_Surface* UpdateImage(ColorRGBA c);

	static SDL_Surface *charlie_black;	///< Image of the black Charlie
	static SDL_Surface *charlie_blue;	///< Image of the blue Charlie
	static SDL_Surface *charlie_brown;	///< Image of the brown Charlie
	static SDL_Surface *charlie_green;	///< Image of the green Charlie
	static SDL_Surface *charlie_orange;	///< Image of the orange Charlie
	static SDL_Surface *charlie_red;	///< Image of the red Charlie
	static SDL_Surface *charlie_violet;	///< Image of the violet Charlie
	static SDL_Surface *charlie_white;	///< Image of the white Charlie
	static SDL_Surface *charlie_yellow;	///< Image of the yellow Charlie
	static unsigned int charliecounter;	///< Number of Charlie's in use

	ColorRGBA prevColor;
	ColorRGBA flowerColor; ///< The Color of the Flower Charlie is standing on, or COLOR_GREEN
	int anim_cnt;
	int anim_timer;
	Vec2f uservel;		///< User-requested velocity (from key presses)
};

#endif /* INCLUDED_CHARLIE_H */
