/// \file ColorRGBA.hpp Header file for the ColorRGBA class

/// Include guard for ColorRGBA.hpp
#ifndef INCLUDED_COLORRGBA_H
#define INCLUDED_COLORRGBA_H 1

#include <sstream>
#include <cassert>

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif

/**
 * \brief Class representing a color (with Red, Green, Blue, Alpha components).
 */
struct ColorRGBA {
	/** \brief Constructs a ColorRGBA of the given values
	* \param r Red component
	* \param g Green component
	* \param b Blue component
	* \param a Alpha component (255 = fully opaque)
	*/
	ColorRGBA(Uint8 r = 0, Uint8 g = 0, Uint8 b = 0, Uint8 a = 255);

	/** \brief Constructs a ColorRGBA from an SDL_Color
	* \param color SDL_Color to duplicate (fully opaque)
	*/
	ColorRGBA(const SDL_Color *color);

	/** \brief Destructs this ColorRGBA */
	~ColorRGBA() {};

	/** \brief String representation of this ColorRGBA
	 * \return String representing this color as #RRGGBB
	 */
	const char *c_str() const;

	/** \brief String representing the name of this color
	 * \return Human-readable name if the color matches one of the
	 * predefined colors, c_str() otherwise.
	 */
	const char *name() const;

	union {
		Uint8 r;
		Uint8 red;
	};

	union {
		Uint8 g;
		Uint8 green;
	};

	union {
		Uint8 b;
		Uint8 blue;
	};

	union {
		Uint8 a;
		Uint8 alpha;
	};
};

/** \name CommonColors
 * Commonly used colors, defined for convenience
 */
//@{
const ColorRGBA COLOR_BROWN(150,100,50);
const ColorRGBA COLOR_RED(255,0,0);
const ColorRGBA COLOR_ORANGE(255,128,0);
const ColorRGBA COLOR_YELLOW(255,255,0);
const ColorRGBA COLOR_GREEN(0,255,0);
const ColorRGBA COLOR_BLUE(0,0,255);
const ColorRGBA COLOR_VIOLET(128,0,128);
const ColorRGBA COLOR_BLACK(0,0,0);
const ColorRGBA COLOR_WHITE(255,255,255);
//@}

inline ColorRGBA::ColorRGBA(Uint8 rc, Uint8 gc, Uint8 bc, Uint8 ac) :
	r(rc),
	g(gc),
	b(bc),
	a(ac)
{
}

inline ColorRGBA::ColorRGBA(const SDL_Color *color) :
	r(color->r),
	g(color->g),
	b(color->b),
	a(255)
{
}

inline bool operator==(const ColorRGBA& color1, const ColorRGBA& color2)
{
	return color1.r == color2.r &&
	       color1.g == color2.g &&
	       color1.b == color2.b &&
	       color1.a == color2.a;
}

inline bool operator!=(const ColorRGBA& color1, const ColorRGBA& color2)
{
	return !(color1 == color2);
}

inline const char *ColorRGBA::c_str() const
{
	static char stringrep[8];
	std::ostringstream c("#");
	c.setf(std::ios_base::hex);
	c.unsetf(std::ios_base::showbase);
	c.width(2);
	c.fill('0');
	c << red << green << blue;
	assert(c.str().length() <= 7);
	strncpy(stringrep, c.str().c_str(), 7);
	stringrep[7] = '\0';
	return stringrep;
}

inline const char *ColorRGBA::name() const
{
	if (*this == COLOR_BROWN)
		return "brown";
	else if (*this == COLOR_RED)
		return "red";
	else if (*this == COLOR_ORANGE)
		return "orange";
	else if (*this == COLOR_YELLOW)
		return "yellow";
	else if (*this == COLOR_GREEN)
		return "green";
	else if (*this == COLOR_BLUE)
		return "blue";
	else if (*this == COLOR_VIOLET)
		return "violet";
	else if (*this == COLOR_BLACK)
		return "black";
	else if (*this == COLOR_WHITE)
		return "white";
	else 
		return c_str();
}

#endif /* INCLUDED_COLORRGBA_H */
