/// \file Configuration.hpp Header file for the Configuration class

/// Include guard for Configuration.hpp
#ifndef INCLUDED_CONFIGURATION_H
#define INCLUDED_CONFIGURATION_H 1

/**
 * \brief Class representing the configuration information for Geuka
 */
class Configuration {
public:
	/** \brief Destructs this Configuration instance and saves the current
	 * configuration to disk.
	 */
	virtual ~Configuration() {}

	/** \brief Save configuration information to disk */
	virtual void Save() const = 0;

	/** \name GetConfiguration
	 * Functions to retrieve configuration options.
	 */
	//@{
	/** \brief Get the number of levels completed in the story sequence
	 * \return Number of levels completed
	 */
	unsigned int GetLevelsCompleted() const { return levelscompleted; }

	/** \brief Get the preference for forcing video mode
	 * \return true if the video mode should be forced, false otherwise
	 */
	bool GetForceVideoMode() const { return forcevideo; }

	/** \brief Get the preferred game window width
	 * \return Preferred width of the game window
	 */
	unsigned int GetPreferredWindowWidth() const { return windowwidth; }

	/** \brief Get the preferred game window height
	 * \return Preferred height of the game window
	 */
	unsigned int GetPreferredWindowHeight() const { return windowheight; }

	/** \brief Get the preferred fullscreen status of the game
	 * \return Does the user prefer fullscreen?
	 */
	bool GetFullscreen() const { return fullscreen; }

	/** \brief Get preferred color depth
	 * \return Preferred color depth.
	 */
	int GetColorDepth() const { return colordepth; }

	/** \brief Get preferred sound state
	 * \return Should sound be enabled?
	 */
	bool GetSoundEnabled() const { return soundon; }
	//@}

	/** \name SetConfiguration
	 * Functions to set configuration options.
	 */
	//@{
	/** \brief Set the number of levels completed in the story sequence
	 * \param n Number of levels completed
	 */
	void SetLevelsCompleted(unsigned int n) { levelscompleted = n; }

	/** \brief Set the preferred game window width
	 * \param w Preferred window width
	 */
	void SetPreferredWindowWidth(unsigned int w) { windowwidth = w; }

	/** \brief Set the preferred game window height
	 * \param w Preferred window height
	 */
	void SetPreferredWindowHeight(unsigned int h) { windowheight = h; }

	/** \brief Set the preferred fullscreen status of the game
	 * \param f Does the user prefer fullscreen?
	 */
	void SetFullscreen(bool f) { fullscreen = f; }

	/** \brief Set the preferred color depth of the game
	 * \param d Preferred color depth
	 */
	void SetColorDepth(int d) { colordepth = d; }

	/** \brief Set the preferred sound state
	 * \param s Should sound be enabled?
	 */
	void SetSoundEnabled(bool s) { soundon = s; }
	//@}

protected:
	Configuration() :
		levelscompleted(0),
		forcevideo(DEFAULT_FORCE_VIDEO),
		fullscreen(DEFAULT_FULLSCREEN),
		windowwidth(DEFAULT_WINDOW_WIDTH),
		windowheight(DEFAULT_WINDOW_HEIGHT),
		colordepth(DEFAULT_COLOR_DEPTH),
		soundon(DEFAULT_SOUND_ENABLED)
	{
	}

protected:
	static const bool DEFAULT_FORCE_VIDEO = false;
	static const bool DEFAULT_FULLSCREEN = true;
	static const unsigned int DEFAULT_WINDOW_WIDTH = 1024;
	static const unsigned int DEFAULT_WINDOW_HEIGHT = 768;
	static const int DEFAULT_COLOR_DEPTH = 32;
	static const bool DEFAULT_SOUND_ENABLED = true;

	unsigned int levelscompleted;	///< Highest level completed
	bool forcevideo;				///< Should the video mode be forced?
	bool fullscreen;				///< Run the game in fullscreen mode
	unsigned int windowwidth;		///< Width of the game window
	unsigned int windowheight;		///< Height of the game window
	int colordepth;					///< Color depth of the game
	bool soundon;					///< Is sound turned on?
};

#endif /* INCLUDED_CONFIGURATION_H */
// vim: set ts=4 sts=4 sw=4 noet:
