/// \file DialogBoxesWindows.cpp Source file for the DialogBoxesWindows class

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <tchar.h>
#include <windows.h>

#include "DialogBoxesWindows.hpp"

DialogBoxesWindows::DialogBoxesWindows()
{
	GetWinHandle();
}

void DialogBoxesWindows::WarningMessage(const char *format, ...)
{
	va_list ap;

	va_start(ap, format);

	bool mustfreemessage = false;
	char *message;
#ifdef HAVE_VASPRINTF
	mustfreemessage = true;
	vasprintf(&message, format, ap);
#elif defined(_MSC_VER)
	mustfreemessage = true;
	message = reinterpret_cast<char *>(malloc(_vscprintf(format, ap) + 1));
	if (message == NULL)
		return;
	vsprintf(message, format, ap);
#else
# error Please define a variadic formatting function
#endif
	va_end(ap);

	MessageBox(windowhandle, message, _T("Error"), MB_OK|MB_ICONWARNING);

	if (mustfreemessage)
		free(message);
}

void DialogBoxesWindows::ErrorMessage(const char *format, ...)
{
	va_list ap;

	va_start(ap, format);

	bool mustfreemessage = false;
	char *message;
#ifdef HAVE_VASPRINTF
# define MUST_FREE_MESSAGE 1
	vasprintf(message, format, ap);
#elif defined(_MSC_VER)
	mustfreemessage = true;
	message = reinterpret_cast<char *>(malloc(_vscprintf(format, ap) + 1));
	if (message == NULL)
		return;
	vsprintf(message, format, ap);
#else
# error Please define a variadic formatting function
#endif

	va_end(ap);

	MessageBox(windowhandle, message, _T("Error"), MB_OK|MB_ICONERROR);

	if (mustfreemessage)
		free(message);
}

bool DialogBoxesWindows::PromptSaveLevel(std::string& filename,
										 const char *dirname)
{
	char fname[MAX_PATH];
	strncpy(fname, filename.c_str(), MAX_PATH-1);

	OPENFILENAME ofn;
#if defined(WINVER) && defined(_WIN32_WINNT) && _WIN32_WINNT >= 0x0500
	ofn.lStructSize = OPENFILENAME_SIZE_VERSION_400;
#else
	ofn.lStructSize = sizeof(OPENFILENAME);
#endif
	ofn.hwndOwner = windowhandle;
	ofn.lpstrFilter = "XML Level File (*.xml)\0*.xml\0All Files (*.*)\0*.*\0\0";
	ofn.lpstrFile= fname;
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxFile = sizeof(fname)/ sizeof(fname[0]);
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = dirname;
	ofn.lpstrTitle = "Save Level File";
	ofn.Flags = OFN_OVERWRITEPROMPT | OFN_NOCHANGEDIR;
	ofn.lpstrDefExt = "xml";

	if (GetSaveFileName(&ofn)) {
		filename = ofn.lpstrFile;
		return true;
	} else {
		return false;
	}
}

bool DialogBoxesWindows::PromptOpenLevel(std::string& filename,
										 const char *dirname)
{
	char fname[MAX_PATH];
	strncpy(fname, filename.c_str(), MAX_PATH-1);

	OPENFILENAME ofn;
#if defined(WINVER) && defined(_WIN32_WINNT) && _WIN32_WINNT >= 0x0500
	ofn.lStructSize = OPENFILENAME_SIZE_VERSION_400;
#else
	ofn.lStructSize = sizeof(OPENFILENAME);
#endif
	ofn.hwndOwner = windowhandle;
	ofn.lpstrFilter = "XML Level File (*.xml)\0*.xml\0All Files (*.*)\0*.*\0\0";
	ofn.lpstrFile= fname;
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxFile = sizeof(fname)/ sizeof(fname[0]);
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = dirname;
	ofn.lpstrTitle = "Load Level File";
	ofn.Flags = OFN_OVERWRITEPROMPT | OFN_NOCHANGEDIR;
	ofn.lpstrDefExt = "xml";

	if (GetOpenFileName(&ofn)) {
		filename = ofn.lpstrFile;
		return true;
	} else {
		return false;
	}
}

/// \brief Test the window to see if it is the one we are looking for (ours)
static BOOL CALLBACK TestWindow(HWND hwnd, LPARAM lparam)
{
	if (GetWindowThreadProcessId(hwnd, NULL) == GetCurrentProcessId()) {
		reinterpret_cast<DialogBoxesWindows *>(lparam)->windowhandle = hwnd;
		return FALSE;
	}
	return TRUE;
}

void DialogBoxesWindows::GetWinHandle()
{
	if (EnumWindows(TestWindow, reinterpret_cast<LPARAM>(this)) == 0) {
		DWORD error = GetLastError();
		LPVOID errormsg;

		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
					  FORMAT_MESSAGE_FROM_SYSTEM,
					  NULL,
					  error,
					  0,
					  reinterpret_cast<LPTSTR>(&errormsg),
					  0,
					  NULL);

		ErrorMessage("Unable to enumerate windows: %s\n", errormsg);

		LocalFree(errormsg);
	}
}

// vim: set ts=4 sts=4 sw=4 noet:
