/// \file Door.cpp Source file for the Door class

#include <sstream>

#include "Door.hpp"

SDL_Surface *Door::doorImage;
Mix_Chunk *Door::doorOpening;
Mix_Chunk *Door::doorClosing;
unsigned int Door::doorcount = 0;

Door::Door(Level *l, Vec2f p, int group) :
	GameObject(l, p, DOOR_DEFAULT_MAX_SPEED, DOOR_DEFAULT_RADIUS),
	switchgroup(group),
	isclosed(true),
	wanttoclose(false),
	mytile(level->GetTile(p))
{
	if (doorcount++ == 0) {
		doorImage = LoadSprite(PKGDATADIR"Artwork/objects/leafdoor/leafdoor.png",
							   1,
							   level->GetTileWidth(),
							   level->GetTileHeight());
		doorOpening = LoadSound(PKGDATADIR"Sounds/DoorOpening.ogg");
		doorClosing = LoadSound(PKGDATADIR"Sounds/DoorClosing.ogg");
	}

	mytile->passable = false;
	image = doorImage;

	UpdateImage();
}

Door::~Door()
{
	if (--doorcount == 0) {
		SDL_FreeSurface(doorImage);
		Mix_FreeChunk(doorOpening);
		Mix_FreeChunk(doorClosing);
	}
}

void Door::Run()
{
	if (wanttoclose) {
		// Check if there is an object on this Door, if so keep waiting
		for (unsigned int i=0; i<mytile->objects.size(); i++) {
			GameObject *object = mytile->objects[i];
			if (object != this && IsCollidingWith(object))
				return;
		}
		isclosed = true;
		wanttoclose = false;
		mytile->passable = false;
	}

	UpdateImage();
}

void Door::Reset()
{
	this->GameObject::Reset();

	max_speed = DOOR_DEFAULT_MAX_SPEED;
	radius = DOOR_DEFAULT_RADIUS;

	isclosed = true;
	wanttoclose = false;
	mytile->passable = false;

	UpdateImage();
}

void Door::SetState(bool newstate)
{
	if (isclosed == newstate) {
		wanttoclose = false;
		return;
	}

	if (newstate) {
		// Check if there is an object on this Door, if so don't close
		for (unsigned int i=0; i<mytile->objects.size(); i++) {
			GameObject *object = mytile->objects[i];
			if (object != this && IsCollidingWith(object)) {
				wanttoclose = true;
				return;
			}
		}
		isclosed = true;
		mytile->passable = false;
		SoundManager::PlayChunk(doorClosing);
		UpdateImage();
	} else {
		isclosed = false;
		mytile->passable = true;
		SoundManager::PlayChunk(doorOpening);
		UpdateImage();
	}
}

void Door::UpdateImage(void)
{
	static unsigned int timercount = 0;
	if ((anim_index == 0 && !isclosed) ||
		(anim_index == 2 && isclosed)) {
		anim_index = 1;
	} else if (anim_index == 1 && timercount++ > 10) {
		if (isclosed)
			anim_index = 0;
		else
			anim_index = 2;
		timercount = 0;
	}
}

std::string Door::GetXMLRepresentation(const std::string& indent) const
{
	std::ostringstream xml;
	xml << indent << "<object x=\"" << level->GetTilePos(position).x + 0.5f
		<< "\" y=\"" << level->GetTilePos(position).y + 0.5f
		<< "\" type=\"leafdoor\"";

	if (HasColor())
		xml << " color=\"" << GetColor().name() << "\"";

	if (switchgroup > 0)
		xml << " group=\"" << switchgroup << "\"";

	xml << "/>\n";

	return xml.str();
}
// vim: set ts=4 sts=4 sw=4 noet:
