/// \file Door.hpp Header file for the Door class

/// Include guard for Door.hpp
#ifndef INCLUDED_DOOR_H
#define INCLUDED_DOOR_H 1

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif

#include "GameObject.hpp"
#include "Switchable.hpp"

/**
 * \brief Class representing a Door.
 */
class Door : public GameObject, public Switchable {
public:
	/** \brief Constructs a Door in Level l at position p.
	 * Constructs a Door in Level l at position p. The door needs to have a
	 * Switch or other control mechanism associated with it to actually toggle its
	 * open/closed state.
	 * \param l a pointer to the Level this Door exists in
	 * \param p the Vec2f position of this Door
	 * \param group number of the switch group for this Door
	 */
	Door(Level *l, Vec2f p, int group = 0);

	/** \brief Destructs this Door. */
	~Door(void);

	/** \brief Runs a cycle of this Door. */
	void Run(void);

	/** \brief Resets this Door to its original state in its Level. */
	void Reset(void);

	/** \brief Sets the state of this Door.
	 * Sets the state of this Door.
	 * The state of a Door is defined to be true == closed, false == open.
	 * \param s the state of this Door
	 */
	void SetState(bool s);

	std::string GetXMLRepresentation(const std::string& indent) const;

	/** \brief Gets the state of this Door.
	 * Gets the state of this Door.
	 * The state of a Door is defined to be true == closed, false == open.
	 * \return the state of this Door
	 */
	inline bool GetState() const { return isclosed; }

	inline int GetSwitchGroup() const { return switchgroup; }
	inline void SetSwitchGroup(const int group)
		{ if (group > 0) switchgroup = group;
		  else switchgroup = 0;}

	/** \brief Gets the GameObjectType of this Door. */
	inline GameObjectType GetType(void) const { return GAME_OBJECT_DOOR; }
private:
	void UpdateImage(void);			///< Updates the image of this Door.

	static SDL_Surface *doorImage;	///< The open image of a Door
	static Mix_Chunk *doorOpening;		///< Sound of the Door opening
	static Mix_Chunk *doorClosing;		///< Sound of the Door closing
	static unsigned int doorcount;	///< Number of Doors created

	int switchgroup;	///< Number of the switch group for this Door
	bool isclosed;		///< Is this Door closed?
	bool wanttoclose;	///< Does the Door want to close, but can't?
	Tile *mytile;		///< The Tile on which this Door is sitting
};

#endif /* INCLUDED_DOOR_H */
