/// \file Flower.hpp Header file for the Flower class

/// Include guard for Flower.hpp
#ifndef INCLUDED_FLOWER_H
#define INCLUDED_FLOWER_H 1

#include "GameObject.hpp"
#include "Switchable.hpp"

/**
 * \brief Class representing a flower.
 */
class Flower : public GameObject, public Switchable {
public:
	/** \brief Constructs a Flower in Level l at position p with Color c.
	* \param l a pointer to the Level this Flower exists in
	* \param p the Vec2f start_position of this Flower
	* \param c the Color of this Flower
	* \param group the number of the switch group for this Flower
	* \param dayopen Is this Flower open during the day?
	* \param nightopen Is this Flower open at night?
	*/
	Flower(Level *l, Vec2f p, ColorRGBA c,
		   bool dayopen = true, bool nightopen = false, int group = 0);

	/** \brief Destructs a Flower */
	~Flower();

	/** \brief Runs one cycle of this Flower.
	*/
	void Run(void);

	/** \brief Resets this Flower to its original state in its Level.
	*/
	void Reset(void);

	//SET methods

	/** \brief Sets the state of this flower.
	 *
	 * The state of a Flower is defined to be true == closed, false == open.
	 * \param s the state of this flower
	 */
	void SetState(bool s);

	//GET methods
	std::string GetXMLRepresentation(const std::string& indent) const;

	/** \brief Is this Flower closed?
	 * \return true if this Flower is closed, false otherwise
	 */
	inline bool IsClosed() const { return closed; }

	/** \brief Gets the state of this Flower.
	 *
	 * The state of a Flower is defined to be true if the flower is being
	 * forced open by a toggle switch and false otherwise
	 * \return the state of this flower
	 */
	inline bool GetState() const { return switchedopen; }

	inline int GetSwitchGroup() const { return switchgroup; }
	inline void SetSwitchGroup(const int group)
		{ if (group > 0) switchgroup = group;
		  else switchgroup = 0;}

	/** \brief Gets the GameObjectType of this Flower.
	 * \return the GameObjectType of this Flower.
	 */
	inline GameObjectType GetType() const { return GAME_OBJECT_FLOWER; }
private:
	/** \brief Can this Flower close?
	 * \return true if this flower is unoccupied
	 */
	bool CanClose() const;

	/** \brief Update the Image of this Flower.
	*/
	void UpdateImage();

	static SDL_Surface *flower_black;
	static SDL_Surface *flower_blue;
	static SDL_Surface *flower_brown;
	static SDL_Surface *flower_green;
	static SDL_Surface *flower_orange;
	static SDL_Surface *flower_red;
	static SDL_Surface *flower_violet;
	static SDL_Surface *flower_white;
	static SDL_Surface *flower_yellow;

	static unsigned int flowercount;	///< Number of Flowers created

	int switchgroup;			///< Number of the switch group for this Flower
	bool closed;					///< Is this Flower closed?
	bool dayopen;					///< Is this Flower open during the day?
	bool nightopen;					///< Is this Flower open at night?
	bool switchedopen;				///< Is this Flower open due to a switch?
	bool wanttoclose;				///< Does this Flower want to close?
	int closing_animation_timer;	///< The animation timer for opening and closing this flower.
};

#endif /* INCLUDED_FLOWER_H */
// vim: set ts=4 sts=4 sw=4 noet:
