/// \file GameObject.cpp Source file for the GameObject class

#include "GameObject.hpp"

GameObject::GameObject(Level *l, Vec2f p, float ms, float r) :
	imagesize(l->GetTileDimension()),
	anim_index(0),
	max_speed(ms),
	radius(r),
	level(l),
	position(p),
	start_position(p),
	hascolor(false),
	color(0,0,0,0),
	tile_position(Vec2i(p.x / l->GetTileDimension().x,
						p.y / l->GetTileDimension().y)),
	velocity(Vec2f(0.0f,0.0f)),
	prevang(0.0f),
	orientation(-1.0f),
	disabled(false)
{
	/* In a child constructor:
	 *	image = LoadImageSDL(HARDCODED_IMAGE_LOCATION);
	 *
	 *  other_members = APPROPRIATE_VALUES;
	 */
}

GameObject::GameObject(Level *l, Vec2f p, ColorRGBA c, float ms, float r) :
	imagesize(l->GetTileDimension()),
	anim_index(0),
	max_speed(ms),
	radius(r),
	level(l),
	position(p),
	start_position(p),
	hascolor(true),
	color(c),
	drawcolor(c),
	tile_position(Vec2i(p.x / l->GetTileDimension().x, 
						p.y / l->GetTileDimension().y)),
	velocity(Vec2f(0.0f,0.0f)),
	prevang(0.0f),
	orientation(-1.0f),
	disabled(false)
{
	/* In a child constructor:
	 *	image = LoadImageSDL(HARDCODED_IMAGE_LOCATION);
	 *
	 *  other_members = APPROPRIATE_VALUES;
	 */
}

void GameObject::Draw(SDL_Surface *screen)
{
	if (disabled)
		return;

	float theta;
	if (velocity.x == 0.0f) {
		if (velocity.y == 0.0f) {
			if (orientation >= 0.0f)
				theta = orientation;
			else
				theta = prevang;
		} else {
			theta = (velocity.y > 0) ? M_PIf/2 : M_PIf*1.5f;
		}
	} else if (velocity.x < 0.0f) {
		theta = atanf(velocity.y/velocity.x) + M_PIf;
	} else {
		theta = atanf(velocity.y/velocity.x);
	}

	if (theta < 0.0f)
		theta += 2*M_PIf;

	// Save angle for later
	prevang = theta;

	unsigned int vindex = static_cast<unsigned int>(theta*4/M_PIf+0.5f)%8;

	SDL_Rect clip, offset;
	clip.x = anim_index*imagesize.x;
	clip.y = vindex*imagesize.y;
	clip.w = imagesize.x;
	clip.h = imagesize.y;
	Vec2i tiledim = level->GetTileDimension();
	Vec2i leveloffset = level->GetOffset();
	offset.x = position.x*tiledim.x+leveloffset.x-imagesize.x/2;
	offset.y = position.y*tiledim.y+leveloffset.y-imagesize.y/2;

	SDL_BlitSurface(image, &clip, screen, &offset);
}

void GameObject::Reset()
{
	position = start_position;
	tile_position = Vec2i(start_position.x / level->GetTileDimension().x,
						  start_position.y / level->GetTileDimension().y);
	velocity = Vec2f(0.0f, 0.0f);

	SetDisabled(false);
}

void GameObject::HandleCollision(GameObject * go1, GameObject * go2)
{
	// Make sure we have a real collision
	if (!go1->IsCollidingWith(go2))
		return;

	// This switch must be complete for all possible collision combinations. It should never reach default.

	switch (go1->GetType()) {
		case GAME_OBJECT_BAT:
			switch (go2->GetType()) {
				case GAME_OBJECT_BAT: break;
				case GAME_OBJECT_BEE: break;
				case GAME_OBJECT_BUTTERFLY: break;
				case GAME_OBJECT_CENTIPEDE: break;
				case GAME_OBJECT_CHARLIE: //Death of Charlie
					go1->PlayCollisionSound();
					go2->SetDisabled(true);
					go1->level->Failed();
					break;
				case GAME_OBJECT_DOOR: break;
				case GAME_OBJECT_FLOWER: break;
				case GAME_OBJECT_GOAL: break;
				case GAME_OBJECT_MILLIPEDE: break;
				case GAME_OBJECT_SNAKE: break;
				case GAME_OBJECT_SWITCH: break;
				case GAME_OBJECT_TIGERCUB: break;
				case GAME_OBJECT_SCORPION: break;
				default: assert(0);
			}
			break;
		case GAME_OBJECT_BEE:
			switch (go2->GetType()) {
				case GAME_OBJECT_BAT: break;
				case GAME_OBJECT_BEE: break;
				case GAME_OBJECT_BUTTERFLY: break;
				case GAME_OBJECT_CENTIPEDE: break;
				case GAME_OBJECT_CHARLIE: //Death of Charlie
					go1->PlayCollisionSound();
					go2->SetDisabled(true);
					go1->level->Failed();
					break;
				case GAME_OBJECT_DOOR: break;
				case GAME_OBJECT_FLOWER: break;
				case GAME_OBJECT_GOAL: break;
				case GAME_OBJECT_MILLIPEDE: break;
				case GAME_OBJECT_SNAKE: break;
				case GAME_OBJECT_SWITCH: break;
				case GAME_OBJECT_TIGERCUB: break;
				case GAME_OBJECT_SCORPION: break;
				default: assert(0);
			}
			break;
		case GAME_OBJECT_BUTTERFLY:
			switch (go2->GetType()) {
				case GAME_OBJECT_BAT: break;
				case GAME_OBJECT_BEE: break;
				case GAME_OBJECT_BUTTERFLY: break;
				case GAME_OBJECT_CENTIPEDE: break;
				case GAME_OBJECT_CHARLIE:
					go1->PlayCollisionSound();
					HandleCollision(dynamic_cast<Butterfly *>(go1), dynamic_cast<Charlie *>(go2));
					break;
				case GAME_OBJECT_DOOR: break;
				case GAME_OBJECT_FLOWER: break;
				case GAME_OBJECT_GOAL: break;
				case GAME_OBJECT_MILLIPEDE: break;
				case GAME_OBJECT_SNAKE: break;
				case GAME_OBJECT_SWITCH: break;
				case GAME_OBJECT_TIGERCUB: break;
				case GAME_OBJECT_SCORPION: break;
				default: assert(0);
			}
			break;
		case GAME_OBJECT_CENTIPEDE:
			switch (go2->GetType()) {
				case GAME_OBJECT_BAT: break;
				case GAME_OBJECT_BEE: break;
				case GAME_OBJECT_BUTTERFLY: break;
				case GAME_OBJECT_CENTIPEDE: HandleCollision(dynamic_cast<Centipede *>(go1), go2); break; //Collide if Centipede is rolled up.
				case GAME_OBJECT_CHARLIE:
					go1->PlayCollisionSound();
					HandleCollision(dynamic_cast<Charlie *>(go2), dynamic_cast<Centipede *>(go1)); break; // Push or Die
				case GAME_OBJECT_DOOR: HandleCollision(dynamic_cast<Door *>(go2), go1); break; // Collide if Door is closed
				case GAME_OBJECT_FLOWER: HandleCollision(dynamic_cast<Flower *>(go2), go1); break; //Collide if Flower is closed
				case GAME_OBJECT_GOAL: break;
				case GAME_OBJECT_MILLIPEDE: HandleCollision(dynamic_cast<Centipede *>(go1), go2); break;
				case GAME_OBJECT_SNAKE: HandleCollision(dynamic_cast<Centipede *>(go1), go2); break; //Collide if Centipede is rolled up.
				case GAME_OBJECT_SWITCH: HandleCollision(dynamic_cast<Switch *>(go2), go1); break; //Toggle Switch
				case GAME_OBJECT_TIGERCUB: HandleCollision(dynamic_cast<Centipede *>(go1), go2); break; //Collide if Centipede is rolled up.
				case GAME_OBJECT_SCORPION: HandleCollision(dynamic_cast<Centipede *>(go1), go2); break; //Collide if Centipede is rolled up.
				default: assert(0);
			}
			break;
		case GAME_OBJECT_CHARLIE:
			switch (go2->GetType()) {
				case GAME_OBJECT_BAT: // Death of Charlie
					go1->SetDisabled(true);
					go2->PlayCollisionSound();
					go1->level->Failed();
					break;
				case GAME_OBJECT_BEE: //Death of Charlie
					go1->SetDisabled(true);
					go2->PlayCollisionSound();
					go1->level->Failed();
					break;
				case GAME_OBJECT_BUTTERFLY:
					HandleCollision(dynamic_cast<Butterfly *>(go2), dynamic_cast<Charlie *>(go1));
					go2->PlayCollisionSound();
					break;
				case GAME_OBJECT_CENTIPEDE: // Push or Die
					HandleCollision(dynamic_cast<Charlie *>(go1), dynamic_cast<Centipede *>(go2));
					go2->PlayCollisionSound();
					break;
				case GAME_OBJECT_CHARLIE: //Charlie does not hit himself.
					assert(0);
					break;
				case GAME_OBJECT_DOOR: // Collide if Door is closed
					HandleCollision(dynamic_cast<Door *>(go2), go1);
					go2->PlayCollisionSound();
					break;
				case GAME_OBJECT_FLOWER: //Collide if Flower is closed else acquire Color
					HandleCollision(dynamic_cast<Charlie *>(go1), dynamic_cast<Flower *>(go2));
					go2->PlayCollisionSound();
					break;
				case GAME_OBJECT_GOAL: //Beats the level
					HandleCollision(dynamic_cast<Charlie *>(go1), dynamic_cast<Goal *>(go2));
					go2->PlayCollisionSound();
					break;
				case GAME_OBJECT_MILLIPEDE:
					HandleCollision(dynamic_cast<Charlie *>(go1), dynamic_cast<Millipede *>(go2));
					go2->PlayCollisionSound();
					break;
				case GAME_OBJECT_SNAKE: //Death of Charlie
					go1->SetDisabled(true);
					go2->PlayCollisionSound();
					go1->level->Failed();
					break;
				case GAME_OBJECT_SWITCH:
					go2->PlayCollisionSound();
					HandleCollision(dynamic_cast<Switch *>(go2), go1);
					break; //Toggle Switch
				case GAME_OBJECT_TIGERCUB:
					go1->SetDisabled(true);
					go2->PlayCollisionSound();
					go1->level->Failed();
					break; //Death of Charlie
				case GAME_OBJECT_SCORPION:
					go1->SetDisabled(true);
					go2->PlayCollisionSound();
					go1->level->Failed();
					break; //Death of Charlie
				default: assert(0);
			}
			break;
		case GAME_OBJECT_DOOR:
			switch (go2->GetType()) {
				case GAME_OBJECT_BAT: break;
				case GAME_OBJECT_BEE: break;
				case GAME_OBJECT_BUTTERFLY: break;
				case GAME_OBJECT_CENTIPEDE: HandleCollision(dynamic_cast<Door *>(go1), go2); break; // Collide if Door is Closed
				case GAME_OBJECT_CHARLIE: go1->PlayCollisionSound(); HandleCollision(dynamic_cast<Door *>(go1), go2); break; // Collide if Door is Closed
				case GAME_OBJECT_DOOR: assert(0); break; //Both are immobile
				case GAME_OBJECT_FLOWER: assert(0); break; //Both are immobile
				case GAME_OBJECT_GOAL: assert(0); break; //Both are immobile
				case GAME_OBJECT_MILLIPEDE: HandleCollision(dynamic_cast<Door *>(go1), go2); break;
				case GAME_OBJECT_SNAKE: HandleCollision(dynamic_cast<Door *>(go1), go2); break; // Collide if Door is Closed
				case GAME_OBJECT_SWITCH: assert(0); break; //Both are immobile
				case GAME_OBJECT_TIGERCUB: HandleCollision(dynamic_cast<Door *>(go1), go2); break; // Collide if Door is Closed
				case GAME_OBJECT_SCORPION: HandleCollision(dynamic_cast<Door *>(go1), go2); break; // Collide if Door is Closed
				default: assert(0);
			}
			break;
		case GAME_OBJECT_FLOWER:
			switch (go2->GetType()) {
				case GAME_OBJECT_BAT: break;
				case GAME_OBJECT_BEE: break; // Later we'll have to notify Bee
				case GAME_OBJECT_BUTTERFLY: break;
				case GAME_OBJECT_CENTIPEDE: HandleCollision(dynamic_cast<Flower *>(go1), go2); break; // Collide if Flower is closed
				case GAME_OBJECT_CHARLIE: go1->PlayCollisionSound(); HandleCollision(dynamic_cast<Charlie *>(go2), dynamic_cast<Flower *>(go1)); break; // Collide if Flower is closed else acquire Color
				case GAME_OBJECT_DOOR: assert(0); break; //Both are immobile
				case GAME_OBJECT_FLOWER: assert(0); break; //Flowers are immobile, they should not hit each other.
				case GAME_OBJECT_GOAL: assert(0); break; //Both are immobile
				case GAME_OBJECT_MILLIPEDE: HandleCollision(go1, dynamic_cast<Millipede *>(go2)); break;
				case GAME_OBJECT_SNAKE: HandleCollision(dynamic_cast<Flower *>(go1), go2); break; // Collide if Flower is closed
				case GAME_OBJECT_SWITCH: assert(0); break; //Both are immobile
				case GAME_OBJECT_TIGERCUB: HandleCollision(dynamic_cast<Flower *>(go1), go2); break; //Collide if Flower is closed
				case GAME_OBJECT_SCORPION: HandleCollision(dynamic_cast<Flower *>(go1), go2); break; // Collide if Door is Closed
				default: assert(0);
			}
			break;
		case GAME_OBJECT_GOAL:
			switch (go2->GetType()) {
				case GAME_OBJECT_BAT: break;
				case GAME_OBJECT_BEE: break;
				case GAME_OBJECT_BUTTERFLY: break;
				case GAME_OBJECT_CENTIPEDE: break;
				case GAME_OBJECT_CHARLIE: go1->PlayCollisionSound(); HandleCollision(dynamic_cast<Charlie *>(go2), dynamic_cast<Goal *>(go1)); break; // Beat the level
				case GAME_OBJECT_DOOR: assert(0); break; //Both are immobile
				case GAME_OBJECT_FLOWER: assert(0); break; //Both are immobile
				case GAME_OBJECT_GOAL: assert(0); break; //Both are immobile
				case GAME_OBJECT_MILLIPEDE: break;
				case GAME_OBJECT_SNAKE: break;
				case GAME_OBJECT_SWITCH: assert(0); break; //Both are immobile
				case GAME_OBJECT_TIGERCUB: break;
				case GAME_OBJECT_SCORPION: break;
				default: assert(0);
			}
			break;
		case GAME_OBJECT_MILLIPEDE:
			switch (go2->GetType()) {
				case GAME_OBJECT_BAT: break;
				case GAME_OBJECT_BEE: break;
				case GAME_OBJECT_BUTTERFLY: break;
				case GAME_OBJECT_CENTIPEDE: HandleCollision(go2, dynamic_cast<Millipede *>(go1)); break; //Collide if Millipede is rolled up.
				case GAME_OBJECT_CHARLIE: go1->PlayCollisionSound(); HandleCollision(dynamic_cast<Charlie *>(go2), dynamic_cast<Millipede *>(go1)); break; // Push or Die
				case GAME_OBJECT_DOOR: HandleCollision(dynamic_cast<Door *>(go2), go1); break; // Collide if Door is closed
				case GAME_OBJECT_FLOWER: HandleCollision(dynamic_cast<Flower *>(go2), go1); break; //Collide if Flower is closed
				case GAME_OBJECT_GOAL: break;
				case GAME_OBJECT_MILLIPEDE: HandleCollision(go2, dynamic_cast<Millipede *>(go1)); break;
				case GAME_OBJECT_SNAKE: HandleCollision(go2, dynamic_cast<Millipede *>(go1)); break; //Collide if Millipede is rolled up.
				case GAME_OBJECT_SWITCH: HandleCollision(dynamic_cast<Switch *>(go2), go1); break; //Toggle Switch
				case GAME_OBJECT_TIGERCUB: HandleCollision(go2, dynamic_cast<Millipede *>(go1)); break; //Collide if Millipede is rolled up.
				case GAME_OBJECT_SCORPION: HandleCollision(go2, dynamic_cast<Millipede *>(go1)); break; //Collide if Millipede is rolled up.
				default: assert(0);
			}
			break;
		case GAME_OBJECT_SNAKE:
			switch (go2->GetType()) {
				case GAME_OBJECT_BAT: break;
				case GAME_OBJECT_BEE: break;
				case GAME_OBJECT_BUTTERFLY: break;
				case GAME_OBJECT_CENTIPEDE: HandleCollision(dynamic_cast<Centipede *>(go2), go1); break; //Collide if Centipede is rolled up.
				case GAME_OBJECT_CHARLIE: go1->PlayCollisionSound(); go2->SetDisabled(true); go1->level->Failed(); break; //Death of Charlie
				case GAME_OBJECT_DOOR: HandleCollision(dynamic_cast<Door *>(go2), go1); break; // Collide if Door is closed
				case GAME_OBJECT_FLOWER: HandleCollision(dynamic_cast<Flower *>(go2), go1); break;//Collide if Flower is closed
				case GAME_OBJECT_GOAL: break;
				case GAME_OBJECT_MILLIPEDE: HandleCollision(go1, dynamic_cast<Millipede *>(go2)); break;
				case GAME_OBJECT_SNAKE: break;
				case GAME_OBJECT_SWITCH: HandleCollision(dynamic_cast<Switch *>(go2), go1); break; //Toggle Switch
				case GAME_OBJECT_TIGERCUB: break;
				case GAME_OBJECT_SCORPION: break;
				default: assert(0);
			}
			break;
		case GAME_OBJECT_SWITCH:
			switch (go2->GetType()) {
				case GAME_OBJECT_BAT: break;
				case GAME_OBJECT_BEE: break;
				case GAME_OBJECT_BUTTERFLY: break;
				case GAME_OBJECT_CENTIPEDE: HandleCollision(dynamic_cast<Switch *>(go1), go2); break; //Toggle Switch
				case GAME_OBJECT_CHARLIE: go1->PlayCollisionSound(); HandleCollision(dynamic_cast<Switch *>(go1), go2); break; //Toggle Switch
				case GAME_OBJECT_DOOR: assert(0); break; //Both are immobile
				case GAME_OBJECT_FLOWER: assert(0); break; //Both are immobile
				case GAME_OBJECT_GOAL: assert(0); break; //Both are immobile
				case GAME_OBJECT_MILLIPEDE: HandleCollision(dynamic_cast<Switch *>(go1), go2); break; 
				case GAME_OBJECT_SNAKE: HandleCollision(dynamic_cast<Switch *>(go1), go2); break; //Toggle Switch
				case GAME_OBJECT_SWITCH: assert(0); break; //Both are immobile
				case GAME_OBJECT_TIGERCUB: HandleCollision(dynamic_cast<Switch *>(go1), go2); break; //Toggle Switch
				case GAME_OBJECT_SCORPION: HandleCollision(dynamic_cast<Switch *>(go1), go2); break; //Toggle Switch
				default: assert(0);
			}
			break;
		case GAME_OBJECT_TIGERCUB:
			switch (go2->GetType()) {
				case GAME_OBJECT_BAT: break;
				case GAME_OBJECT_BEE: break;
				case GAME_OBJECT_BUTTERFLY: break;
				case GAME_OBJECT_CENTIPEDE: HandleCollision(dynamic_cast<Centipede *>(go2), go1); break; //Collide if Centipede is rolled up.
				case GAME_OBJECT_CHARLIE: go1->PlayCollisionSound(); go2->SetDisabled(true); go1->level->Failed(); break; //Death of Charlie
				case GAME_OBJECT_DOOR: HandleCollision(dynamic_cast<Door *>(go2), go1); break; // Collide if Door is closed
				case GAME_OBJECT_FLOWER: HandleCollision(dynamic_cast<Flower *>(go2), go1); break; //Collide if Flower is closed
				case GAME_OBJECT_GOAL: break;
				case GAME_OBJECT_MILLIPEDE: HandleCollision(go1, dynamic_cast<Millipede *>(go2)); break;
				case GAME_OBJECT_SNAKE: break;
				case GAME_OBJECT_SWITCH: HandleCollision(dynamic_cast<Switch *>(go2), go1); break; //Toggle Switch
				case GAME_OBJECT_TIGERCUB: break;
				case GAME_OBJECT_SCORPION: break;
				default: assert(0);
			}
		case GAME_OBJECT_SCORPION:
			switch (go2->GetType()) {
				case GAME_OBJECT_BAT: break;
				case GAME_OBJECT_BEE: break;
				case GAME_OBJECT_BUTTERFLY: break;
				case GAME_OBJECT_CENTIPEDE: HandleCollision(dynamic_cast<Centipede *>(go2), go1); break; //Collide if Centipede is rolled up.
				case GAME_OBJECT_CHARLIE: go1->PlayCollisionSound(); go2->SetDisabled(true); go1->level->Failed(); break; //Death of Charlie
				case GAME_OBJECT_DOOR: HandleCollision(dynamic_cast<Door *>(go2), go1); break; // Collide if Door is closed
				case GAME_OBJECT_FLOWER: HandleCollision(dynamic_cast<Flower *>(go2), go1); break; //Collide if Flower is closed
				case GAME_OBJECT_GOAL: break;
				case GAME_OBJECT_MILLIPEDE: HandleCollision(go1, dynamic_cast<Millipede *>(go2)); break;
				case GAME_OBJECT_SNAKE: break;
				case GAME_OBJECT_SWITCH: HandleCollision(dynamic_cast<Switch *>(go2), go1); break; //Toggle Switch
				case GAME_OBJECT_TIGERCUB: break;
				case GAME_OBJECT_SCORPION: break;
				default: assert(0);
			}
			break;
		default: assert(0);
	}
}

void GameObject::HandleCollision(Butterfly *b, Charlie *c)
{
	if(b->IsDayButterfly())
		b->level->MakeItDay();
	else
		b->level->MakeItNight();

	b->SetDisabled(true);
}

void GameObject::HandleCollision(Centipede *c, GameObject * go)
{
	if(!c->GetRolledUp()) return;

	c->SetPosition(c->GetPosition() - c->GetVelocity().Rescaled(c->GetOverlap(go)));
}

void GameObject::HandleCollision(GameObject *go, Millipede *m)
{
	if(!m->GetRolledUp()) return;

	m->SetPosition(m->GetPosition() - m->GetVelocity().Rescaled(m->GetOverlap(go)));
}

void GameObject::HandleCollision(Charlie *ch, Centipede *ce)
{
	if(ce->GetRolledUp())
	{
		const Vec2f directionTo = ch->GetPosition() - ce->GetPosition();
		float overlap = ch->GetOverlap(ce);

		if(fabs(directionTo.x) >= fabs(directionTo.y))
		{
			ce->SetPosition(ce->GetPosition() - Vec2f(overlap * directionTo.x/fabs(directionTo.x), 0.0f));
			ce->SetVel(Vec2f(-1 * overlap * directionTo.x/fabs(directionTo.x),0));

		}
		else
		{
			ce->SetPosition(ce->GetPosition() - Vec2f(0.0f, overlap * directionTo.y/fabs(directionTo.y)));
			ce->SetVel(Vec2f(0,-1 * overlap * directionTo.y/fabs(directionTo.y)));
		}
		//overlap = ch->GetRadius() + ce->GetRadius() - (ch->GetPosition() - ce->GetPosition()).Length();
		//assert(overlap != 0);
		
	}
}

void GameObject::HandleCollision(Charlie *c, Flower *f)
{
	if(f->IsClosed())
	{
		c->SetPosition(c->GetPosition() - c->GetVelocity().Rescaled(c->GetOverlap(f))); //Move this Charlie off this Flower. The Flower does not move.
	}
	else
	{
		c->SetFlowerColor(f->GetColor()); //This Charlie has the opportunity to change to this Flower's Color.
	}
}

void GameObject::HandleCollision(Door *d, GameObject *go)
{
	if(d->GetState())
	{
		go->SetPosition(go->GetPosition() - go->GetVelocity().Rescaled(d->GetOverlap(go)));
	}
}

void GameObject::HandleCollision(Flower *f, GameObject *go)
{
	if(f->IsClosed())
	{
		go->SetPosition(go->GetPosition() - go->GetVelocity().Rescaled(f->GetOverlap(go)));
	}
}

void GameObject::HandleCollision(GameObject *go, Tile *t)
{
	if(go->GetType() == GAME_OBJECT_BEE || go->GetType() == GAME_OBJECT_BAT || go->GetType() == GAME_OBJECT_BUTTERFLY) return;

	if(t->IsCollidingWith(go)){
		Vec2f pushBackX = go->GetPosition() - Vec2f(go->GetVelocity().x,0);
		Vec2f pushBackY = go->GetPosition() - Vec2f(0,go->GetVelocity().y);
		Vec2f pushBack = go->GetPosition() - go->GetVelocity();
		if(go->level->GetTile(pushBackX)->passable)
			go->SetPosition(pushBackX);
		else if(go->level->GetTile(pushBackY)->passable)
			go->SetPosition(pushBackY);
		else
			go->SetPosition(pushBack);
		//assert(!go->level->GetTile(pushBack)->passable);
	}
}

void GameObject::HandleCollision(Charlie *c, Goal *g)
{
	c->level->Complete();
}

void GameObject::HandleCollision(Charlie *c, Millipede *m)
{
	if(m->GetRolledUp())
	{
		const Vec2f directionTo = c->GetPosition() - m->GetPosition();
		float overlap = c->GetOverlap(m) + 0.05f;

		if(fabs(directionTo.x) >= fabs(directionTo.y))
		{
			m->SetPosition(m->GetPosition() - Vec2f(overlap * directionTo.x/fabs(directionTo.x), 0.0f));
			m->SetVel(Vec2f(-1 * overlap * directionTo.x/fabs(directionTo.x),0));

		}
		else
		{
			m->SetPosition(m->GetPosition() - Vec2f(0.0f, overlap * directionTo.y/fabs(directionTo.y)));
			m->SetVel(Vec2f(0,-1 * overlap * directionTo.y/fabs(directionTo.y)));
		}
	}
	else
	{
		m->PlayCollisionSound();
		c->SetDisabled(true);
		m->level->Failed();

	}
}

void GameObject::HandleCollision(Switch *s, GameObject *go)
{
	s->Trigger();
}

void GameObject::Rebound(Charlie *c, GameObject *go)
{
	float overlap = c->GetOverlap(go);
	if (overlap <= 0.0f)
		return;

	if (dynamic_cast<Centipede*>(go) &&
		dynamic_cast<Centipede*>(go)->GetRolledUp())
		c->SetPosition(c->GetPosition() - c->GetVelocity().Rescaled(overlap));
	else if (dynamic_cast<Millipede*>(go) &&
			 dynamic_cast<Millipede*>(go)->GetRolledUp())
		c->SetPosition(c->GetPosition() - c->GetVelocity().Rescaled(overlap));
}

// vim: set ts=4 sts=4 sw=4 noet:
