/// \file GameObject.hpp Header file for the GameObject class

/// Include guard for GameObject.hpp
#ifndef INCLUDED_GAMEOBJECT_H
#define INCLUDED_GAMEOBJECT_H 1

#include "Utility.hpp"
#include "Saveable.hpp"

// Forward declarations to prevent horrible include loops...
// Files included after GameObject definition
class Level;
class Tile;
class Centipede;
class Charlie;
class Flower;
class Goal;
class TigerCub;
class Switch;
class Door;
class Butterfly;
class Millipede;

/** \brief Represents an object on the Level in a Level that can perform
 * actions.
 *
 * The GameObject class represents an object on the Level in a Level that can
 * perform actions.  GameObject is a virtual class and does not represent an
 * actual, specific object; rather it provides a generalized interface to those
 * objects, which are its child classes. GameObject specifies an enumerated
 * type for describing specific objects, as well as the fields and methods that
 * all specific objects should implement.
 */
class GameObject : public Saveable {
public:
	/** \brief Describes specific objects.
	 *
	 * This enum describes the specific objects which GameObject provides an
	 * interface to.  These objects are all child classes of GameObject.
	 */
    enum GameObjectType {
		GAME_OBJECT_BAT,
		GAME_OBJECT_BEE,		/**< Swarm of Bees */
		GAME_OBJECT_BUTTERFLY,	/**< Butterfly */
		GAME_OBJECT_CENTIPEDE,	/**< Centipede (rolly-polly) */
		GAME_OBJECT_CHARLIE,	/**< Charlie the Chameleon (human-controlled) */
		GAME_OBJECT_DOOR,		/**< A Door */
		GAME_OBJECT_FLOWER,		/**< Immobile flower, for color changing */
		GAME_OBJECT_GOAL,		/**< The goal tile for the level */
		GAME_OBJECT_MILLIPEDE,	/**< Millipede (Killer bug!!) */
		GAME_OBJECT_SNAKE,		/**< Snake */
		GAME_OBJECT_SWITCH,		/**< Switch */
		GAME_OBJECT_TIGERCUB,	/**< Tiger Cub */
		GAME_OBJECT_SCORPION	/**< Scorpion */
	};

	/** \brief Destructs a GameObject.
	 *
	 * This is GameObject's destructor. Unlike its constructor, most child
	 * classes do not need to do any additional cleanup, and can rely on the
	 * default GameObject destructor to do it for them.
	 */
    virtual ~GameObject() {}

	/** \brief Runs one cycle of this GameObject.
	 *
	 * Performs one cycle's worth of action from this GameObject. This is a
	 * true virtual method and must be implemented by each child class. The
	 * actions that can be performed, and the logic that chooses which actions
	 * to perform, is specific to each child class.
	 */
	virtual void Run(void) = 0;

	/** \brief Draws one frame of this GameObject.
	 */
    virtual void Draw(SDL_Surface *screen);

	/** \brief Resets this GameObject to its original state in its Level.
	 *
	 * Resets this GameObject to the state it began this Level in. Taken
	 * together with SetDisabled() and GetDisabled(), these methods provide an
	 * easy way to "destroy" GameObjects and then later reset the Level without
	 * having to constantly destruct and reload data from the Level file.
	 */
	virtual void Reset(void);

	/** \brief Sets whether or not this GameObject is disabled.
	 * \param d whether or not this GameObject is disabled
	 */
	virtual void SetDisabled(bool d) { disabled = d; }

	/** \brief Sets the max_speed of this GameObject.
	 * \param ms the float max_speed to set
	 */
	inline void SetMaxSpeed(float ms) { max_speed = ms; }

	/** \brief Sets the position of this GameObject.
	 * \param p the Vec2f position to set
	 */
	inline void  SetPosition(Vec2f p) { position = p; }

	/** \brief Sets the radius of this GameObject.
	 * \param r the float radius to set
	 */
	inline void SetRad(float r) { radius = r; }

	/** \brief Sets the tile_position of this GameObject.
	 * \param tp the Vec2i tile_position to set
	 */
	inline void SetTilePos(Vec2i tp) { tile_position = tp; }

	/** \brief Sets the velocity of this GameObject.
	 * \param v the Vec2f velocity to set
	 */
	inline void  SetVel(Vec2f v) { velocity = v; }

	/** \brief Set the orientation of this GameObject
	 *
	 * The orientation is the rotation with which the object is
	 * drawn when it has 0 velocity (it is used as if it had a
	 * velocity at this angle (in radians).
	 * \param orientang The orientation angle (in radians).
	 */
	inline void SetOrientation(float orientang) { orientation = orientang; }

	/** \brief Gets whether or not this GameObject is disabled.
	 * \return whether or not this GameObject is disabled
	 */
	inline bool GetDisabled(void) const { return disabled; }

	/** \brief Returns the max_speed of this GameObject.
	 * \return the const float max_speed of this GameObject
	 */
	inline float GetMaxSpeed(void) const { return max_speed; }

	/** \brief Returns the position of this GameObject.
	 * \return the const Vec2f position of this GameObject
	 */
	inline Vec2f GetPosition(void) const { return position; }

	/** \brief Gets the radius of this GameObject.
	 * \return the const float radius of this GameObject
	 */
	inline float GetRadius(void) const { return radius; }

	/** \brief Gets the tile_position of this GameObject.
	 * \return the const Vec2i tile_position of this GameObject
	 */
	inline Vec2i GetTilePos(void) const { return tile_position; }

	/** \brief Returns the GameObjectType of this GameObject.
	 *
	 * Returns the GameObjectType of this GameObject. Since only child classes
	 * have a GameObjectType, this is a true virtual method that must be
	 * implemented by each child class.
	 *
	 * \return the GameObjectType of this GameObject
	 */
	virtual GameObjectType GetType(void) const = 0;

	/** \brief Returns the velocity of this GameObject.
	 * \return the const Vec2f velocity of this GameObject
	 */
	inline Vec2f GetVelocity(void) const { return velocity; }

	/** \brief Does this GameObject have an associated color?
	  * \return true if this GameObject has a color, false otherwise
	  */
	inline bool HasColor() const { return hascolor; }

	/** \brief Gets the Color of this GameObject.
	 * \return the Color of this GameObject
	 */
	inline const ColorRGBA& GetColor(void) const { return color; }

	/** \brief Get the overlap of this GameObject with another
	 * \param go The GameObject to find the overlap with
	 * \return The sum of radii of the objects minus the distance between them
	 */
	inline float GetOverlap(const GameObject *go) const
	{
		return GetRadius() + go->GetRadius() -
			   (GetPosition() - go->GetPosition()).Length();
	}

	/** \brief Is this GameObject colliding with a given GameObject?
	 * \param go The GameObject to check for a collision with
	 * \return true if the objects collision radii overlap
	 */
	inline bool IsCollidingWith(const GameObject *go) const
	{
		return !disabled && !go->disabled && GetOverlap(go) > 0.0f;
	}

	/** \brief Sets the Color of this GameObject.
	 * \param c the Color to set
	 */
	void SetColor(ColorRGBA c) {color = c; drawcolor = c;};

	/** \brief Play the sound of this GameObject colliding (if appropriate)
	 */
	virtual void PlayCollisionSound() const {};

	/** \brief Handles a collision between two GameObjects.
	 *
	 * Determines and performs the consequences of a collision between two
	 * GameObjects. This method does not perform collision detection, which is
	 * the responsibility of the Level; rather it is called by the Level once a
	 * collision has been detected. Its behavior may be unexpected if it is
	 * called on two GameObjects that are not actually colliding.
	 *
	 * \param go1 One of the GameObjects involved in the collision
	 * \param go2 The other GameObject involved in the collision
	 */
	static void HandleCollision(GameObject *go1, GameObject *go2);

	/** \brief Handles a collision between a GameObject and a Tile
	 *
	 * This function is called by the Level once a * collision has been
	 * detected between a GameObject and a Tile.
	 *
	 * \param go The GameObject involved in the collision
	 * \param t The Tile involved in the collision
	 */
	static void HandleCollision(GameObject *go, Tile *t);

	/** \brief checks if this gameobject is a patroller
	 *
	 *  returns true if it is a patroller, false otherwise
	 */
	virtual bool IsPatroller(){return false;}
	
	static void Rebound(Charlie *c, GameObject *g);

protected:
	/** \brief Constructs a GameObject in Level l at position p.
	 *
	 * \param l a pointer to the Level this GameObject exists in
	 * \param p the Vec2f start_position of this GameObject
	 * \param max_speed the maximum speed for this GameObject
	 * \param radius the collision radius of this GameObject
	 */
	GameObject(Level *l, Vec2f p,
			   float max_speed = 0.0f, float radius = 0.5f);

	/** \brief Constructs a GameObject in Level l at position p with color c.
	 *
	 * \param l a pointer to the Level this GameObject exists in
	 * \param p the Vec2f start_position of this GameObject
	 * \param c the color of this GameObject
	 * \param max_speed the maximum speed for this GameObject
	 * \param radius the collision radius of this GameObject
	 */
	GameObject(Level *l, Vec2f p, ColorRGBA c,
			   float max_speed = 1.0f, float radius = 1.0f);

	Level *level;				///< The Level in which this GameObject exists.

	Vec2f position;				///< The position of this GameObject.
	const Vec2f start_position; ///< The starting position of this GameObject.
	Vec2f velocity;				///< The velocity of this GameObject.
	Vec2i tile_position;		///< The tile position of this GameObject.

	bool  disabled;				///< Whether or not this GameObject is disabled.
	float max_speed;			///< The maximum speed of this GameObject.
	float radius;				///< The radius of this GameObject.

	SDL_Surface *image;			///< The image of this GameObject.
	unsigned int anim_index;	///< Position in animation sequence
	Vec2i imagesize;			///< Height and width of image
	float orientation;			///< Preferred rotation for drawing when still
	float prevang;				///< The last recorded velocity angle

private:
	bool hascolor;				///< Does this GameObject have a color?
	ColorRGBA color;				///< The color of this GameObject.
	ColorRGBA drawcolor;			///< The color that will be drawn over the image.

	static void HandleCollision(Butterfly *b, Charlie *c);
	static void HandleCollision(Centipede *c, GameObject * go);
	static void HandleCollision(Charlie *ch, Centipede *ce);
	static void HandleCollision(Charlie *c, Flower *f);
	static void HandleCollision(Charlie *c, Goal *g);
	static void HandleCollision(Charlie *c, Millipede *m);
	static void HandleCollision(Door *d, GameObject *go);
	static void HandleCollision(Flower *f, GameObject *go);
	static void HandleCollision(GameObject *go, Millipede *m);
	static void HandleCollision(Switch *s, GameObject *go);
};

#include "Level.hpp"
#include "Centipede.hpp"
#include "Charlie.hpp"
#include "Flower.hpp"
#include "Goal.hpp"
#include "TigerCub.hpp"
#include "Tiles.hpp"
#include "Switch.hpp"
#include "Door.hpp"
#include "Butterfly.hpp"
#include "Millipede.hpp"
#include "Scorpion.hpp"

#endif /* INCLUDED_GAMEOBJECT_H */
// vim: set ts=4 sts=4 sw=4 noet:
