/// \file Goal.cpp Source file for the Goal class

#include <sstream>

#include "SDL_gfxPrimitives.h"

#include "Goal.hpp"
#include "LevelWriter.hpp"

SDL_Surface *Goal::goalimg;
unsigned int Goal::goalcount = 0;

Goal::Goal(Level *l, Vec2f p) :
   	GameObject(l, p, GOAL_DEFAULT_MAX_SPEED, GOAL_DEFAULT_RADIUS),
   	waypoints(1,p),
   	wp_index(0),
   	state(GOAL_STATE_INTRO),
	anim_timer(0)
{
	if (goalcount++ == 0) {
		goalimg = LoadSprite(PKGDATADIR"Artwork/creatures/goal/goal.png",
							 CREATURE_ROTATIONS,
							 level->GetTileWidth(),
							 level->GetTileHeight());
	}

	SetOrientation(0.0f);
	image = goalimg;

	exitvelocity = Vec2f(level->GetTileDimension().x/2 - position.x,
						 position.y - level->GetTileDimension().y/2);
	exitvelocity *= max_speed/exitvelocity.Length();
}

Goal::~Goal()
{
	if (--goalcount == 0) {
		SDL_FreeSurface(goalimg);
	}
}

void Goal::Run()
{
	if (disabled)
	   	return;

	anim_timer = (anim_timer + 1) % GOAL_ANIMATION_SPEED;
	if(anim_timer == 0)
		anim_index = (anim_index + 1) % 4;

	switch (state) {
		case GOAL_STATE_INTRO:
			if (wp_index == waypoints.size()) {
				state = GOAL_STATE_RUNNING;
				velocity = Vec2f(0.0f, 0.0f);
			} else if (position == waypoints[wp_index]) {
				wp_index++;
			} else {
				velocity = waypoints[wp_index] - position;
				if (velocity.Length() > max_speed) {
					velocity *= max_speed/velocity.Length();
					velocity += velocity.PerpVec() * GetOscillatingValue(-0.5f, 0.5f);
				}
				position += velocity;
			}
			break;
		case GOAL_STATE_OUTRO:
			if (position.x < -1 ||
				position.x > level->GetCols() ||
				position.y < -1 ||
				position.y > level->GetRows()) {
				state = GOAL_STATE_DONE;
			} else {
				velocity = exitvelocity;
				velocity += velocity.PerpVec() * GetOscillatingValue(-0.5f,0.5f);
				position += velocity;
			}
			break;
	}
}

void Goal::Reset()
{
	this->GameObject::Reset();

	position = waypoints.back();
	image = goalimg;

	max_speed = GOAL_DEFAULT_MAX_SPEED;
	radius = GOAL_DEFAULT_RADIUS;
	velocity = Vec2f(0.0f, 0.0f);

	anim_index = 0;
	anim_timer = 0;
	state = GOAL_STATE_RUNNING;
}

std::string Goal::GetXMLRepresentation(const std::string& indent) const
{
	std::ostringstream xml;
	xml << indent << "<object x=\"" << level->GetTilePos(position).x + 0.5f
		<< "\" y=\"" << level->GetTilePos(position).y + 0.5f
		<< "\" type=\"goal\"";

	std::string wptstring = LevelWriter::waypointsToXML(indent+"  ", position,
														waypoints.begin(),
														waypoints.end());

	if (wptstring.size() == 0)
		xml << "/>\n";
	else
		xml << ">\n" << wptstring << indent << "</object>\n";

	return xml.str();
}

void Goal::DrawWaypoints(SDL_Surface *screen, ColorRGBA color)
{
	Vec2f first = waypoints.front();
	Vec2f prev;
	for (std::vector<Vec2f>::iterator i = waypoints.begin();
		 i != waypoints.end(); i++) {
		if (*i != first)
			aalineRGBA(screen, prev.x, prev.y, (*i).x, (*i).y,
					   color.r, color.g, color.b, color.a);

		prev = *i;
	}
	aalineRGBA(screen, prev.x, prev.y, first.x, first.y,
			   color.r, color.g, color.b, color.a);
}

void Goal::RestartFlyThrough()
{
	state = GOAL_STATE_INTRO;
	wp_index = 0;
	position = waypoints.front();
}

void Goal::ResetWaypoints(Vec2f pos)
{
	waypoints.clear();
	waypoints.push_back(pos);
}

// vim: set ts=4 sts=4 sw=4 noet:
