/// \file Goal.hpp Header file for the Goal class

/// Include guard for Goal.hpp
#ifndef INCLUDED_GOAL_H
#define INCLUDED_GOAL_H 1

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif
#include "SDL_image.h"

#include "GameObject.hpp"
#include "Patroller.hpp"

/**
 * \brief Class representing a goal tile.
 */
class Goal : public GameObject, public Patroller
{
public:
	/** \brief Constructs a Goal in Level l at position p.
	* \param l a pointer to the Level this Goal exists in
	* \param p the Vec2f start_position of this goal tile
	*/
	Goal(Level *l, Vec2f p);

	/** \brief Destructs a Goal */
	~Goal();

	/** \brief Runs one cycle of this Goal.
	*/
	void Run(void);

	/** \brief Resets this Goal to its original state in its Level.
	 *
	 * For the Goal, this state is the one after its introductory fly-through
	 * of the Level.
	*/
	void Reset(void);

	void AddWaypoint(const Vec2f waypoint, bool duringday = true,
					 bool duringnight = true)
		{
			waypoints.push_back(waypoint);
			exitvelocity = Vec2f(level->GetTileDimension().x/2 - waypoint.x,
								 waypoint.y - level->GetTileDimension().y/2);
			exitvelocity *= max_speed/exitvelocity.Length();
		}

	//void ResetWaypoints(Vec2f pos) {waypoints.clear(); waypoints.push_back(pos);}

	/** \brief draws the creature's waypoints
	 *
	 * Draws the creature's waypoints as dots and path between them as lines
	 */
	void DrawWaypoints(SDL_Surface *screen, ColorRGBA color);

	std::string GetXMLRepresentation(const std::string& indent) const;

	/** \brief Gets the GameObjectType of this Goal tile.
	 * \return the GameObjectType of this Goal tile.
	 */
	inline GameObjectType GetType(void) const { return GAME_OBJECT_GOAL; }

	/** \brief Restart the Level-indroductory fly-through. */
	void RestartFlyThrough();

	bool IsPatroller() { return true; }

	

	/** \brief resets creature's waypoints
	 *
	 * Clears all the creature's waypoints except for the origin
	 */
	void ResetWaypoints(Vec2f pos);

	/** \brief Is the introductory fly-through complete for this Goal?
	 * \return true if the introductory fly-through is complete
	 */
	inline bool IsIntroDone() const { return state != GOAL_STATE_INTRO; }

	/** \brief Start the level-end fly-out sequence */
	inline void StartOutro() { state = GOAL_STATE_OUTRO; }

	/** \brief Is the level-end fly-out complete for this Goal?
	 * \return true if the level-end fly-out is complete
	 */
	inline bool IsOutroDone() const { return state == GOAL_STATE_DONE; }
private:
	static SDL_Surface *goalimg;	///< image of Goal facing right
	static unsigned int goalcount;	///< Number of Goals created

	enum GoalState {
		GOAL_STATE_INTRO,
		GOAL_STATE_RUNNING,
		GOAL_STATE_OUTRO,
		GOAL_STATE_DONE
	};

	GoalState state;				///< The state of this Goal
	unsigned int anim_timer;		///< Time until this Goal changes frames.

	std::vector<Vec2f> waypoints;	///< The waypoints of this Goal
	unsigned int wp_index;			///< The current index of this Goal's waypoints
	Vec2f exitvelocity;				///< Velocity at which this Goal should exit the Level
};

#endif /* INCLUDED_GOAL_H */
// vim: set ts=4 sts=4 sw=4 noet:
