/// \file Level.cpp Source file for the Level class

#include <cmath>

#include "Utility.hpp"
#include "Level.hpp"
#include "Bee.hpp"
#include "Snake.hpp"
#include "Bat.hpp"

Level::Level() : tiledim(Vec2i(32,32)),
				 daylength(90*60),
				 initialdayremaining(90*60),
				 dayremaining(0),
				 isday(true),
				 iscomplete(false),
				 hasfailed(false),
				 victorytype(VICTORY_GOALTILE),
				 chameleon(NULL),
				 state(LEVEL_STATE_INTRO),
				 editorSel(NULL)
{
	day_sound = LoadSound(PKGDATADIR"Sounds/DayTransition.ogg");
	night_sound = LoadSound(PKGDATADIR"Sounds/NightTransition.ogg");

	if (FileExists(PKGDATADIR"Sounds/Geuka-DayMusic2.wav"))
		daymusic = LoadSound(PKGDATADIR"Sounds/Geuka-DayMusic2.wav");
	else
		daymusic = LoadSound(PKGDATADIR"Sounds/Geuka-DayMusic2.ogg");

	if (FileExists(PKGDATADIR"Sounds/Geuka-NightMusic2.wav"))
		nightmusic = LoadSound(PKGDATADIR"Sounds/Geuka-NightMusic2.wav");
	else
		nightmusic = LoadSound(PKGDATADIR"Sounds/Geuka-NightMusic2.ogg");
}

Level::~Level()
{
	for (unsigned int x=0; x<tiles.size(); x++)
		for (unsigned int y=0; y<tiles.size(); y++)
			delete tiles[x][y];

	for (unsigned int i=0; i<creatures.size(); i++)
		delete creatures[i];

	for (unsigned int i=0; i<objects.size(); i++)
		delete objects[i];

	for (unsigned int i=0; i<goals.size(); i++)
		delete goals[i];

	for (unsigned int x=0; x<editorTiles.size(); x++)
		for (unsigned int y=0; y<editorTiles[x].size(); y++)
			delete editorTiles[x][y];

	if (editorSel)
		SDL_FreeSurface(editorSel);

	SoundManager::StopChunk(day_sound);
	SoundManager::StopChunk(night_sound);
	SoundManager::StopMusic(daymusic);
	SoundManager::StopMusic(nightmusic);

	Mix_FreeChunk(day_sound);
	Mix_FreeChunk(night_sound);
	Mix_FreeChunk(daymusic);
	Mix_FreeChunk(nightmusic);
}

void Level::Initialize(const unsigned int width, const unsigned int height,
					   const Vec2i view_offset,
					   const unsigned int view_width,
					   const unsigned int view_height)
{
	tiles.resize(width);
	visited.resize(width);
	tiledim.x = view_width/width;
	tiledim.y = view_height/height;
	view.offset = view_offset;
	view.dimension.x = tiledim.x*width;
	view.dimension.y = tiledim.y*height;

	markcount = 0;
	for (unsigned int i=0; i<tiles.size(); i++)
		visited[i].resize(height, markcount);

	// Start with a level of all LandTiles
	for (unsigned int x=0; x<tiles.size(); x++) {
		tiles[x].resize(height);
		for (unsigned int y=0; y<height; y++)
			tiles[x][y] = new LandTile(x, y, tiledim);
	}
}

void Level::InitializeEditor(const unsigned int width,
							 const unsigned int height,
							 const Vec2i view_offset,
							 const unsigned int view_width,
							 const unsigned int view_height)
{
	tiles.resize(width);
	editorTiles.resize(width);
	visited.resize(width);
	tiledim.x = view_width/width;
	tiledim.y = view_height/height;
	view.offset = view_offset;
	view.dimension.x = view_width;
	view.dimension.y = view_height;

	markcount = 0;
	for (unsigned int i=0; i<tiles.size(); i++)
		visited[i].resize(height, markcount);

	// Start with a level of all LandTiles
	for (unsigned int x=0; x<tiles.size(); x++) {
		tiles[x].resize(height);
		editorTiles[x].resize(height);
		for (unsigned int y=0; y<height; y++) {
			if (y==0 || y==height-1 || x==0 || x==width-1)
				tiles[x][y] = new RockTile(x, y, tiledim);
			else
				tiles[x][y] = new LandTile(x, y, tiledim);
			editorTiles[x][y] = new EditorTile(x, y, tiledim);
		}
	}

	editorSel = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/selected.png");
}

void Level::InitializeEditor()
{
	editorTiles.resize(tiles.size());

	// Start with a level of all LandTiles
	for (unsigned int x=0; x<editorTiles.size(); x++) {
		editorTiles[x].resize(tiles[x].size());
		for (unsigned int y=0; y<editorTiles[x].size(); y++) {
			editorTiles[x][y] = new EditorTile(x, y, tiledim);
		}
	}
	
	editorSel = LoadImageSDL(PKGDATADIR"Artwork/ui/level_editor/selected.png");
}

void Level::Reset()
{
	for(unsigned int x=0; x<tiles.size(); x++)
		for(unsigned int y=0; y<tiles[x].size(); y++)
				tiles[x][y]->RemoveAllObjects();
	
	for (unsigned int i=0; i<creatures.size(); i++)	{
		creatures[i]->Reset();
		AddObjectToTile(creatures[i]);
	}
	
	for (unsigned int i=0; i<objects.size(); i++) {
		objects[i]->Reset();
		AddObjectToTile(objects[i]);
	}

	for (unsigned int i=0; i<goals.size(); i++) {
		goals[i]->Reset();
		AddObjectToTile(goals[i]);
	}

	if (initialdayremaining < 0) {
		dayremaining = -initialdayremaining;
		isday = false;
	} else {
		dayremaining = initialdayremaining;
		isday = true;
	}

	iscomplete = false;
	hasfailed = false;
	survivaltimeremaining = survivaltime;
	state = LEVEL_STATE_PLAYING;

	if (SoundManager::IsMusicPlaying(daymusic))
		SoundManager::StopMusic(daymusic, 0);
	if (SoundManager::IsMusicPlaying(nightmusic))
		SoundManager::StopMusic(nightmusic, 0);

	StartMusic();
}

void Level::StartMusic() const {
	if (isday)
		SoundManager::PlayMusic(daymusic);
	else
		SoundManager::PlayMusic(nightmusic);
}

void Level::ClearVisited() {
	for(unsigned int x=0; x<tiles.size(); x++)
		for(unsigned int y=0; y<tiles[x].size(); y++)
				visited[x][y] = 0;
}

void Level::AddCreature(GameObject *creature)
{
	if (dynamic_cast<Charlie*>(creature))
		chameleon = dynamic_cast<Charlie*>(creature);

	creatures.push_back(creature);
	AddObjectToTile(creature);
}

void Level::AddCreature(Button::ButtonType type, Vec2f pos, ColorRGBA color)
{
	if(GetTile(pos)->objects.empty()){
		//float x = tiledim.x * floor(pos.x / tiledim.x) + tiledim.x / 2.0f;
		//float y = tiledim.y * floor(pos.y / tiledim.y) + tiledim.y / 2.0f;
		float x = pos.x;
		float y = pos.y;
		GameObject *creature;
		switch (type) {
			case Button::GAME_OBJECT_BEE:
				creature = new Bee(this,Vec2f(x,y),color);
				break;
			case Button::GAME_OBJECT_CENTIPEDE:
				creature = new Centipede(this,Vec2f(x,y));
				break;
			case Button::GAME_OBJECT_CHARLIE:
				chameleon = new Charlie(this,Vec2f(x,y),color);
				creature = chameleon;
				break;
			case Button::GAME_OBJECT_SNAKE:
				creature = new Snake(this,Vec2f(x,y),color);
				break;
			case Button::GAME_OBJECT_TIGERCUB:
				creature = new TigerCub(this,Vec2f(x,y));
				break;
			case Button::GAME_OBJECT_MILLIPEDE:
				creature = new Millipede(this,Vec2f(x,y));
				break;
			case Button::GAME_OBJECT_SCORPION:
				creature = new Scorpion(this,Vec2f(x,y));
				break;
			case Button::GAME_OBJECT_BAT:
				creature = new Bat(this,Vec2f(x,y));
				break;
			case Button::GAME_OBJECT_BUTTERFLY:
				creature = new Butterfly(this,Vec2f(x,y),color != COLOR_VIOLET);
				break;
		}
		creatures.push_back(creature);
		AddObjectToTile(creature);
	}
}

void Level::AddSpecial(Button::ButtonType type, Vec2f pos, ColorRGBA color)
{
	if (GetTile(pos)->objects.empty()) {
		GameObject *obj;
		switch (type) {
			case Button::GAME_OBJECT_SWITCH:
				obj = new Switch(this,pos);
				break;
			case Button::GAME_OBJECT_FLOWER:
				obj = new Flower(this,pos,color);
				break;
			case Button::GAME_OBJECT_DOOR:
				obj = new Door(this,pos);
				break;
			case Button::GAME_OBJECT_GOAL:
				obj = new Goal(this,pos);
				break;
			
		}
		if (dynamic_cast<Goal*>(obj))
			goals.push_back(dynamic_cast<Goal*>(obj));
		else
			objects.push_back(obj);
		AddObjectToTile(obj);
	}
}


void Level::AddObject(GameObject *object)
{
	if (dynamic_cast<Goal *>(object))
		goals.push_back(dynamic_cast<Goal *>(object));
	else
		objects.push_back(object);

	AddObjectToTile(object);
}

void Level::RemoveObject(const GameObject *object)
{
	tiles[object->GetTilePos().x][object->GetTilePos().y]->RemoveObject(object);

	bool removed = false;
	for (std::vector<GameObject*>::iterator o = objects.begin();
		 !removed && o != objects.end() ; o++)
		if (object == *o) {
			objects.erase(o);
			removed = true;
			break;
		}

	for (std::vector<GameObject*>::iterator c = creatures.begin();
		 !removed && c != creatures.end() ; c++)
		if (object == *c) {
			creatures.erase(c);
			removed = true;
			break;
		}
	
	for (std::vector<Goal*>::iterator g = goals.begin();
		 !removed && g != goals.end() ; g++)
		if (object == *g) {
			goals.erase(g);
			removed = true;
			break;
		}

	if (object == chameleon)
		chameleon = NULL;
}

void Level::AddObjectToTile(GameObject *object)
{
	int x = static_cast<int>(object->GetPosition().x);
	int y = static_cast<int>(object->GetPosition().y);
	assert(x >= 0 && x < static_cast<signed>(tiles.size()));
	assert(y >= 0 && y < static_cast<signed>(tiles[x].size()));

	tiles[x][y]->AddObject(object);
	object->SetTilePos(Vec2i(x,y));
}

void Level::ReplayIntro()
{
	Reset();
	state = LEVEL_STATE_INTRO;
	for (unsigned int i=0; i<goals.size(); i++)
		goals[i]->RestartFlyThrough();
}

void Level::Run()
{
	if (state == LEVEL_STATE_INTRO) {
		bool alldone = true;
		for (unsigned int i=0; i<goals.size(); i++) {
			goals[i]->Run();
			alldone = alldone && goals[i]->IsIntroDone();
		}

		if (alldone)
			state = LEVEL_STATE_PLAYING;

		return;
	} else if (state == LEVEL_STATE_VICTORY) {
		bool alldone = true;
		for (unsigned int i=0; i<goals.size(); i++) {
			goals[i]->Run();
			alldone = alldone && goals[i]->IsOutroDone();
		}
		if (alldone)
			iscomplete = true;
		return;
	}

	dayremaining--;
	if (dayremaining < 0) {
		ChangeDay();
	} else if (dayremaining == MUSIC_FADE_TIME_TICKS) {
		// Start fading out the music
		if (isday)
			SoundManager::StopMusic(daymusic, MUSIC_FADE_TIME);
		else
			SoundManager::StopMusic(nightmusic, MUSIC_FADE_TIME);
	}

	if (victorytype == VICTORY_SURVIVAL && survivaltimeremaining-- <= 0) {
		Complete();
	} else if (victorytype == VICTORY_COLLECT && Butterfly::LiveButterflyCount() == 0) {
		Complete();
	}

	for (unsigned int i=0; i<creatures.size(); i++)	{
		creatures[i]->Run();
		UpdateObject(creatures[i]);
	}
	
	for (unsigned int i=0; i<objects.size(); i++) {
		objects[i]->Run();
		UpdateObject(objects[i]);
	}

	for (unsigned int i=0; i<goals.size(); i++) {
		goals[i]->Run();
		UpdateObject(goals[i]);
	}
	
	CheckCollide();
}

void Level::KeyboardEvent(SDL_KeyboardEvent *event)
{
	switch (state) {
		case LEVEL_STATE_INTRO:
			if (event->type != SDL_KEYDOWN)
				return;

			for (unsigned int i=0; i<goals.size(); i++) {
				goals[i]->Reset();
				UpdateObject(goals[i]);
			}
			state = LEVEL_STATE_PLAYING;
			break;

		case LEVEL_STATE_PLAYING:
			chameleon->KeyboardEvent(event);
			break;

		default:
			// Key presses ignored if not playing or viewing the intro
			break;
	}
}

void Level::MouseMotionEvent(SDL_MouseMotionEvent *event)
{
}

void Level::MouseButtonEvent(SDL_MouseButtonEvent *event)
{
	switch (state) {
		case LEVEL_STATE_INTRO:
			if (event->type != SDL_MOUSEBUTTONDOWN)
				return;

			for (unsigned int i=0; i<goals.size(); i++) {
				goals[i]->Reset();
				UpdateObject(goals[i]);
			}
			state = LEVEL_STATE_PLAYING;
			break;

		case LEVEL_STATE_PLAYING:
			chameleon->MouseButtonEvent(event);
			break;

		default:
			// Mouse ignored if not playing or viewing intro
			break;
	}
}

void Level::UpdateObject(GameObject *go)
{
	int x = static_cast<int>(go->GetPosition().x);
	int y = static_cast<int>(go->GetPosition().y);

	//keeps all the objects inbound
	if(x < 0)
		x = 0;
	else if(static_cast<unsigned int>(x) >= tiles.size())
		x = tiles.size() - 1;
	if(y < 0)
		y = 0;
	else if(static_cast<unsigned int>(y) >= tiles[x].size())
		y = tiles[x].size() - 1;

	if(go->GetTilePos().x != x || go->GetTilePos().y != y) {
		tiles[go->GetTilePos().x][go->GetTilePos().y]->RemoveObject(go);
		tiles[x][y]->AddObject(go);
		go->SetTilePos(Vec2i(x,y));
	}	
}

void Level::CheckCollide()
{
	// for each tile not along the right or bottom edge, we check that
	// for every creature in that tile,
	// the creature is not colliding with another creature in that tile and
	// the creature is not colliding with a creature in the tile to the right and
	// the creature is not colliding with a creature in the tile below
	for (unsigned int x=0; x < tiles.size(); x++) {
		for (unsigned int y=0; y < tiles[x].size(); y++) {
			for (std::vector<GameObject*>::iterator c1 = tiles[x][y]->objects.begin();
				 c1 != tiles[x][y]->objects.end(); c1++) {
					if((*c1) != chameleon)
						GameObject::HandleCollision(*c1, chameleon);

					// Check this tile
					for (std::vector<GameObject*>::iterator c2 = c1+1;
						 c2 != tiles[x][y]->objects.end(); c2++)
						GameObject::HandleCollision(*c1, *c2);

					// Check tile to the right
					if (x < tiles.size()-1)
						for (std::vector<GameObject*>::iterator c2 = tiles[x+1][y]->objects.begin();
							 c2 != tiles[x+1][y]->objects.end(); c2++)
							GameObject::HandleCollision(*c1, *c2);

					// Check tile below
					if (y < tiles[x].size()-1)
						for (std::vector<GameObject*>::iterator c2 = tiles[x][y+1]->objects.begin();
							 c2 != tiles[x][y+1]->objects.end(); c2++)
							GameObject::HandleCollision(*c1, *c2);

					// Check tile below and to the right
					if (x < tiles.size()-1 && y < tiles.size()-1)
						for (std::vector<GameObject*>::iterator c2 = tiles[x+1][y+1]->objects.begin();
							 c2 != tiles[x+1][y+1]->objects.end(); c2++)
							GameObject::HandleCollision(*c1, *c2);

					
					GameObject::HandleCollision(*c1, tiles[x][y]);

					if((*c1) != chameleon)
						GameObject::Rebound(chameleon, *c1);
			}
		}
	}
}

void Level::Draw(SDL_Surface *screen) const
{
	for(unsigned int x=0; x<tiles.size(); x++)
		for(unsigned int y=0; y<tiles[x].size(); y++)
			tiles[x][y]->Draw(screen, x*tiledim.x + view.offset.x,
									  y*tiledim.y + view.offset.y);

	for (unsigned int i=0; i<objects.size(); i++)
		objects[i]->Draw(screen);

	for (unsigned int i=0; i<creatures.size(); i++) {
		if (creatures[i] != chameleon)
			creatures[i]->Draw(screen);
	}

	for (unsigned int i=0; i<goals.size(); i++) {
		goals[i]->Draw(screen);
	}

	if (state != LEVEL_STATE_INTRO)
		chameleon->Draw(screen);
}

void Level::DrawEditor(SDL_Surface *screen, GameObject* selected) const
{
	for(unsigned int x=0; x<tiles.size(); x++)
		for(unsigned int y=0; y<tiles[x].size(); y++) {
			tiles[x][y]->Draw(screen,x,y);
			editorTiles[x][y]->Draw(screen,x,y);
		}
	 

	for (unsigned int i=0; i<objects.size(); i++) {
		if (objects[i] == selected)
			continue;
		objects[i]->Draw(screen);
	}

	for (unsigned int i=0; i<creatures.size(); i++) {
		if (creatures[i] == selected)
			continue;

		if (dynamic_cast<Patroller *>(creatures[i]))
			dynamic_cast<Patroller *>(creatures[i])->DrawWaypoints(screen, ColorRGBA(0,0,255));

		creatures[i]->Draw(screen);
	}

	for (unsigned int i=0; i<goals.size(); i++) {
		if (goals[i] == selected)
			continue;
		goals[i]->Draw(screen);
	}

	if (selected) {
		SDL_Rect offset;
		offset.x = selected->GetPosition().x*tiledim.x - editorSel->w/2;
		offset.y = selected->GetPosition().y*tiledim.y - editorSel->h/2;
		SDL_BlitSurface(editorSel, NULL, screen, &offset);

		if (dynamic_cast<Patroller *>(selected))
			dynamic_cast<Patroller *>(selected)->DrawWaypoints(screen, ColorRGBA(255,0,0));
		selected->Draw(screen);
	}
}

void Level::SetTile(Tile *tile, const unsigned int x, const unsigned int y)
{
	assert(x < tiles.size() && y < tiles[x].size());

	delete tiles[x][y];

	tiles[x][y] = tile;
	UpdateTile(x,y);

	// Update surrounding tiles
	if (y > 0)
		UpdateTile(x, y-1);
	if (y+1 < tiles[x].size())
		UpdateTile(x, y+1);
	if (x > 0)
		UpdateTile(x-1, y);
	if (x+1 < tiles.size())
		UpdateTile(x+1,y);
}

void Level::SetTile(Button::ButtonType type, Vec2f pos, ColorRGBA color,
					bool showcolor)
{
	int sx = static_cast<int>(pos.x);
	int sy = static_cast<int>(pos.y);
	assert(sx >= 0 && sx < static_cast<signed>(tiles.size()));
	assert(sy >= 0 && sy < static_cast<signed>(tiles[sx].size()));
	unsigned int x = static_cast<unsigned int>(sx);
	unsigned int y = static_cast<unsigned int>(sy);

	delete tiles[x][y];

	switch (type) {
		case Button::TILE_BRIDGE:
			tiles[x][y] = new BridgeTile(x,y,GetTileDimension());
			break;
		case Button::TILE_LAND:
			if(color == COLOR_GREEN)
				tiles[x][y] = new LandTile(x,y,GetTileDimension());
			else
				tiles[x][y] = new LandTile(x,y,GetTileDimension(),color);
			break;
		case Button::TILE_ROCK:
			tiles[x][y] = new RockTile(x,y,GetTileDimension());
			break;
		case Button::TILE_TREE:
			tiles[x][y] = new TreeTile(x,y,GetTileDimension());
			break;
		case Button::TILE_WATER:
			tiles[x][y] = new WaterTile(x,y,GetTileDimension());
			break;
	}

	UpdateTile(x,y);

	// Update surrounding tiles
	if (y > 0)
		UpdateTile(x, y-1);
	if (y+1 < tiles[x].size())
		UpdateTile(x, y+1);
	if (x > 0)
		UpdateTile(x-1, y);
	if (x+1 < tiles.size())
		UpdateTile(x+1,y);
}

void Level::UpdateTile(const unsigned int x, const unsigned int y)
{
	assert(x >= 0 && x < tiles.size() &&
		   y >= 0 && y < tiles[x].size());

	Tile *above = (y == 0) ? NULL : tiles[x][y-1];
	Tile *below = (y+1 >= tiles.size()) ? NULL : tiles[x][y+1];
	Tile *left = (x == 0) ? NULL : tiles[x-1][y];
	Tile *right = (x+1 >= tiles.size()) ? NULL : tiles[x+1][y];

	tiles[x][y]->SetSurroundingTiles(above, right, below, left);
}

Tile *Level::GetTile (Vec2f pos) {
	int x = static_cast<int>(pos.x);
	int y = static_cast<int>(pos.y);
	assert(x >= 0 && x < static_cast<signed>(tiles.size()));
	assert(y >= 0 && y < static_cast<signed>(tiles[x].size()));

	return tiles[x][y];
}

EditorTile *Level::GetEdTile (Vec2f pos) {
	int x = static_cast<int>(pos.x);
	int y = static_cast<int>(pos.y);
	assert(x >= 0 && x < static_cast<signed>(editorTiles.size()));
	assert(y >= 0 && y < static_cast<signed>(editorTiles[x].size()));

	return editorTiles[x][y];
}

Tile *Level::GetTile(unsigned int x, unsigned int y) {
	if (x >= tiles.size())
		x=tiles.size()-1;
	if( y >= tiles[x].size())
		y=tiles[x].size()-1;

	return tiles[x][y];
}

EditorTile *Level::GetEdTile(unsigned int x, unsigned int y) {
	if (x >= editorTiles.size())
		x=editorTiles.size()-1;
	if( y >= editorTiles[x].size())
		y=editorTiles[x].size()-1;

	return editorTiles[x][y];
}

Vec2f Level::GetShortestPath(Vec2f s, Vec2f d, float vel){
	markcount++;
	Vec2i source = Vec2i(static_cast<int>(s.x),static_cast<int>(s.y));
	Vec2i dest = Vec2i(static_cast<int>(d.x), static_cast<int>(d.y));

	if(dest == source){
		Vec2f dir = Vec2f(d.x - s.x , d.y - s.y);
		if(dir.Length() > vel)
			return dir.Rescaled(vel);
		return dir;
	}
	else{
		pathNode root;
		root.pos = Vec2i(source.x, source.y);
		root.direction = DIRECTION_NONE;  //Vec2f(0,0);
		std::queue<pathNode> pathQ;
		pathQ.push(root);
		visited[root.pos.x][root.pos.y] = markcount;
		while(!pathQ.empty()){
			pathNode currNode = pathQ.front();
			pathQ.pop();
			int x = currNode.pos.x;
			int y = currNode.pos.y;
			if(currNode.pos == dest){
				if(currNode.direction  == DIRECTION_UP)
					return Vec2f(0,-1).Rescaled(vel);
				else if(currNode.direction == DIRECTION_RIGHT)
					return Vec2f(1,0).Rescaled(vel);
				else if(currNode.direction == DIRECTION_DOWN)
					return Vec2f(0,1).Rescaled(vel);
				else
					return Vec2f(-1,0).Rescaled(vel);
			}else{
				if(tiles[x][y]->passable){
					//top
					TryTile(&pathQ, currNode.pos.x,currNode.pos.y-1, currNode.direction == root.direction ? DIRECTION_UP : currNode.direction);//Vec2f(0,-1).Rescaled(vel) : currNode.direction);
					//right
					TryTile(&pathQ, currNode.pos.x+1,currNode.pos.y, currNode.direction == root.direction ? DIRECTION_RIGHT : currNode.direction);//Vec2f(1,0).Rescaled(vel) : currNode.direction);
					//bottom
					TryTile(&pathQ, currNode.pos.x,currNode.pos.y+1, currNode.direction == root.direction ? DIRECTION_DOWN : currNode.direction);//Vec2f(0,1).Rescaled(vel) : currNode.direction);
					//left
					TryTile(&pathQ, currNode.pos.x-1,currNode.pos.y, currNode.direction == root.direction ? DIRECTION_LEFT : currNode.direction);//Vec2f(-1,0).Rescaled(vel) : currNode.direction);
					//topleft
					//TryTile(&pathQ, currNode.pos.x-1,currNode.pos.y-1,Vec2f(-1,-1).Rescaled(vel), currNode.direction);
					//topright
					//TryTile(&pathQ, currNode.pos.x+1,currNode.pos.y-1,Vec2f(1,-1).Rescaled(vel), currNode.direction);
					//bottomleft
					//TryTile(&pathQ, currNode.pos.x-1,currNode.pos.y+1,Vec2f(-1,1).Rescaled(vel), currNode.direction);
					//bottomright
					//TryTile(&pathQ, currNode.pos.x+1,currNode.pos.y+1,Vec2f(1,1).Rescaled(vel), currNode.direction);
				}
			}
		}
		return Vec2f(0,0);
	}
	return Vec2f(0,0);	
}

void Level::TryTile(std::queue<pathNode> *pathQ,
					unsigned int x, unsigned int y,
					Direction d) {
	if(x < visited.size() && y < visited[x].size() && visited[x][y] != markcount){
		pathNode topNode;
		topNode.pos.x = x;
		topNode.pos.y = y;
		topNode.direction = d;
		pathQ->push(topNode);
		visited[x][y] = markcount;
	}
}

Vec2f Level::ViewPos2RealPos(const Vec2f pos) const{
	return Vec2f(pos.x + view.offset.x, pos.y + view.offset.y);
}


bool Level::HasLineOfSight(Vec2f pos1, Vec2f pos2) const
{
	assert(pos1.x >= 0.0f && pos1.y >= 0.0f && 
		   pos2.x >= 0.0f && pos2.y >= 0.0f);
	// Definition of line between the points y = mx + b
	float m = (pos1.y - pos2.y)/(pos1.x - pos2.x);
	float b = pos1.y - m*pos1.x;

	if (pos1.x > pos2.x) {
		Vec2f temppos = pos1;
		pos1 = pos2;
		pos2 = temppos;
	}

	unsigned int x1 = static_cast<unsigned int>(pos1.x) + 1;
	unsigned int x2 = static_cast<unsigned int>(pos2.x);


	// Check the vertical sides
	for (unsigned int x = x1;
		 x<x2 && x<tiles.size(); x++) {
		unsigned int y = static_cast<unsigned int>(m*x+b);		// Floor it
		if (!tiles[x][y]->seethrough)
			return false;
	}

	if (pos1.y > pos2.y) {
		Vec2f temppos = pos1;
		pos1 = pos2;
		pos2 = temppos;
	}

	unsigned int y1 = static_cast<unsigned int>(pos1.y) + 1;
	unsigned int y2 = static_cast<unsigned int>(pos2.y);

	// Check the horizontal sides
	for (unsigned int y = y1;
		 y<y2 && y<tiles[0].size(); y++) {
		unsigned int x = x2;
		if (pos1.x != pos2.x)
			x = static_cast<unsigned int>((y - b)/m);	// Floor it
		if (!tiles[x][y]->seethrough)
			return false;
	}

	return true;
}

Vec2i Level::GetTilePos(const Vec2f pos){
	return Vec2i(static_cast<int>(pos.x),static_cast<int>(pos.y));
}

void Level::Complete()
{
	state = LEVEL_STATE_VICTORY;
	for (unsigned int i=0; i<goals.size(); i++) {
		goals[i]->StartOutro();
	}
}

// vim: set ts=4 sts=4 sw=4 noet:
