/// \file Level.hpp Header file for the Level class

/// Include guard for Level.hpp
#ifndef INCLUDED_BOARD_H
#define INCLUDED_BOARD_H 1

#include <queue>

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif
#include "SDL_image.h"

#include "Utility.hpp"
#include "SoundManager.hpp"
#include "Button.hpp"

// GameObject forward-declared to avoid include-loop
// GameObject.hpp included after class definition
class GameObject;
class Charlie;
class Bee;
class Goal;
class Snake;
class TigerCub;
class Centipede;
class Millipede;
class Butterfly;
class Scorpion;
class Bat;
class Tile;
class EditorTile;

/**
 * \brief Representation of the level for a level
 *
 * This class handles the representation of the tileset,
 * containing a 2D array of tiles.  Also provides functions
 * for other classes to get the information of each tile, and
 * helper functions relating to the gamelevel like path finding.
 * It Also provides functions for other classes to get the information 
 * of each tile, and helper functions relating to the gamelevel 
 * like path finding.
 */
class Level {
	friend class LevelParser;
	friend class LevelWriter;
	friend class LevelEditor;

public:

	/** \brief Possible victory types (win conditions) */
	enum VictoryType {
		VICTORY_GOALTILE,		///< Win condition: Get to the goal tile
		VICTORY_SURVIVAL,		///< Win condition: Survive for a certain amount of time
		VICTORY_COLLECT		///< Win condition: Collect all the butterflies
	};

	enum LevelState {
		LEVEL_STATE_INTRO,
		LEVEL_STATE_PLAYING,
		LEVEL_STATE_VICTORY
	};

    /**	\brief Constructor for a Level */
	Level();

    /** \brief Destructor for a Level */
    ~Level();

	/** \brief Initialize the level
	 *
	 *	Set the size of the level to width x and height y,
	 *	and intialize all of the tiles.
	 *
	 *	\param x			number of tile columns
	 *	\param y			number of tile rows
	 *	\param view_offset	offset of the viewport
	 *	\param view_width	width of the viewport
	 *	\param view_height	height of the viewport
	 */
	void Initialize(const unsigned int x, const unsigned int y,
					const Vec2i view_offset = Vec2i(0,0),
					const unsigned int view_width = SDL_GetVideoSurface()->w,
					const unsigned int view_height = SDL_GetVideoSurface()->h);

	/** \brief Initialize the level editor
	 *
	 *	Set the size of the level editor to width x and height y,
	 *	and intialize all of the tiles.
	 *
	 *	\param x			number of tile columns
	 *	\param y			number of tile rows
	 *	\param view_offset	offset of the viewport
	 *	\param view_width	width of the viewport
	 *	\param view_height	height of the viewport
	 */
	void InitializeEditor(const unsigned int x, const unsigned int y,
					const Vec2i view_offset = Vec2i(0,0),
					const unsigned int view_width = SDL_GetVideoSurface()->w,
					const unsigned int view_height = SDL_GetVideoSurface()->h);

	/** \brief Initialize the level editor
	 *
	 *	Set the size of the level editor to width x and height y,
	 *	and intialize all of the tiles.
	 * */
	void InitializeEditor();

	/** \brief Start the level background music
	 */
	void StartMusic() const;

	/** \brief Reset the level to an empty state	
	 *
	 *	Basically clears all the vectors in the tile, while
	 *	keeping the background image.  Used to restart a level
	 */
    void Reset();


	/** \brief Get the number of rows of tiles on the level
	 * \return The horizontal dimension of the level (in tiles)
	 */
	unsigned int GetRows() const { return tiles.size(); }

	/** \brief Get the number of columns of tiles on the level
	 * \return The vertical dimension of the level (in tiles)
	 */
	unsigned int GetCols() const { if (tiles.size()) return tiles[0].size(); else return 0; }

	/** \brief Get the size of a single tile (in pixels)
	 * \return The width and height of a single tile (in pixels)
	 */
	Vec2i GetTileDimension() const { return tiledim; }
	int GetTileWidth() const { return tiledim.x; }
	int GetTileHeight() const { return tiledim.y; }

    /**	\brief Add a creature to the level
	 *
	 *	Called whenever a new GameObject is created, based on the position
	 *	of the GameObject, this function will assign the GameObject to the
	 *	appropriate tile.
	 *
	 *	\param creature pointer to the creature to be added
	 */
    void AddCreature(GameObject *creature);

	/**	\brief Add a creature to the level
	 *
	 *	Called whenever a new GameObject is created, based on the position
	 *	of the GameObject, this function will assign the GameObject to the
	 *	appropriate tile.
	 *
	 *	\param type type of creature to be added
	 */
	void AddCreature(Button::ButtonType type, Vec2f pos, ColorRGBA color);

		/**	\brief Add a creature to the level
	 *
	 *	Called whenever a new GameObject is created, based on the position
	 *	of the GameObject, this function will assign the GameObject to the
	 *	appropriate tile.
	 *
	 *	\param type type of creature to be added
	 */
	void AddSpecial(Button::ButtonType type, Vec2f pos, ColorRGBA color);

    /**	\brief Add a object to the level
	 *
	 *	Called whenever a new GameObject is created, based on the position
	 *	of the GameObject, this function will assign the GameObject to the
	 *	appropriate tile.
	 *
	 *	\param object pointer to the object to be added
	 */
    void AddObject(GameObject *object);

	/**	\brief Remove a GameObject from the level
	 *
	 *	Remove the game object reference for game object "go" in the tile it
	 *	was in.
	 *
	 *	\param go pointer to the gameobject to be removed
	 */
    void RemoveObject(const GameObject *go);

	/**	\brief Update the Level's state for GameObject
	 *
	 *  This function is called after a GameObject is updated to allow
	 *  Level to take account of the GameObject's new state.  The level may
	 *  move the GameObject from one tile to another, or perform any other
	 *  tasks to update its internal state.
	 *
	 *	\param go reference to the gameobject to be updated
	 */
    void UpdateObject(GameObject *go);

	/** \brief Runs one cycle of this Level.
	 *
	 * Performs one cycle's worth of action for this Level.
	 */
    void Run();

	/** \brief Report a keyboard event (key press or release)
	 * \param event The event to process
	 */
	void KeyboardEvent(SDL_KeyboardEvent *event);

	/** \brief Report a mouse motion event
	 * \param event The event to process
	 */
	void MouseMotionEvent(SDL_MouseMotionEvent *event);

	/** \brief Report a mouse button event (button press or release)
	 * \param event The event to process
	 */
	void MouseButtonEvent(SDL_MouseButtonEvent *event);

    /** \brief Draw the level
	 *
	 *	Draws the level by traversing through each tiles and drawing thier
	 *	image in the appropriate places on the level.
	 */
	void Draw(SDL_Surface *screen) const;

	/** \brief Draw the level editor
	 *
	 *	Draws the level editor by traversing through each tiles and drawing thier
	 *	image in the appropriate places on the level editor.
	 */
	void DrawEditor(SDL_Surface *screen, GameObject* selected) const;

    /** \brief Set the tile (x,y) to the given tile
	 *
	 *	\param tile	the tile representation
	 *	\param x	x position on the level
	 *	\param y	y position on the level
	 */
    void SetTile(Tile *tile, const unsigned int x, const unsigned int y);

	/** \brief Set the tile (x,y) to the given tile
	 *
	 *	\param type	the tile representation
	 *	\param pos The position of the Tile to retrieve
	 */
	void SetTile(Button::ButtonType type, Vec2f pos, ColorRGBA color, bool showColor) ;

    /** \brief Get the tile at the given position
	 *
	 *	\param pos The position of the Tile to retrieve
	 *	\return The Tile at the specified position
	 */
    Tile *GetTile(Vec2f pos);

    /** \brief Get the tile at the given position
	 *
	 *	\param x The x-position of the Tile to retrieve
	 *	\param y The y-position of the Tile to retrieve
	 *	\return The Tile at the specified position
	 */
	Tile *GetTile(unsigned int x, unsigned int y);

	/** \brief Get the editor tile at the given position
	 *
	 *	\param pos The position of the EditorTile to retrieve
	 *	\return The EditorTile at the specified position
	 */
    EditorTile *GetEdTile(Vec2f pos);

    /** \brief Get the editor tile at the given position
	 *
	 *	\param x The x-position of the EditorTile to retrieve
	 *	\param y The y-position of the EditorTile to retrieve
	 *	\return The EditorTile at the specified position
	 */
	EditorTile *GetEdTile(unsigned int x, unsigned int y);

	/** \brief Get the GameObject for Charlie */
	inline Charlie *GetCharlie();

	//@{
    /** \brief Get the first step along the shortest path from source to dest
	 *
	 *	\param source	position to start the path
	 *	\param dest		destination of the path
	 *	\return			first step along the shortest path from source to dest
	 */
	Vec2f GetShortestPath (Vec2f source, Vec2f dest, float vel);

	//@}

	/** \brief Can an object at pos1 see an object at pos2?
	 *
	 * \param pos1 Position of the observer
	 * \param pos2 Position of the observed
	 * \return Can an object at pos1 see an object at pos2?
	 */
	bool HasLineOfSight(Vec2f pos1, Vec2f pos2) const;

	/** \brief Translate the viewport coordinate to real coordinate
	 *
	 * \param pos viewport position
	 * \return position relative to GameX window
	 */
	Vec2f ViewPos2RealPos(const Vec2f pos) const;

	void ClearVisited();
	void CheckCollide();

	/** \brief Set the length of the day and night
	 *
	 * \param length length of the day in "ticks" (number of calls to Update)
	 */
	void SetDayLength(const int length);

	/** \brief Is it day or is it night?
	 *
	 * \return Is it day?
	 */
	bool IsItDay() const { return isday; }

	/** \brief Change Day to Night and Night to Day?
	 *
	 *	Change Day to Night and Night to Day?
	 */
	void ChangeDay();

	/** \brief Make it day. */
	void MakeItDay();

	/** \brief Make it night. */
	void MakeItNight();

	/** \brief Signal that the objective fot this Level can not be achieved
	 */
	void Failed() { hasfailed = true; }

	/** \brief Has the objective failed to be reached?
	 */
	bool HasFailed() const { return hasfailed; }

	/** \brief Signal that the objective for this Level has been completed
	 */
	void Complete();

	/** \brief Has the objective been reached?
	 */
	bool IsComplete() const { return iscomplete; }

	/** \brief Get the type of victory for this level
	 * \return the victory condition
	 */
	VictoryType GetVictoryType() const { return victorytype; }

	/** \brief Get the name of this level (for display purposes)
	 * \return the name of this level
	 */
	const std::string GetName() const { return levelname; }

	/** \brief Get the offset of this level (for display purposes)
	 * \return the offset of this level
	 */
	Vec2i GetOffset() const { return view.offset; }
	void SetOffset(const int x, const int y) { view.offset = Vec2i(x,y); }
	Vec2i GetDimension() const { return view.dimension; }

	Vec2i GetTilePos(const Vec2f pos);

	/** \brief Get the progress towards the next day/night change as a fraction.
	 * \return the progress towards the next day/night change
	 */
	float GetDayProgress() const { return static_cast<float>(daylength - dayremaining)/daylength; }

	/** \brief Get the amount of time remaining for survival mode (in seconds)
	 * \return the remaining time to completion for survival mode (in seconds)
	 */
	int GetSurvivalTimeRemaining() const { return survivaltimeremaining; }

	/** \brief Get the introductory message for this Level
	 * \return the introductory message for this Level
	 */
	inline const std::vector<std::string>& GetIntroMessage() const;

	/** \brief Replay the introductory sequence */
	void ReplayIntro();

	void setLevelName(std::string l){levelname = l;}

private:
	/** \brief Add an object to its corresponding Tile
	 *
	 * This function is called for objects which have no associated Tile or
	 * have been removed from their previous Tile (because of movement) and
	 * should be added to the Tile on which they are standing.
	 * \param object The object to be added to a Tile
	 */
	void AddObjectToTile(GameObject *object);

	/** \brief Non-existent copy constructor
	 * This is declared private to prevent the default copy constructor from being
	 * used.
	 */
	Level(const Level&) {}

	/** \brief Non-existent assignment operator
	 * This is declared private to prevent the default assignment operator from
	 * being used.
	 */
	Level& operator=(const Level&) { return *this; }

	// Note:  Both matrices are in column-major order tile[col][row]
	std::vector<std::vector<Tile *> > tiles;	///< constituent Tiles of Level
	std::vector<std::vector<EditorTile *> > editorTiles;	///< constituent EditorTiles of Level
	std::vector<std::vector<unsigned int> > visited;	///< has Tile been visited?
    std::vector<GameObject *> creatures;	///< All creatures in this Level
	std::vector<GameObject *> objects;		///< All objects in this Level
	std::vector<Goal *> goals;		///< All Goals in this Level
    Charlie *chameleon;						///< Charlie the Chameleon
	Vec2i tiledim;			///< width and height of a tile (in pixels)
	SDL_Surface *editorSel;
	LevelState state;

	struct viewport {
		Vec2i offset;
		Vec2i dimension;
	} view;

	struct pathNode{
		Vec2i pos;
		Direction direction;
	};

	struct pathNode2{
		Vec2i pos;
		Vec2f direction;
	};

	int markcount;

	std::string levelname;		///< The name of this Level
	std::vector<std::string> intromessage;	///< Tooltip message displayed during Level introduction

	int daylength;				///< 1/2 length of day/night cycle
	int dayremaining;			///< Counter for the day/night change
	int initialdayremaining;	///< Initial value of dayremaining (for reset)
	bool isday;					///< Is it daytime?
	VictoryType victorytype;	///< Victory condition for this Level
	bool iscomplete;			///< Has the victory condition been met?
	bool hasfailed;				///< Has the victory condition failed to be met?
	int survivaltime;			///< The length of time that Charlie must survive
	int survivaltimeremaining;	///< Counter for the survival time remaining

	Mix_Chunk *daymusic;				///< Background music during day
	Mix_Chunk *nightmusic;				///< Background music during night

	Mix_Chunk *day_sound;				///< Sound to signal user of day change
	Mix_Chunk *night_sound;				///< Sound to signal user of night change

	/** \brief Update the Tile at location [row,col]
	 *
	 * This function is called on all adjacent Tiles to one that was recently
	 * changed to signal those Tiles to change their graphics if necessary.
	 * \param row The row of the Tile to be updated
	 * \param col The column of the Tile to be updated
	 */
	void UpdateTile(const unsigned int row, const unsigned int col);

	/** \brief Helper function for pathfinding
	 *
	 * Tries if a particular tile is passable, and put it in queue
	 * \param pathQ		the path finding queue
	 * \param x			tile x index
	 * \param y			tile y index
	 * \param d			the direction
	 * \param currdir	direction of current tile
	*/
	void TryTile(std::queue<pathNode> *pathQ, unsigned int x, unsigned int y,
				 Direction d);
};

inline Charlie *Level::GetCharlie() { return chameleon; }

inline void Level::ChangeDay()
{
	isday = !isday;
	dayremaining = daylength;

	if (isday) {
		SoundManager::StopChunk(night_sound);
		SoundManager::StopMusic(nightmusic);
		SoundManager::PlayChunk(day_sound);
		SoundManager::PlayMusic(daymusic, 100, -1, 0, -1, MUSIC_FADE_TIME);
	} else {
		SoundManager::StopChunk(day_sound);
		SoundManager::StopMusic(daymusic);
		SoundManager::PlayChunk(night_sound);
		SoundManager::PlayMusic(nightmusic, 100, -1, 0, -1, MUSIC_FADE_TIME);
	}
}

inline void Level::MakeItDay()
{
	if(isday)
		dayremaining = daylength;
	else
		dayremaining = 0;
}

inline void Level::MakeItNight()
{
	if(!isday)
		dayremaining = daylength;
	else
		dayremaining = 0;
}

inline const std::vector<std::string>& Level::GetIntroMessage() const
{
	return intromessage;
}

#include "GameObject.hpp"
#include "Tiles.hpp"

#endif /* INCLUDED_BOARD_H */
// vim: set ts=4 sts=4 sw=4 noet:
