/// \file LevelEditor.hpp Header File for LevelEditor class

/// Include guard for LevelEditor.hpp
#ifndef INCLUDED_LEVELEDITOR_H
#define INCLUDED_LEVELEDITOR_H 1

#include "Utility.hpp"
#include "Level.hpp"
#include "Button.hpp"

/**
 * \brief Representation of an instance of the level editor.
 */
class LevelEditor {
public:
	enum EditorState {
		EDITOR_STATE_NEW,	/**< Clear the level editor */
		EDITOR_STATE_RUN,	/**< The level editor is running */
		EDITOR_BOARD_STATE_SELECT,/**< a creature on the board is selected */
		EDITOR_BOARD_STATE_DRAG,  /**< a creature is being dragged */
		EDITOR_BOARD_STATE_WP,	/**< waypoint editing */
		EDITOR_STATE_EXIT	/**< Exit the level editor */
    };

	enum TileTypes {
		TILE_TYPE_CREATURES,
		TILE_TYPE_WALLS,
		TILE_TYPE_SPECIAL
    };
    /** \brief LevelEditor Constructor for editing an existing level.
	 * Read level information from levelfile, if specified, and begin editing
	 * this level.
	 *
	 * \param levelfile The name of a file containing the level to load
	 */
    LevelEditor(const char *const levelfile);

    /** \brief LevelEditor Constructor for creating a new level.
	 * Create a new level with dimension rowsxcols, and begin editing
	 * this level.
	 *
	 * \param rows The number of rows in the new level
	 * \param cols The number of columns in the new level
	 */
    LevelEditor(const int rows = 10, const int cols = 10);

    /** \brief LevelEditor destructor.
	 * Deallocates everything that comprises this level editor instance.
	 */
    ~LevelEditor();

    /** \brief Update the current state of the level editor
	 *
	 * This function checks for button and key presses
	 */
    void Run();

	/** \brief Report a keyboard event (key press or release)
	 * \param event The event to process
	 */
	void KeyboardEvent(SDL_KeyboardEvent *event);

	/** \brief Report a mouse motion event
	 * \param event The event to process
	 */
	void MouseMotionEvent(SDL_MouseMotionEvent *event);

	/** \brief Report a mouse button event (button press or release)
	 * \param event The event to process
	 */
	void MouseButtonEvent(SDL_MouseButtonEvent *event);

    /** \brief Redraw the level editor */
    void Draw(SDL_Surface *screen);

	bool New() { return estate == EDITOR_STATE_NEW; }

	bool Exit() { return estate == EDITOR_STATE_EXIT; }

private:
	Level *level;		/**< All creatures in the level */
	bool showColor;
	ColorRGBA colorSelected;
	bool isDrag;
	GameObject* objSelected;

	EditorState estate;	/**< State of the level editor */
	TileTypes tileCat;	/**< Tiles to display in the ui */
	Button::ButtonType typeSelected;
	TileTypes tileTypeSelected;
	Button Charlie, Bee, Snake, TigerCub, Centipede, Millipede, Bat, Butterfly,
		   Scorpion,
		   Rock, Water, Tree, Bridge, Land,
		   Switch, Door, Flower, Goal;
	Button Black, Blue, Brown, Green, Orange, Red, Violet, White;
	Button NewLvl, LoadLvl, SaveLvl, MainMenu, DayLNext, DayLPrev, StartNext,
		   StartPrev, TypeNext, TypePrev, TimeLimitNext, TimeLimitPrev, Name;
	SDL_Surface *creatures_txt, *walls_txt, *objects_txt, *colors_txt,
				*daylength_txt, *levelstart_txt, *leveltype_txt, *timelimit_txt,
				*day_txt, *night_txt, *goal_txt, *time_txt, *collect_txt,
				*s10s_txt, *s20s_txt, *s30s_txt, *s40s_txt, *s50s_txt,
				*s60s_txt, *m05min_txt, *m1min_txt, *m15min_txt, *m2min_txt,
				*m3min_txt, *m5min_txt;
	std::vector<Button *> creatures, walls, special, colors, menu, arrows;
	std::vector<SDL_Surface *> daylength_v, levelstart_v, leveltype_v, timelimit_v;
	int daylength_idx, levelstart_idx, leveltype_idx, timelimit_idx;
	bool typingName;
	std::string LevelName;

	void SaveLevel();
	void LoadLevel();
	void ClearSelected(); /**< Set other selected tiles to default */
};

#endif /* INCLUDED_LEVELEDITOR_H */
// vim: set ts=4 sts=4 sw=4 noet:
