/// \file LevelParser.cpp Source file for the LevelParser class

#include <sstream>
#include <stdexcept>
#include <cstring>

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>
#include <xercesc/util/OutOfMemoryException.hpp>

#include "LevelParser.hpp"
#include "Utility.hpp"
#include "Bat.hpp"
#include "Bee.hpp"
#include "Butterfly.hpp"
#include "Patroller.hpp"
#include "Snake.hpp"


XERCES_CPP_NAMESPACE_USE

/** \brief Class to facilitate initialization and termination of Xerces */
class XercesInitializerClass {
public:
	/** \brief Initialize Xerces */
	XercesInitializerClass();
	/** \brief Terminate Xerces */
	~XercesInitializerClass();
};

XercesInitializerClass::XercesInitializerClass()
{
	try {
		XMLPlatformUtils::Initialize();
	} catch (const XMLException& e) {
		char *emsg = XMLString::transcode(e.getMessage());
		const char *mmsg = "Error initializing Xerces:\n";
		char *msg = new char[strlen(mmsg)+strlen(emsg)+1];
		strcpy(msg, mmsg);
		strcat(msg, emsg);
		dialogs->ErrorMessage("%s", msg);
		delete[] msg;
		XMLString::release(&emsg);
		exit(EXIT_FAILURE);
	}
}

XercesInitializerClass::~XercesInitializerClass()
{
	XMLPlatformUtils::Terminate();
}

/// This variable causes the initialization and termination of Xerces
// as it is created and destroyed.
XercesInitializerClass initializeit;

LevelParser::LevelParser(Level *l, Vec2i off, Vec2i bs) :
	level(l),
	offset(off),
	boardsize(bs),
	currentobject(NULL),
	parsingmessage(false)
{
}

LevelParser::~LevelParser()
{
}

void LevelParser::handleLevelElement(const Attributes& attrs)
{
	bool setdaylength = false;
	bool setdayremaining = false;

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));
		char *uri = XMLString::transcode(attrs.getURI(i));

		// Ignore xsi attributes
		if (!XMLString::compareIString(uri, "http://www.w3.org/2001/XMLSchema-instance")) {
			XMLString::release(&uri);
			continue;
		}

		if (!XMLString::compareIString(attr, "daylength")) {
			int daylen;
			std::istringstream(attrval) >> daylen;
			if (daylen < 1) {
				errors += "The day length for this level should be > 0\n";
				daylen = 1;
			} else if (daylen > 60*5)
				warnings += "The day length is > 5 minutes.\n";

			level->daylength = daylen*UPDATES_PER_SECOND;
			setdaylength = true;
			if (!setdayremaining) {
				level->initialdayremaining = daylen*UPDATES_PER_SECOND;
				level->dayremaining = level->initialdayremaining;
			}
		} else if (!XMLString::compareIString(attr, "dayremaining")) {
			int dayremaining;
			std::istringstream(attrval) >> dayremaining;
			if (dayremaining > 0) {
				level->isday = true;
				level->initialdayremaining = dayremaining*UPDATES_PER_SECOND;
				level->dayremaining = dayremaining*UPDATES_PER_SECOND;
				setdayremaining = true;
			} else if (dayremaining < 0) {
				level->isday = false;
				level->initialdayremaining = dayremaining*UPDATES_PER_SECOND;
				level->dayremaining = -dayremaining*UPDATES_PER_SECOND;
				setdayremaining = true;
			} // if dayremaining == 0, let daylength handle it
		} else if (!XMLString::compareIString(attr, "victorytype")) {
			if (!XMLString::compareIString(attrval, "goaltile")) {
				level->victorytype = Level::VICTORY_GOALTILE;
			} else if (!XMLString::compareIString(attrval, "survival")) {
				level->victorytype = Level::VICTORY_SURVIVAL;
			} else if (!XMLString::compareIString(attrval, "collection")) {
				level->victorytype = Level::VICTORY_COLLECT;
			} else {
				errors += "Unrecognized victory type \"";
				errors += attrval;
				errors += "\"!\n";
			}
		} else if (!XMLString::compareIString(attr, "survivaltime")) {
			int dayremaining;
			std::istringstream(attrval) >> dayremaining;
			level->survivaltime = dayremaining*UPDATES_PER_SECOND;
			level->survivaltimeremaining = level->survivaltime;
		} else if (!XMLString::compareIString(attr, "name")) {
			level->levelname = attrval;
		} else {
			warnings += "Unrecognized attribute \"";
			warnings += attr;
			warnings += "\" of level element ignored.\n";
		}

		XMLString::release(&attr);
		XMLString::release(&attrval);
		XMLString::release(&uri);
	}
}

void LevelParser::handleMessageElement(const Attributes& attrs)
{
	parsingmessage = true;
	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		if (!XMLString::compareIString(attr, "image")) {
//			level->introimage = LoadImageSDL(attrval);
		} else {
			warnings += "Unrecognized attribute \"";
			warnings += attr;
			warnings += "\" of message element ignored.\n";
		}

		XMLString::release(&attr);
		XMLString::release(&attrval);
	}
}

void LevelParser::handleTilesElement(const Attributes& attrs)
{
	int cols = -1;
	int rows = -1;

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		if (!XMLString::compareIString(attr, "cols")) {
			std::istringstream(attrval) >> cols;
		} else if (!XMLString::compareIString(attr, "rows")) {
			std::istringstream(attrval) >> rows;
		} else {
			warnings += "Unrecognized attribute \"";
			warnings += attr;
			warnings += "\" of tiles element ignored.\n";
		}

		XMLString::release(&attr);
		XMLString::release(&attrval);
	}

	if (rows <= 0) {
		std::stringstream err;
		err << "The \"rows\" attribute of the tiles element must be > 0, " <<
			   "got " << rows << "!\n";
		errors += err.str();
		rows = 1;
	}

	if (cols <= 0) {
		std::stringstream err;
		err << "The \"cols\" attribute of the tiles element must be > 0, " <<
			   "got " << cols << "!\n";
		errors += err.str();
		cols = 1;
	}

	level->Initialize(rows, cols, offset, boardsize.x, boardsize.y);
}

void LevelParser::handleTileElement(const Attributes& attrs)
{
	int row = -1;
	int col = -1;
	bool colorspecified = false;
	ColorRGBA color;
	std::string type;

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		if (!XMLString::compareIString(attr, "row")) {
			std::istringstream(attrval) >> row;
		} else if (!XMLString::compareIString(attr, "col")) {
			std::istringstream(attrval) >> col;
		} else if (!XMLString::compareIString(attr, "color")) {
			colorspecified = true;
			color = parseColor(attrval);
		} else if (!XMLString::compareIString(attr, "type")) {
			/* Already handled by DTD and Schema checking */
			type = attrval;
		} else {
			warnings += "Unrecognized attribute \"";
			warnings += attr;
			warnings += "\" of the tiles element ignored.\n";
		}

		XMLString::release(&attrval);
		XMLString::release(&attr);
	}

	if (row < 0 || row >= static_cast<signed>(level->GetRows())) {
		std::ostringstream err;
		err << "The \"row\" attribute of the tile element must be between "
			   "0 and the number of rows-1 (" << level->GetRows()-1 <<
			   ").  Got " << row << ".\n";
		errors += err.str();
		row = (row<0)?0:level->GetRows()-1;
	}

	if (col < 0 || col >= static_cast<signed int>(level->GetCols())) {
		std::ostringstream err;
		err << "The \"col\" attribute of the tile element must be between "
			   "0 and the number of columns-1 (" << level->GetCols()-1 <<
			   ").  Got " << col << ".\n";
		errors += err.str();
		col = (col<0)?0:level->GetCols()-1;
	}

	if (!XMLString::compareIString(type.c_str(), "bridge")) {
		if (colorspecified)
			level->SetTile(new BridgeTile(col, row, level->GetTileDimension(), color), col, row);
		else
			level->SetTile(new BridgeTile(col, row, level->GetTileDimension()), col, row);
	} else if (!XMLString::compareIString(type.c_str(), "land")) {
		if (colorspecified)
			level->SetTile(new LandTile(col, row, level->GetTileDimension(), color), col, row);
		else
			level->SetTile(new LandTile(col, row, level->GetTileDimension()), col, row);
	} else if (!XMLString::compareIString(type.c_str(), "rock")) {
		if (colorspecified)
			level->SetTile(new RockTile(col, row, level->GetTileDimension(), color), col, row);
		else
			level->SetTile(new RockTile(col, row, level->GetTileDimension()), col, row);
	} else if (!XMLString::compareIString(type.c_str(), "tree")) {
		if (colorspecified)
			level->SetTile(new TreeTile(col, row, level->GetTileDimension(), color), col, row);
		else
			level->SetTile(new TreeTile(col, row, level->GetTileDimension()), col, row);
	} else if (!XMLString::compareIString(type.c_str(), "water")) {
		if (colorspecified)
			level->SetTile(new WaterTile(col, row, level->GetTileDimension(), color), col, row);
		else
			level->SetTile(new WaterTile(col, row, level->GetTileDimension()), col, row);
	} else {
		errors += "Unrecognized type \"" + type + "\"\n";
	}
}

void LevelParser::handleCreaturesElement(const Attributes& attrs)
{
	/* This is a current limitation of the level loading code... */
	if (level->GetRows() == 0)
		errors += "The tiles element should precede the creatures element.";

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		warnings += "Unrecognized attribute \"";
		warnings += attr;
		warnings += "\" of the creatures element ignored.\n";

		XMLString::release(&attr);
		XMLString::release(&attrval);
	}
}

void LevelParser::handleCreatureElement(const Attributes& attrs)
{
	float x = -1.0f;
	float y = -1.0f;
	std::string type;
	ColorRGBA color;

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		if (!XMLString::compareIString(attr, "x")) {
			std::istringstream(attrval) >> x;
		} else if (!XMLString::compareIString(attr, "y")) {
			std::istringstream(attrval) >> y;
		} else if (!XMLString::compareIString(attr, "type")) {
			/* Mostly handled by Schema and DTD checking */
			type = attrval;
		} else if (!XMLString::compareIString(attr, "color")) {
			color = parseColor(attrval);
		} else {
			warnings += "Unrecognized attribute \"";
			warnings += attr;
			warnings += "\" of the creature element ignored.\n";
		}

		XMLString::release(&attr);
		XMLString::release(&attrval);
	}

	if (x < 0.0f || x >= level->GetRows()) {
		std::ostringstream err;
		err << "The \"x\" attribute of the creature element must be between "
			   "0 and the number of rows-1 (" << level->GetRows()-1 <<
			   ").  Got " << x << ".\n";
		errors += err.str();
		x = (x<0.0f)?0.0f:level->GetRows()-1;
	}

	if (y < 0.0f || y >= level->GetCols()) {
		std::ostringstream err;
		err << "The \"y\" attribute of the creature element must be between "
			   "0 and the number of columns-1 (" << level->GetCols()-1 <<
			   ").  Got " << y << ".\n";
		errors += err.str();
		y = (y<0.0f)?0.0f:level->GetCols()-1;
	}

	Vec2i tiledim = level->GetTileDimension();
	if (!XMLString::compareIString(type.c_str(), "bat")) {
		currentobject = new Bat(level, Vec2f(x,y));
		level->AddCreature(currentobject);
	} else if (!XMLString::compareIString(type.c_str(), "bee")) {
		currentobject = new Bee(level, Vec2f(x, y), color);
		level->AddCreature(currentobject);
	} else if (!XMLString::compareIString(type.c_str(), "butterfly")) {
		currentobject = new Butterfly(level, Vec2f(x,y), color == COLOR_ORANGE);
		level->AddCreature(currentobject);
	} else if (!XMLString::compareIString(type.c_str(), "centipede")) {
		currentobject = new Centipede(level, Vec2f(x,y));
		level->AddCreature(currentobject);
	} else if (!XMLString::compareIString(type.c_str(), "chameleon")) {
		currentobject = new Charlie(level, Vec2f(x,y), color);
		level->AddCreature(currentobject);
	} else if (!XMLString::compareIString(type.c_str(), "millipede")) {
		currentobject = new Millipede(level, Vec2f(x,y));
		level->AddCreature(currentobject);
	} else if (!XMLString::compareIString(type.c_str(), "scorpion")) {
		currentobject = new Scorpion(level, Vec2f(x,y));
		level->AddCreature(currentobject);
	} else if (!XMLString::compareIString(type.c_str(), "snake")) {
		currentobject = new Snake(level, Vec2f(x,y), color);
		level->AddCreature(currentobject);
	} else if (!XMLString::compareIString(type.c_str(), "tigercub")) {
		currentobject = new TigerCub(level, Vec2f(x,y));
		level->AddCreature(currentobject);
	} else {
		errors += "Unrecognized type \"" + type + "\"\n";
	}
}

void LevelParser::handleWaypointElement(const Attributes& attrs)
{
	float x = -1.0f;
	float y = -1.0f;
	bool duringday = true;
	bool duringnight = true;

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		if (!XMLString::compareIString(attr, "x")) {
			std::istringstream(attrval) >> x;
		} else if (!XMLString::compareIString(attr, "y")) {
			std::istringstream(attrval) >> y;
		} else if (!XMLString::compareIString(attr, "during")) {
			if (!XMLString::compareIString(attrval, "day"))
				duringnight = false;
			else if (!XMLString::compareIString(attrval, "night"))
				duringday = false;
			else if (!XMLString::compareIString(attrval, "neither"))
				duringday = duringnight = false;
		} else {
			warnings += "Unrecognized attribute \"";
			warnings += attr;
			warnings += "\" of waypoint element ignored.\n";
		}

		XMLString::release(&attr);
		XMLString::release(&attrval);
	}

	if (x < 0.0f || x >= level->GetRows()) {
		std::ostringstream err;
		err << "The \"x\" attribute of the waypoint element must be between "
			   "0 and the number of rows-1 (" << level->GetRows()-1 <<
			   ").  Got " << x << ".\n";
		errors += err.str();
		x = (x<0.0f)?0.0f:level->GetRows()-1;
	}

	if (y < 0.0f || y >= level->GetCols()) {
		std::ostringstream err;
		err << "The \"y\" attribute of the waypoint element must be between "
			   "0 and the number of columns-1 (" << level->GetCols()-1 <<
			   ").  Got " << y << ".\n";
		errors += err.str();
		y = (y<0.0f)?0.0f:level->GetCols()-1;
	}

	Vec2i tiledim = level->GetTileDimension();
	Patroller *p = dynamic_cast<Patroller *>(currentobject);
	if (!currentobject)
		errors += "Error, waypoint encountered with no current creature!\n";
	else if (p)
		p->AddWaypoint(Vec2f(x,y), duringday, duringnight);
	else
		warnings += "Waypoint encountered for non-patroller creature.\n";
}

void LevelParser::handleObjectsElement(const Attributes& attrs)
{
	/* This is a current limitation of the level loading code... */
	if (level->GetRows() == 0)
		errors += "The tiles element should precede the creatures element.";

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		warnings += "Unrecognized attribute \"";
		warnings += attr;
		warnings += "\" of the objects element ignored.\n";

		XMLString::release(&attr);
		XMLString::release(&attrval);
	}
}

void LevelParser::handleObjectElement(const Attributes& attrs)
{
	float x = -1.0f;
	float y = -1.0f;
	bool toggleswitch = true;
	bool dayopen = true;
	bool nightopen = true;
	int group = 0;
	std::string type;
	ColorRGBA color;

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		if (!XMLString::compareIString(attr, "x")) {
			std::istringstream(attrval) >> x;
		} else if (!XMLString::compareIString(attr, "y")) {
			std::istringstream(attrval) >> y;
		} else if (!XMLString::compareIString(attr, "type")) {
			type = attrval;
		} else if (!XMLString::compareIString(attr, "color")) {
			color = parseColor(attrval);
		} else if (!XMLString::compareIString(attr, "group")) {
			std::istringstream(attrval) >> group;
			if (group <= 0) {
				errors += "Group attribute must be >= 1!\n";
				group = 0;
			}
		} else if (!XMLString::compareIString(attr, "switch")) {
			if (!XMLString::compareIString(attrval, "button"))
				toggleswitch = false;
			else
				toggleswitch = true;
		} else if (!XMLString::compareIString(attr, "open")) {
			if (!XMLString::compareIString(attrval, "neither")) {
				dayopen = false;
				nightopen = false;
			} else if (!XMLString::compareIString(attrval, "day")) {
				dayopen = true;
				nightopen = false;
			} else if (!XMLString::compareIString(attrval, "night")) {
				dayopen = false;
				nightopen = true;
			} else {
				dayopen = true;
				nightopen = true;
			}
		} else {
			warnings += "Unrecognized attribute \"";
			warnings += attr;
			warnings += "\" of object element ignored.\n";
		}

		XMLString::release(&attr);
		XMLString::release(&attrval);
	}

	if (x < 0.0f || x >= level->GetRows()) {
		std::ostringstream err;
		err << "The \"x\" attribute of the object element must be between "
			   "0 and the number of rows-1 (" << level->GetRows()-1 <<
			   ").  Got " << x << ".\n";
		errors += err.str();
		x = (x<0.0f)?0.0f:level->GetRows()-1;
	}

	if (y < 0.0f || y >= level->GetCols()) {
		std::ostringstream err;
		err << "The \"y\" attribute of the objectelement must be between "
			   "0 and the number of columns-1 (" << level->GetCols()-1 <<
			   ").  Got " << y << ".\n";
		errors += err.str();
		y = (y<0.0f)?0.0f:level->GetCols()-1;
	}

	Vec2i tiledim = level->GetTileDimension();
	if (group >= 1) {
		if (!XMLString::compareIString(type.c_str(), "switch")) {
			if (switches.size() <= static_cast<unsigned>(group))
				switches.resize(group+1);
			Switch *s = new Switch(level, Vec2f(x,y), toggleswitch, group);
			currentobject = s;
			level->AddObject(s);
			switches[group].push_back(s);
		} else if (!XMLString::compareIString(type.c_str(), "flower")) {
			if (switchables.size() <= static_cast<unsigned>(group))
				switchables.resize(group+1);
			Flower *f = new Flower(level, Vec2f(x, y), color, dayopen, nightopen, group);
			level->AddObject(f);
			currentobject = f;
			switchables[group].push_back(f);
		} else if (!XMLString::compareIString(type.c_str(), "leafdoor")) {
			if (switchables.size() <= static_cast<unsigned>(group))
				switchables.resize(group+1);
			Door *d = new Door(level, Vec2f(x, y), group);
			level->AddObject(d);
			currentobject = d;
			switchables[group].push_back(d);
		} else {
			errors += "Error, ";
			errors += type;
			errors += " is not switchable, so it can not have an associated group!\n";
		}
	} else {
		if (!XMLString::compareIString(type.c_str(), "flower")) {
			currentobject = new Flower(level, Vec2f(x,y),
									   color, dayopen, nightopen);
			level->AddObject(currentobject);
		} else if (!XMLString::compareIString(type.c_str(), "goal")) {
			currentobject = new Goal(level, Vec2f(x,y));
			level->AddObject(currentobject);
		} else if (!XMLString::compareIString(type.c_str(), "switch")) {
			currentobject = new Switch(level, Vec2f(x, y),
									   toggleswitch);
			level->AddObject(currentobject);
		} else if (!XMLString::compareIString(type.c_str(), "leafdoor")) {
			currentobject = new Door(level, Vec2f(x, y));
			level->AddObject(currentobject);
		} else {
			errors += "Error, unrecognized object of type \"";
			errors += type;
			errors += "\" encountered!\n";
		}
	}
}

ColorRGBA LevelParser::parseColor(const char *color)
{
	if (!XMLString::compareIString(color, "black")) {
		return COLOR_BLACK;
	} else if (!XMLString::compareIString(color, "blue")) {
		return COLOR_BLUE;
	} else if (!XMLString::compareIString(color, "brown")) {
		return COLOR_BROWN;
	} else if (!XMLString::compareIString(color, "green")) {
		return COLOR_GREEN;
	} else if (!XMLString::compareIString(color, "orange")) {
		return COLOR_ORANGE;
	} else if (!XMLString::compareIString(color, "red")) {
		return COLOR_RED;
	} else if (!XMLString::compareIString(color, "violet")) {
		return COLOR_VIOLET;
	} else if (!XMLString::compareIString(color, "white")) {
		return COLOR_WHITE;
	} else if (!XMLString::compareIString(color, "yellow")) {
		return COLOR_YELLOW;
	} else {
		std::ostringstream err;
		err << "Color \"" << color << "\" not recognized.\n";
		errors += err.str();
		return COLOR_BLACK;
	}
}

void LevelParser::startElement(const XMLCh* const uri,
										 const XMLCh* const localname,
										 const XMLCh* const qname,
										 const Attributes& attrs)
{
	char *element = XMLString::transcode(localname);
	if (!XMLString::compareIString(element, "level")) {
		handleLevelElement(attrs);
	} else if (!XMLString::compareIString(element, "message")) {
		handleMessageElement(attrs);
	} else if (!XMLString::compareIString(element, "tiles")) {
		handleTilesElement(attrs);
	} else if (!XMLString::compareIString(element, "tile")) {
		handleTileElement(attrs);
	} else if (!XMLString::compareIString(element, "creatures")) {
		handleCreaturesElement(attrs);
	} else if (!XMLString::compareIString(element, "creature")) {
		handleCreatureElement(attrs);
	} else if (!XMLString::compareIString(element, "waypoint")) {
		handleWaypointElement(attrs);
	} else if (!XMLString::compareIString(element, "objects")) {
		handleObjectsElement(attrs);
	} else if (!XMLString::compareIString(element, "object")) {
		handleObjectElement(attrs);
	} else {
		warnings += "Unrecognized element \"";
		warnings += element;
		warnings += "\" ignored.\n";
	}

	XMLString::release(&element);
}

void LevelParser::endElement(const XMLCh* const uri,
							 const XMLCh* const localname,
							 const XMLCh* const qname)
{
	char *element = XMLString::transcode(localname);
	if (!XMLString::compareIString(element, "message")) {
		parsingmessage = false;
	}

	XMLString::release(&element);
}

void LevelParser::endDocument()
{
	if (switches.size() == 0 && switchables.size() == 0)
		return;

	if (switches.size() != switchables.size())
		errors += "There is not a switch or a switchable object for every group!\n";

	for (unsigned int i=0; i<switches.size() && i<switchables.size(); i++) {
		if (switches[i].size() == 0 && switches[i].size() != 0) {
			std::ostringstream err;
			err << "Group " << i << " does not have any switches!\n";
			errors += err.str();
		} else if (switches[i].size() != 0 && switchables[i].size() == 0) {
			std::ostringstream err;
			err << "Group " << i << " does not have any switchable objects!\n";
			errors += err.str();
		}

		for (unsigned int j=0; j<switches[i].size(); j++)
			for (unsigned int k=0; k<switchables[i].size(); k++)
				switches[i][j]->AddSwitchable(switchables[i][k]);
	}
}

void LevelParser::characters(const XMLCh* const chars,
							 const unsigned int length)
{
	if (!parsingmessage)
		return;

	char *mess = XMLString::transcode(chars);
	std::string message = mess;

	std::string::size_type start = 0;
	std::string::size_type end = 0;
	while (end != std::string::npos) {
		end = message.find('\n', start);
		level->intromessage.push_back(message.substr(start, end-start));
		start = end + 1;
	}
	XMLString::release(&mess);
}

void LevelParser::ignorableWhitespace(const XMLCh* const chars,
									  const unsigned int length)
{
}

void LevelParser::resetDocument()
{
}

void LevelParser::warning(const SAXParseException& e)
{
	char *emsg = XMLString::transcode(e.getMessage());
	warnings += emsg;
	XMLString::release(&emsg);
}

void LevelParser::error(const SAXParseException& e)
{
	char *emsg = XMLString::transcode(e.getMessage());
	const char *mmsg = "Error Parsing Level File:\n";
	char *msg = new char[strlen(mmsg)+strlen(emsg)+1];
	strcpy(msg, mmsg);
	strcat(msg, emsg);
	dialogs->ErrorMessage("%s", msg);
	delete[] msg;
	XMLString::release(&emsg);
}

void LevelParser::fatalError(const SAXParseException& e)
{
	char *emsg = XMLString::transcode(e.getMessage());
	const char *mmsg = "Fatal Error Parsing Level File:\n";
	char *msg = new char[strlen(mmsg)+strlen(emsg)+1];
	strcpy(msg, mmsg);
	strcat(msg, emsg);
	dialogs->ErrorMessage("%s", msg);
	delete[] msg;
	XMLString::release(&emsg);
}

void LevelParser::resetErrors()
{
}

void LevelParser::parseLevelFile(const char *levelfile, Level *level,
								 Vec2i off, Vec2i tiledim)
{
	SAX2XMLReader* parser = XMLReaderFactory::createXMLReader();

	// For debugging, turn on DTD validation and full Schema checking
	parser->setFeature(XMLUni::fgSAX2CoreValidation, true);
	parser->setFeature(XMLUni::fgXercesSchema, true);
	parser->setFeature(XMLUni::fgXercesSchemaFullChecking, true);

	LevelParser handler(level, off, tiledim);
    parser->setContentHandler(&handler);
	parser->setErrorHandler(&handler);

	try {
		parser->parse(levelfile);
	} catch (const OutOfMemoryException&) {
		throw std::runtime_error("Out of Memory");
	} catch (const XMLException& e) {
		char *emsg = XMLString::transcode(e.getMessage());
		std::string errormsg = emsg;
		XMLString::release(&emsg);
		throw std::runtime_error(errormsg);
	} catch (...) {
		throw std::runtime_error("Unexpected exception during level parsing.");
	}

	delete parser;

	if (handler.errors.size() > 0)
		throw std::runtime_error(handler.errors);

	if (handler.warnings.size() > 0)
		dialogs->WarningMessage("%s", handler.warnings.c_str());
}

// vim: set ts=4 sts=4 sw=4 noet:
