/// \file LevelParser.hpp Header file for the level parser

/// Include guard for LevelParser.hpp
#ifndef INCLUDED_LEVELPARSER_H
#define INCLUDED_LEVELPARSER_H 1

#include <xercesc/sax2/Attributes.hpp>
#include <xercesc/sax2/DefaultHandler.hpp>

#include "Utility.hpp"

class Level;
class GameObject;
class Switch;
class Switchable;

XERCES_CPP_NAMESPACE_USE

/**
 * \brief Class used to parse an XML level file and initialize a Level
 */
class LevelParser : public DefaultHandler
{
public:
	/** \brief Construct and initialize a LevelParser
	 * \param level The Level to be initialized from the level data
	 */
	LevelParser(Level *level, Vec2i off, Vec2i boardsize);
	/** \brief Destroy a LevelParser */
	~LevelParser();

	/** \name SAXContentHandlers
	 * Handlers for the SAX ContentHandler interface
	 */
	//@{
	/** \brief Called to signal the start of an XML element */
    void startElement(const XMLCh* const uri, const XMLCh* const localname,
					  const XMLCh* const qname, const Attributes& attrs);
	/** \brief Called to signal the end of an XML element */
	void endElement(const XMLCh* const uri, const XMLCh* const localname,
					const XMLCh *const qname);
	/** \brief Called for characters within an XML element */
    void characters(const XMLCh* const chars, const unsigned int length);
	/** \brief Called for ignorable whitespace within an XML element */
    void ignorableWhitespace(const XMLCh* const chars, const unsigned int length);
	/** \brief Called to reset the handler so it can be reused */
    void resetDocument();
	/** \brief Called to signal the end of the document being parsed */
	void endDocument();
	//@}

	/** \name SAXErrorHandlers
	 * Handlers for the SAX ErrorHandler interface
	 */
	//@{
	/** \brief Called to indicate a warning condition during parsing */
	void warning(const SAXParseException& exc);
	/** \brief Called to indicate an error condition during parsing */
    void error(const SAXParseException& exc);
	/** \brief Called to indicate a fatal error condition during parsing */
    void fatalError(const SAXParseException& exc);
	/** \brief Called to reset the handler, so it may be reused */
    void resetErrors();
	//@}

	/** \brief Parse a level file and use it to initialize level
	 *
	 * \param levelfile The name of the level file to parse
	 * \param level The Level to be initialized
	 * \return 0 for success, <0 on error
	 */
	static void parseLevelFile(const char *levelfile, Level *level, Vec2i off = Vec2i(192,128), Vec2i boardsize = Vec2i(1280,1280));

private:
	/** \name InternalElementHandlers
	 * Internal Element Handlers
	 */
	//@{
	/// \brief Handle a level element
	void handleLevelElement(const Attributes& attrs);
	/// \brief Handle a message element
	void handleMessageElement(const Attributes& attrs);
	/// \brief Handle a tiles element
	void handleTilesElement(const Attributes& attrs);
	/// \brief Handle a tile element
	void handleTileElement(const Attributes& attrs);
	/// \brief Handle a creatures element
	void handleCreaturesElement(const Attributes& attrs);
	/// \brief Handle a creature element
	void handleCreatureElement(const Attributes& attrs);
	/// \brief Handle a waypoint element
	void handleWaypointElement(const Attributes& attrs);
	/// \brief Handle a features element
	void handleObjectsElement(const Attributes& attrs);
	/// \brief Handle a feature element
	void handleObjectElement(const Attributes& attrs);
	//@}

	/** \brief Parse a string representing a color
	 * \param color A string representing a color (e.g. "orange")
	 * \return The ColorRGBA equivalent of the string
	 */
	ColorRGBA parseColor(const char *color);

	Level *level;	///< The level being initialized
	GameObject *currentobject;	///< The object currently being initialized
	bool parsingmessage;	///< true while parsing the infomessage
	Vec2i offset;		///< offset of the level viewport being drawn
	Vec2i boardsize;	///< size of the board to be drawn

	/// The Switch members of each switch group
	std::vector<std::vector<Switch*> > switches;
	/// The Switchable members of each switch group
	std::vector<std::vector<Switchable*> > switchables;

	std::string errors;		///< Error messages accumulated during parsing

	std::string warnings;	///< Warning messages accumulated during parsing
};

#include "Level.hpp"
#include "GameObject.hpp"
#include "Switch.hpp"
#include "Switchable.hpp"

#endif /* INCLUDED_LEVELPARSER_H */
// vim: set ts=4 sts=4 sw=4 noet:
