/// \file LevelUI.cpp Source file for the LevelUI class

#include <vector>
#include <string>
#include <iostream>
#include <fstream>

#include "LevelParser.hpp"
#include "LevelWriter.hpp"
#include "LevelUI.hpp"

static const Uint8 NIGHT_SHADING_ALPHA = 150;
static const Uint8 MENU_SHADING_ALPHA = 200;

LevelUI::LevelUI(const char *levelfile) :
	state(GAME_STATE_INTRO),
	introLength(360),
	Resume (screensize.x*1/3,screensize.y*9/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/ingame/resume.png"),
	Restart (screensize.x*1/3,screensize.y*12/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/ingame/restart.png"),
	RestartIntro (screensize.x*1/3,screensize.y*15/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/mainmenu/restart_intro.png"),
	Exit (screensize.x*1/3,screensize.y*18/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/mainmenu/backtomain.png"),
	RestartGO (screensize.x*1/3,screensize.y*10/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/ingame/restart.png"),
	RestartGOIntro (screensize.x*1/3,screensize.y*14/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/mainmenu/restart_intro.png"),
	ExitGO (screensize.x*1/3,screensize.y*18/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/mainmenu/backtomain.png"),
	NextLC (screensize.x*1/3,screensize.y*12/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/ingame/next_level.png"),
	RestartLC (screensize.x*1/3,screensize.y*16/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/ingame/restart.png")
{
	int boardsize = min(screensize.x, 7*screensize.y/8);

	LevelParser::parseLevelFile(levelfile, &level,
								Vec2i((screensize.x-boardsize)/2, boardsize/7),
								Vec2i(boardsize, boardsize));

	// Compensate for changes based on tile sizes (rounding)
	boardsize = level.GetDimension().x;
	level.SetOffset((screensize.x-boardsize)/2, screensize.y - boardsize);

	paused = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/paused.png");
	level_complete = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/level_complete.png");
	game_over = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/game_over.png");

	int sceneheight = screensize.y - boardsize + 1;
	day_sky = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/day_sky.png", boardsize, sceneheight);
	night_sky = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/night_sky.png", boardsize, sceneheight);
	day_scene = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/day_scene.png", boardsize, sceneheight);
	night_scene = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/night_scene.png", boardsize, sceneheight);

	sun = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/sun.png", boardsize/25, boardsize/25);
	moon = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/moon.png", boardsize/25, boardsize/25);
//	water = LoadImageSDL(PKGDATADIR"Artwork/tiles/lake/water.png");
	cloud_cover = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/clouds/cloud_cover1.png");
	cloud = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/clouds/cloud1.png");

	int wingsize = (screensize.x-boardsize)/2 + 1;
	sidecovertl = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/sidecovertl.png", wingsize, screensize.y/2);
	sidecovertr = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/sidecovertr.png", wingsize, screensize.y/2);
	sidecoverbl = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/sidecoverbl.png", wingsize, screensize.y/2+1);
	sidecoverbr = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/sidecoverbr.png", wingsize, screensize.y/2+1);

	SDL_Surface *screen = SDL_GetVideoSurface();
	black_shading = SDL_CreateRGBSurface(screen->flags, boardsize, boardsize,
										 screen->format->BitsPerPixel,
										 screen->format->Rmask,
										 screen->format->Gmask,
										 screen->format->Bmask,
										 screen->format->Amask);
	SDL_FillRect(black_shading, NULL, SDL_MapRGB(black_shading->format,
												 0, 0, 0));

	SDL_Color textcolor;
	textcolor.r = 25;
	textcolor.g = 200;
	textcolor.b = 50;
	switch (level.GetVictoryType()) {
		case Level::VICTORY_GOALTILE:
			statustext = TTF_RenderText_Solid(gamefont, "Catch the Butterfly!",
											  textcolor);
			break;
		case Level::VICTORY_SURVIVAL:
			statustext = TTF_RenderText_Solid(gamefont, "Time Remaining: ",
											  textcolor);
			break;
		case Level::VICTORY_COLLECT:
			statustext = TTF_RenderText_Solid(gamefont,
											  "Butterflies Left: ",
											  textcolor);
	}

	victory_theme = LoadSound(PKGDATADIR"Sounds/Geuka-VictoryShort.wav");
	defeat_theme = LoadSound(PKGDATADIR"Sounds/Geuka-DefeatShort.wav");
	restart_theme = LoadSound(PKGDATADIR"Sounds/RestartChant.ogg");

	InGameMenus.push_back(&Resume);
	InGameMenus.push_back(&Restart);
	InGameMenus.push_back(&RestartIntro);
	InGameMenus.push_back(&Exit);

	GameOverMenus.push_back(&RestartGO);
	GameOverMenus.push_back(&RestartGOIntro);
	GameOverMenus.push_back(&ExitGO);

	LevelCompleteMenus.push_back(&NextLC);
	LevelCompleteMenus.push_back(&RestartLC);

	level.StartMusic();
}

LevelUI::LevelUI(const int rows, const int cols) :
	state(GAME_STATE_INTRO),
	introLength(360),
	Resume (screensize.x*1/3,screensize.y*9/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/ingame/resume.png"),
	Restart (screensize.x*1/3,screensize.y*12/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/ingame/restart.png"),
	RestartIntro (screensize.x*1/3,screensize.y*15/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/mainmenu/restart_intro.png"),
	Exit (screensize.x*1/3,screensize.y*18/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/mainmenu/backtomain.png"),
	RestartGO (screensize.x*1/3,screensize.y*10/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/ingame/restart.png"),
	RestartGOIntro (screensize.x*1/3,screensize.y*14/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/mainmenu/restart_intro.png"),
	ExitGO (screensize.x*1/3,screensize.y*18/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/mainmenu/backtomain.png"),
	NextLC (screensize.x*1/3,screensize.y*12/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/ingame/next_level.png"),
	RestartLC (screensize.x*1/3,screensize.y*16/25,screensize.x*1/12,screensize.x*1/3,"",PKGDATADIR"Artwork/ui/ingame/restart.png")
{
	int boardsize = min(screensize.x, 7*screensize.y/8);
	int wingsize = (screensize.x-boardsize)/2;

	level.Initialize(rows, cols,
					 Vec2i(wingsize, boardsize/7),
					 boardsize, boardsize);

	paused = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/paused.png");
	level_complete = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/level_complete.png");
	game_over = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/game_over.png");
	day_sky = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/day_sky.png", boardsize, boardsize/7);
	night_sky = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/night_sky.png", boardsize, boardsize/7);
	day_scene = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/day_scene.png", boardsize, boardsize/7);
	night_scene = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/night_scene.png", boardsize, boardsize/7);
	sun = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/sun.png", boardsize/27, boardsize/27);
	moon = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/moon.png", boardsize/27, boardsize/27);
//	water = LoadImageSDL(PKGDATADIR"Artwork/tiles/lake/water.png");
	cloud_cover = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/clouds/cloud_cover1.png");
	cloud = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/clouds/cloud1.png");
	sidecovertl = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/sidecovertl.png", wingsize, screensize.y/2);
	sidecovertr = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/sidecovertr.png", wingsize, screensize.y/2);
	sidecoverbl = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/sidecoverbl.png", wingsize, screensize.y/2);
	sidecoverbr = LoadImageSDL(PKGDATADIR"Artwork/ui/ingame/sidecoverbr.png", wingsize, screensize.y/2);

	SDL_Surface *screen = SDL_GetVideoSurface();
	black_shading = SDL_CreateRGBSurface(screen->flags, boardsize, boardsize,
										 screen->format->BitsPerPixel,
										 screen->format->Rmask,
										 screen->format->Gmask,
										 screen->format->Bmask,
										 screen->format->Amask);
	SDL_FillRect(black_shading, NULL, SDL_MapRGB(black_shading->format,
												 0, 0, 0));

	SDL_Color textcolor;
	textcolor.r = 25;
	textcolor.g = 200;
	textcolor.b = 50;
	switch (level.GetVictoryType()) {
		case Level::VICTORY_GOALTILE:
			statustext = TTF_RenderText_Solid(gamefont, "Catch the Butterfly!",
											  textcolor);
			break;
		case Level::VICTORY_SURVIVAL:
			statustext = TTF_RenderText_Solid(gamefont, "Time Remaining: ",
											  textcolor);
			break;
		case Level::VICTORY_COLLECT:
			statustext = TTF_RenderText_Solid(gamefont,
											  "Butterflies Remaining: ",
											  textcolor);
	}

	victory_theme = LoadSound(PKGDATADIR"Sounds/Geuka-VictoryShort.wav");
	defeat_theme = LoadSound(PKGDATADIR"Sounds/Geuka-DefeatShort.wav");
	restart_theme = LoadSound(PKGDATADIR"Sounds/Pickup_HolyCross.wav");

	InGameMenus.push_back(&Resume);
	InGameMenus.push_back(&Restart);
	InGameMenus.push_back(&RestartIntro);
	InGameMenus.push_back(&Exit);

	GameOverMenus.push_back(&RestartGO);
	GameOverMenus.push_back(&RestartGOIntro);
	GameOverMenus.push_back(&ExitGO);

	LevelCompleteMenus.push_back(&NextLC);
	LevelCompleteMenus.push_back(&RestartLC);

	level.StartMusic();
}

LevelUI::~LevelUI()
{
	SDL_FreeSurface(paused);
	SDL_FreeSurface(level_complete);
	SDL_FreeSurface(game_over);
	SDL_FreeSurface(day_sky);
	SDL_FreeSurface(night_sky);
	SDL_FreeSurface(day_scene);
	SDL_FreeSurface(night_scene);
	SDL_FreeSurface(sun);
	SDL_FreeSurface(moon);
//	SDL_FreeSurface(water);
	SDL_FreeSurface(black_shading);
	SDL_FreeSurface(cloud_cover);
	SDL_FreeSurface(cloud);
	SDL_FreeSurface(sidecovertl);
	SDL_FreeSurface(sidecovertr);
	SDL_FreeSurface(sidecoverbl);
	SDL_FreeSurface(sidecoverbr);
	SDL_FreeSurface(statustext);

	SoundManager::StopChunk(victory_theme);
	SoundManager::StopChunk(defeat_theme);
	SoundManager::StopChunk(restart_theme);

	Mix_FreeChunk(victory_theme);
	Mix_FreeChunk(defeat_theme);
	Mix_FreeChunk(restart_theme);
}

void LevelUI::WriteLevelUI(const char *levelfile)
{
	LevelWriter::writeLevelFile(levelfile, &level);
}

void LevelUI::Reset()
{
	SoundManager::PlayChunk(restart_theme, 100, 0, 0, -1, 20);

	level.Reset();
}

void LevelUI::Pause()
{
	state = GAME_STATE_PAUSED;
	SoundManager::SetMasterVolume(25);
}

void LevelUI::Unpause()
{
	state = GAME_STATE_RUNNING;
	SoundManager::SetMasterVolume(100);
}

void LevelUI::Update()
{
	switch(state){
		case GAME_STATE_RUNNING:
			level.Run();
			CreateCloud();

			if (level.IsComplete()) {
				state = GAME_STATE_WIN;
				SoundManager::PlayChunk(victory_theme, 100, 0, 0, -1, 20);
			} else if (level.HasFailed()) {
				state = GAME_STATE_OVER;
				SoundManager::PlayChunk(defeat_theme, 100, 0, 0, -1, 20);
			}

			break;
	}
}

void LevelUI::KeyboardEvent(SDL_KeyboardEvent *event)
{
	level.KeyboardEvent(event);

	if (event->type != SDL_KEYDOWN)
		return;

	switch (state) {
		case GAME_STATE_INTRO:
			Unpause();
			break;
		case GAME_STATE_RUNNING:
			switch (event->keysym.sym) {
				case SDLK_ESCAPE:
					Pause();
					break;
				case SDLK_i:
					state = GAME_STATE_INTRO;
					break;
				case SDLK_q:
					state = GAME_STATE_QUIT;
					break;
				default:
					// No other hotkeys
					break;
			}
			break;
		case GAME_STATE_OVER:
			switch (event->keysym.sym) {
				case SDLK_m:
					state = GAME_STATE_QUIT;
					break;
				case SDLK_r:
					Reset();
					state = GAME_STATE_RUNNING;
					break;
				case SDLK_i:
					level.ReplayIntro();
					state = GAME_STATE_RUNNING;
					break;
				default:
					// No other hotkeys
					break;
			}
			break;
		case GAME_STATE_PAUSED:
			switch (event->keysym.sym) {
				case SDLK_ESCAPE:
					Unpause();
					break;
				case SDLK_r:
					Reset();
					state = GAME_STATE_RUNNING;
					break;
				case SDLK_m:
					state = GAME_STATE_QUIT;
					break;
				case SDLK_i:
					level.ReplayIntro();
					state = GAME_STATE_RUNNING;
					break;
				default:
					// No other hotkeys
					break;
			}
			break;
		case GAME_STATE_WIN:
		case GAME_STATE_FINISHED:
			switch (event->keysym.sym) {
				case SDLK_n:
					state = GAME_STATE_FINISHED;
					break;
				case SDLK_r:
					Reset();
					state = GAME_STATE_RUNNING;
					break;
				default:
					// No other hotkeys
					break;
			}
			break;
	}
}

void LevelUI::MouseMotionEvent(SDL_MouseMotionEvent *event)
{
	if (state == GAME_STATE_RUNNING)
		level.MouseMotionEvent(event);

	switch (state) {
		case GAME_STATE_OVER:
			for (unsigned int i=0; i<GameOverMenus.size(); i++) {
				if (GameOverMenus[i]->OnButton(event->x, event->y)) {
					GameOverMenus[i]->SetState(Button::BUTTON_STATE_HOVER);
				} else {
					GameOverMenus[i]->SetState(Button::BUTTON_STATE_DEFAULT);
				}
			}
			break;
		case GAME_STATE_PAUSED:
			for (unsigned int i=0; i<InGameMenus.size(); i++) {
				if (InGameMenus[i]->OnButton(event->x, event->y))
					InGameMenus[i]->SetState(Button::BUTTON_STATE_HOVER);
				else
					InGameMenus[i]->SetState(Button::BUTTON_STATE_DEFAULT);
			}
			break;
		case GAME_STATE_WIN:
		case GAME_STATE_FINISHED:
			for (unsigned int i=0; i<LevelCompleteMenus.size(); i++) {
				if (LevelCompleteMenus[i]->OnButton(event->x, event->y))
					LevelCompleteMenus[i]->SetState(Button::BUTTON_STATE_HOVER);
				else
					LevelCompleteMenus[i]->SetState(Button::BUTTON_STATE_DEFAULT);
			}
			break;
	}
}

void LevelUI::MouseButtonEvent(SDL_MouseButtonEvent *event)
{
	if (state == GAME_STATE_RUNNING)
		level.MouseButtonEvent(event);

	if (event->type != SDL_MOUSEBUTTONDOWN)
		return;

	switch (state) {
		case GAME_STATE_INTRO:
			Unpause();
			break;
		case GAME_STATE_OVER:
			for (unsigned int i=0; i<GameOverMenus.size(); i++) {
				if (GameOverMenus[i]->OnButton(event->x, event->y)) {
					GameOverMenus[i]->SetState(Button::BUTTON_STATE_SELECTED);

					if (GameOverMenus[i] == &RestartGO) {
						Reset();
						state = GAME_STATE_RUNNING;
					} else if (GameOverMenus[i] == &RestartGOIntro) {
						level.ReplayIntro();
						state = GAME_STATE_RUNNING;
					} else if (GameOverMenus[i] == &ExitGO) {
						state = GAME_STATE_QUIT;
					}
#ifdef DEBUG
					else
						std::cerr << "Unrecognized GameOver Menu button\n";
#endif
				}
			}
			break;
		case GAME_STATE_PAUSED:
			for (unsigned int i=0; i<InGameMenus.size(); i++) {
				if (InGameMenus[i]->OnButton(event->x, event->y)) {
					InGameMenus[i]->SetState(Button::BUTTON_STATE_SELECTED);

					if (InGameMenus[i] == &Resume) {
						Unpause();
					} else if (InGameMenus[i] == &Restart) {
						Reset();
						state = GAME_STATE_RUNNING;
					} else if (InGameMenus[i] == &RestartIntro) {
						level.ReplayIntro();
						state = GAME_STATE_RUNNING;
					} else if (InGameMenus[i] == &Exit) {
						state = GAME_STATE_QUIT;
					}
#ifdef DEBUG
					else
						std::cerr << "Unrecognized In-game menu button\n";
#endif
				}
			}
			break;
		case GAME_STATE_WIN:
		case GAME_STATE_FINISHED:
			for (unsigned int i=0; i<LevelCompleteMenus.size(); i++) {
				if (LevelCompleteMenus[i]->OnButton(event->x, event->y)) {
					LevelCompleteMenus[i]->SetState(Button::BUTTON_STATE_SELECTED);

					if (LevelCompleteMenus[i] == &NextLC) {
						state = GAME_STATE_FINISHED;
					} else if (LevelCompleteMenus[i] == &RestartLC) {
						Reset();
						state = GAME_STATE_RUNNING;
					}
#ifdef DEBUG
					else
						std::cerr << "Unrecognized level-complete menu button\n";
#endif
				}
			}
			break;
	}
}

void LevelUI::Draw(SDL_Surface *screen) const
{
	// FIXME:  Re-add transition shading
	SDL_Rect offset;
	if (level.IsItDay()) {
		offset.x = level.GetOffset().x;
		offset.y = 0;
		SDL_BlitSurface(day_sky, NULL, screen, &offset);

		offset.x += day_sky->w/5;
		offset.y = static_cast<Sint16>(day_scene->h*1.7f*fabsf(0.5f-level.GetDayProgress()));
		SDL_BlitSurface(sun, NULL, screen, &offset);

		DrawCloud(screen);
		offset.x = level.GetOffset().x;
		offset.y = 0;
		SDL_BlitSurface(day_scene, NULL, screen, &offset);

		// Draw the level elements
		level.Draw(screen);
		DrawCloudCover(screen);
	} else {
		offset.x = level.GetOffset().x;
		offset.y = 0;
		SDL_BlitSurface(night_sky, NULL, screen, &offset);

		offset.x += day_sky->w/5;
		offset.y = static_cast<Sint16>(night_scene->h*1.7f*fabsf(0.5f-level.GetDayProgress()));
		SDL_BlitSurface(moon, NULL, screen, &offset);

		DrawCloud(screen);

		offset.x = level.GetOffset().x;
		offset.y = 0;
		SDL_BlitSurface(night_scene, NULL, screen, &offset);

		// Draw the level elements
		level.Draw(screen);
		DrawCloudCover(screen);

		// Draw the transition to night from day
		offset.y = level.GetOffset().y;
		SDL_SetAlpha(black_shading, SDL_SRCALPHA|SDL_RLEACCEL,
					 NIGHT_SHADING_ALPHA);
		SDL_BlitSurface(black_shading, NULL, screen, &offset);
	}

	offset.x = level.GetOffset().x + level.GetDimension().x - statustext->w - 5;
	offset.y = 13*day_scene->h/16 - 5;
	SDL_BlitSurface(statustext, NULL, screen, &offset);

	SDL_Color textcolor;
	textcolor.r = 25;
	textcolor.g = 200;
	textcolor.b = 50;
	switch (level.GetVictoryType()) {
		case Level::VICTORY_SURVIVAL:
			{
				offset.x += statustext->w;
				std::stringstream ss;
				ss << level.GetSurvivalTimeRemaining()/UPDATES_PER_SECOND;
				BlitText(gamefont, ss.str().c_str(), &textcolor,
						 screen, &offset);
			}
			break;
		case Level::VICTORY_COLLECT:
			{
				offset.x += statustext->w;
				std::stringstream ss;
				ss << Butterfly::LiveButterflyCount();
				BlitText(gamefont, ss.str().c_str(), &textcolor,
						 screen, &offset);
			}
			break;
	}

	offset.x = 0;
	offset.y = 0;
	SDL_BlitSurface(sidecovertl, NULL, screen, &offset);
	offset.y = sidecovertl->h;
	SDL_BlitSurface(sidecoverbl, NULL, screen, &offset);
	offset.x = screen->w-sidecovertr->w;
	offset.y = 0;
	SDL_BlitSurface(sidecovertr, NULL, screen, &offset);
	offset.y = sidecovertr->h;
	SDL_BlitSurface(sidecoverbr, NULL, screen, &offset);

	switch (state) {
		case GAME_STATE_INTRO:
			offset.x = level.GetOffset().x;
			offset.y = level.GetOffset().y;
			SDL_SetAlpha(black_shading, SDL_SRCALPHA|SDL_RLEACCEL,
						 MENU_SHADING_ALPHA);
			SDL_BlitSurface(black_shading, NULL, screen, &offset);

			offset.x += level.GetDimension().x/20;
			offset.y += level.GetDimension().y/20;
			BlitText(introfont, level.GetName().c_str(), &textcolor, screen,
					 &offset);

			textcolor.r = 200;
			textcolor.g = 200;
			textcolor.b = 25;
			offset.y += TTF_FontHeight(introfont);
			switch (level.GetVictoryType()) {
				case Level::VICTORY_GOALTILE:
					BlitText(introfont, "Catch the butterfly!", &textcolor,
							 screen, &offset);
					offset.y += TTF_FontHeight(introfont);
					break;
				case Level::VICTORY_SURVIVAL:
					BlitText(introfont, "Survive!", &textcolor, screen, &offset);
					offset.y += TTF_FontHeight(introfont);
					break;
				case Level::VICTORY_COLLECT:
					BlitText(introfont, "Collect all the butterflies!",
							 &textcolor, screen, &offset);
					offset.y += TTF_FontHeight(introfont);
					break;
			}

			textcolor.r = 255;
			textcolor.g = 255;
			textcolor.b = 255;

			{
				const std::vector<std::string>& intromessage = level.GetIntroMessage();
				for (unsigned int i=0; i<intromessage.size();
					 i++, offset.y+=TTF_FontHeight(introfont))
					BlitText(introfont, intromessage[i].c_str(),
							 &textcolor, screen, &offset);

				BlitText(introfont, "Press any key to start.", &textcolor,
						 screen, &offset);
			}
			break;
		case GAME_STATE_OVER:
			offset.x = level.GetOffset().x;
			offset.y = level.GetOffset().y;
			SDL_SetAlpha(black_shading, SDL_SRCALPHA|SDL_RLEACCEL,
						 MENU_SHADING_ALPHA);
			SDL_BlitSurface(black_shading, NULL, screen, &offset);

			offset.x = screen->w/2-game_over->w/2;
			offset.y = screen->h/4;
			SDL_BlitSurface(game_over, NULL, screen, &offset);
			for (unsigned int i=0; i<GameOverMenus.size(); i++)
				GameOverMenus[i]->Draw(screen);
			break;
		case GAME_STATE_PAUSED:
			offset.x = level.GetOffset().x;
			offset.y = level.GetOffset().y;
			SDL_SetAlpha(black_shading, SDL_SRCALPHA|SDL_RLEACCEL,
						 MENU_SHADING_ALPHA);
			SDL_BlitSurface(black_shading, NULL, screen, &offset);

			offset.x = screen->w/2 - paused->w/2;
			offset.y = screen->h/4;
			SDL_BlitSurface(paused, NULL, screen, &offset);
			for (unsigned int i=0; i<InGameMenus.size(); i++)
				InGameMenus[i]->Draw(screen);			
			break;
		case GAME_STATE_WIN:
		case GAME_STATE_FINISHED:
			offset.x = level.GetOffset().x;
			offset.y = level.GetOffset().y;
			SDL_SetAlpha(black_shading, SDL_SRCALPHA|SDL_RLEACCEL,
						 MENU_SHADING_ALPHA);
			SDL_BlitSurface(black_shading, NULL, screen, &offset);

			offset.x = screen->w/2 - level_complete->w/2;
			offset.y = screen->w/4;
			SDL_BlitSurface(level_complete, NULL, screen, &offset);
			for (unsigned int i=0; i<LevelCompleteMenus.size(); i++)
				LevelCompleteMenus[i]->Draw(screen);
			break;
	}
}

// vim: set ts=4 sts=4 sw=4 noet:
