/// \file LevelUI.hpp Header File for the LevelUI class

/// Include guard for LevelUI.hpp
#ifndef INCLUDED_LEVEL_H
#define INCLUDED_LEVEL_H 1

#include "Utility.hpp"
#include "Level.hpp"
#include "Button.hpp"

class GameObject;
class Charlie;
class Flower;
class TigerCub;

/**
 * \brief Representation of a single level in the game.
 */
class LevelUI {
	friend class LevelParser;
	friend class LevelWriter;

public:
	/** \brief Possible level states */
    enum GameState {
		GAME_STATE_INTRO,		/**< The introduction to the level */
		GAME_STATE_RUNNING,		/**< The level is active (running) */
		GAME_STATE_PAUSED,		/**< The level is inactive (paused) */
		GAME_STATE_OVER,		/**< Game over */
		GAME_STATE_WIN,			/**< The level is complete */
		GAME_STATE_FINISHED,	/**< This level is finished running */
		GAME_STATE_QUIT			/**< This user quit the level */
    };

    /** \brief LevelUI Constructor using a saved level file.
	 * Read level information from levelfile and create the tiles and creatures
	 * from that information.  Starts in the paused state
	 *
	 * \param levelfile The name of a file containing the level to load.
	 */
    LevelUI(const char *levelfile);

	/** \brief LevelUI Constructor starting from scratch.
	 * Create a new, empty, level.
	 *
	 * \param rows The number of rows of tiles in the level
	 * \param cols The number of columns of tiles in the level
	 */
	LevelUI(const int rows, const int cols);

    /** \brief LevelUI destructor.
	 * Deallocates everything that comprises the level.
	 */
    ~LevelUI();

    /** \brief Reset the levelui to its initial state */
    void Reset();

    /** \brief Set Game state to paused
	 *
	 * This function suspends the current levelui and suspends all GameObjects
	 * in the levelui (where necessary).  It may also darken the map area, as a
	 * visual clue to the user.
	 */
    void Pause();

    /** \brief Set Game state to running
	 *
	 * This function resumes the current levelui and resumes all GameObjects in
	 * the levelui (where necessary).  It also removes darkening, if applied in
	 * Pause().
	 */
    void Unpause();

    /** \brief Get the current levelui state (paused or unpaused)
	 * 
	 * \return The current levelui state
	 */
    GameState GetState() const;

	/** \brief Get the Level for this levelui
	 *
	 * \return The game level
	 */
	Level *GetLevel();

    /** \brief Update the current state of the levelui
	 *
	 * This function updates its internal variables and calls Update for
	 * all GameObjects, then calls the level's update function for each
	 * GameObject.
	 */
    void Update();

	/** \brief Report a keyboard event (key press or release)
	 * \param event The event to process
	 */
	void KeyboardEvent(SDL_KeyboardEvent *event);

	/** \brief Report a mouse motion event
	 * \param event The event to process
	 */
	void MouseMotionEvent(SDL_MouseMotionEvent *event);

	/** \brief Report a mouse button event (button press or release)
	 * \param event The event to process
	 */
	void MouseButtonEvent(SDL_MouseButtonEvent *event);

    /** \brief Redraw the levelui, including the level and all GameObjects */
    void Draw(SDL_Surface *screen) const;

	/** \brief Save the current LevelUI to a file
	 * \param levelfile file to which this LevelUI should be saved
	 */
	void WriteLevelUI(const char *levelfile);

	/** \brief Is this LevelUI finished? */
	bool IsFinished() const { return state == GAME_STATE_FINISHED; }

	/** \brief Did the user exit the level? */
	bool IsQuit() const { return state == GAME_STATE_QUIT; }

	/** \brief Create a cloud over the level */
	void CreateCloud();

	/** \brief Update the cloud in the level */
	void UpdateCloud();

	/** \brief Draw the cloud in the level */
	void DrawCloud(SDL_Surface *screen) const;

	/** \brief Draw the cloud cover in the level */
	void DrawCloudCover(SDL_Surface *screen) const;

private:
	SDL_Surface *paused, *game_over, *level_complete, *statustext;
	SDL_Surface *night_sky, *day_sky, *night_scene, *day_scene, *sun, *moon,
				*sidecovertl, *sidecovertr, *sidecoverbl, *sidecoverbr;
	SDL_Surface *water;
	SDL_Surface *black_shading;
	SDL_Surface *cloud_cover, *cloud;

	Vec2f cloud_cover_pos;
	Vec2f cloud_pos;
	Vec2f cloud_cover_speed;
	Vec2f cloud_speed;
	float cloud_scale;
	bool isThereCloud;

	Mix_Chunk *victory_theme;			///< Sound played to signal victory
	Mix_Chunk *defeat_theme;			///< Sound played to signal defeat
	Mix_Chunk *restart_theme;			///< Sound played when the level is restarted

    Level level;					///< Game level for this LevelUI
    GameState state;				///< Current levelui state
	int introLength;				///< The length of the introduction to this LevelUI (show name)

	Button Resume, Restart, RestartIntro, Exit, RestartGO, RestartGOIntro, ExitGO, NextLC, RestartLC;
	std::vector<Button *> InGameMenus, GameOverMenus, LevelCompleteMenus;
};

inline LevelUI::GameState LevelUI::GetState() const { return state; }
inline Level *LevelUI::GetLevel() { return &level; }

inline void LevelUI::UpdateCloud() {
	cloud_cover_pos += cloud_cover_speed;
	cloud_pos += cloud_speed;
	if (cloud_speed.x > 0 && cloud_cover_pos.x > 1650 ||
		cloud_speed.x < 0 && cloud_cover_pos.x < -1074) {
		isThereCloud = false;
	}
}

inline void LevelUI::CreateCloud() {
	if (isThereCloud) {
		UpdateCloud();
	} else if (GetRandomInt(0,100) == 0) {
		float xspeed = GetRandomFloat(0.0f, 2.0f);
		float ypos = GetRandomFloat(100.0f, 500.0f);
		cloud_cover_speed = Vec2f(xspeed,0.0f);
		cloud_cover_pos = Vec2f(-1074.0f,ypos);
		cloud_speed = Vec2f(cloud_cover_speed.x*0.57f,cloud_cover_speed.y);
		cloud_pos = Vec2f(100,ypos/1000*60);
		cloud_scale = GetRandomFloat(0.75f, 1.0f);
		isThereCloud = true;
	}
}
inline void LevelUI::DrawCloud(SDL_Surface *screen) const
{
	if (isThereCloud) {
		SDL_Rect offset;
		offset.x = static_cast<Sint16>(cloud_pos.x);
		offset.y = static_cast<Sint16>(cloud_pos.y);
		SDL_BlitSurface(cloud, NULL, screen, &offset);
	}
}
inline void LevelUI::DrawCloudCover(SDL_Surface *screen) const {
	if (isThereCloud) {
		SDL_Rect offset;
		offset.x = static_cast<Sint16>(cloud_pos.x);
		offset.y = static_cast<Sint16>(cloud_pos.y);
		SDL_BlitSurface(cloud_cover, NULL, screen, &offset);
	}
}

#include "GameObject.hpp"
#include "Centipede.hpp"
#include "Charlie.hpp"
#include "Flower.hpp"
#include "Snake.hpp"
#include "TigerCub.hpp"

#endif /* INCLUDED_LEVEL_H */
// vim: set ts=4 sts=4 sw=4 noet:
