/// \file LevelWriter.cpp Source file for the LevelWriter class

#include <fstream>
#include <sstream>

#include "Utility.hpp"
#include "LevelWriter.hpp"

std::ofstream LevelWriter::saved;

int LevelWriter::writeLevelFile(const char *levelfile, Level *level)
{
	saved.open(levelfile);

	saved << "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"
			 "<!DOCTYPE level SYSTEM \"geukalevel.dtd\">\n"
			 "<level xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
			 "  xsi:noNamespaceSchemaLocation=\"geukalevel.xsd\"\n"
			 "  name=\"" << level->levelname << "\" "
			 "daylength=\"" << level->daylength/UPDATES_PER_SECOND << "\" "
			 "dayremaining=\"" << level->initialdayremaining/static_cast<signed int>(UPDATES_PER_SECOND) << "\" ";

	switch (level->victorytype) {
		case Level::VICTORY_GOALTILE:
			saved << "victorytype=\"goaltile\"";
			break;
		case Level::VICTORY_SURVIVAL:
			saved << "victorytype=\"survival\" survivaltime=\""
				  << level->survivaltime << "\"";
			break;
		case Level::VICTORY_COLLECT:
			saved << "victorytype=\"collection\"";
			break;
		default:
			assert(0);
			break;
	}
	saved << ">\n";

	writeTiles(level);
	writeObjects(level);
	writeCreatures(level);

	saved << "</level>\n"
			 "<!-- vim: set ts=8 sts=2 sw=2 noet: -->";

	bool failed = saved.fail();
	saved.close();
	return failed ? -1 : 0;
}

void LevelWriter::writeTiles(Level *level)
{
	saved << "  <tiles cols=\"" << level->GetCols() << "\" "
			 "rows=\"" << level->GetRows() << "\">\n";

	for (unsigned int row=0; row<level->GetRows(); row++) {
		for (unsigned int col=0; col<level->GetCols(); col++) {
			saved << level->GetTile(col,row)->GetXMLRepresentation("    ");
		}
	}

	saved << "  </tiles>\n";
}

void LevelWriter::writeCreatures(Level *level)
{
	saved << "  <creatures>\n";
	for (unsigned int i=0; i<level->creatures.size(); i++)
		saved << level->creatures[i]->GetXMLRepresentation("    ");
	saved << "  </creatures>\n";
}

void LevelWriter::writeObjects(Level *level)
{
	saved << "  <objects>\n";
	for (unsigned int i=0; i<level->objects.size(); i++)
		saved << level->objects[i]->GetXMLRepresentation("    ");
	for (unsigned int i=0; i<level->goals.size(); i++)
		saved << level->goals[i]->GetXMLRepresentation("    ");
	saved << "  </objects>\n";
}

std::string LevelWriter::waypointsToXML(const std::string& indent,
										const Vec2f position,
										std::vector<Vec2f>::const_iterator start,
										std::vector<Vec2f>::const_iterator end,
										bool duringday, bool duringnight)
{
	std::stringstream xml;
	std::vector<Vec2f>::const_iterator curr = start;
	
	// Don't need to print anything if not during day or night
	if (!duringday && !duringnight)
		return "";

	// Skip initial waypoints at starting position
	if (duringday && duringnight)
		while (curr != end && *curr == position)
			curr++;

	while (curr != end) {
		xml << indent << "<waypoint x=\""
			<< floorf((*curr).x) + 0.5f
			<< "\" y=\"" << floorf((*curr).y) + 0.5f
			<< "\"";
		
		if (duringday && !duringnight)
			xml << " during=\"day\"";
		else if (!duringday && duringnight)
			xml << " during=\"night\"";
		
		xml << "/>\n";

		// Skip over redundant waypoints
		std::vector<Vec2f>::const_iterator next = curr + 1;
		while (next != end && *curr == *next)
			next++;
		curr = next;
	}

	return xml.str();
}

// vim: set ts=4 sts=4 sw=4 noet:
