/// \file MainMenu.hpp Header File for the MainMenu class

/// Include guard for MainMenu.hpp
#ifndef INCLUDED_MAINMENU_H
#define INCLUDED_MAINMENU_H 1

#include "Utility.hpp"
#include "Button.hpp"
#include "ToggleButton.hpp"

class LevelUI;
class LevelEditor;

/**
 * \brief Representation of the main menu system for the game.
 */
class MainMenu {

public:
	/** \brief Possible Main Menu "states" */
    enum MenuState {
		MENU_STATE_INTRO,		/**< The introduction */
		MENU_STATE_MAIN,		/**< The main menu */
		MENU_STATE_STORY,		/**< The story mode */
		MENU_STATE_LOAD,		/**< The load menu */
		MENU_STATE_OPTIONS,		/**< The options menu */
		MENU_STATE_EXIT,		/**< The options menu */
		MENU_STATE_EDITOR,		/**< The level editor menu */
		MENU_STATE_GAME,		/**< The game is running */
		MENU_STATE_EDITOR_RUN,	/**< The level editor is running */
		MENU_STATE_VICTORY		/**< The game has been completed! */
    };

	/** \brief Possible Game "modes" */
    enum GameMode {
		GAME_MODE_STORY,		/**< Playing story mode */
		GAME_MODE_LOADLEVEL,	/**< Playing a specific level */
    };

    /** \brief MainMenu Constructor.
	 * Starts the game in the introduction state
	 */
    MainMenu(int width, int height);

    /** \brief MainMenu destructor.
	 * Deallocates everything that comprises the main menu.
	 */
    ~MainMenu();

    /** \brief Reset the level to its initial state */
    void Reset();

    /** \brief Get the menu state state
	 * 
	 * \return The current menu state
	 */
    MenuState GetState() const;

	/** \brief Get the game mode
	 * 
	 * \return The current game mode
	 */
    GameMode GetMode() const;

    /** \brief Update the menu state of menu system */
    void Update();

	/** \brief Report a keyboard event (key press or release)
	 * \param event The event to process
	 */
	void KeyboardEvent(SDL_KeyboardEvent *event);

	/** \brief Report a mouse motion event
	 * \param event The event to process
	 */
	void MouseMotionEvent(SDL_MouseMotionEvent *event);

	/** \brief Report a mouse button event (button press or release)
	 * \param event The event to process
	 */
	void MouseButtonEvent(SDL_MouseButtonEvent *event);

    /** \brief Draw the menu system */
    void Draw(SDL_Surface *screen) const;

	/** \brief Load a Level from story mode and begin playing
	 * \param levelNumber The sequence number of the Level to load
	 */
	void LoadLevel(unsigned int levelNumber);

	/** \brief Load a Level from a file
	 * \param levelfile The file to load from
	 */
	void LoadLevel(const std::string& levelfile);

	/** \brief Start the level editor */
	void StartLevelEditor();

	/** \brief Start the level editor on an existing level
	 * \param levelfile path to the Level to be edited
	 */
	void StartLevelEditor(const std::string& levelfile);

private:
	/** \brief Change the state of this LevelEditor
	 * \param newstate State to transition into
	 */
	void SwitchState(MenuState newstate);

	SDL_Surface *intro_pic;			///< Introductory image
	SDL_Surface *victory_pic;		///< Victory screen image
	SDL_Surface *mainmenubg_pic;	///< Background for the main menu
	SDL_Surface *mainmenu_border1;	///< Main menu button border
	SDL_Surface *mainmenu_border2;	///< Level selector button border
	SDL_Surface *needrestart_pic;	///< Need to restart for changes to appear
	SDL_Surface *level_editor_pic;

	Mix_Chunk *menumusic;			///< The background music for the menu

	LevelUI *runningLevel;
	LevelEditor *runningEditor;
	std::vector<std::string> storyLevels;
	unsigned int levelNumber;		///< Current Level (in Story mode)
    MenuState state;				///< Current menu state
	GameMode mode;					///< Current game mode
	bool needrestart;				///< Restart for changes
	int pageSelected;

	Button Story, Load, Editor, Options, Exit, StartStory, ContinueStory, BackToMain, LoadFile, CreateLevel;
	ToggleButton Fullscreen, Music;
	ToggleButton LoadLevel1, LoadLevel2, LoadLevel3, LoadLevel4, LoadLevel5,
				 LoadLevel6, LoadLevel7, LoadLevel8, LoadLevel9, LoadLevel10,
				 LoadLevel11, LoadLevel12, LoadLevel13, LoadLevel14, LoadLevel15,
				 LoadLevel16, LoadLevel17, LoadLevel18, LoadLevel19, LoadLevel20,
				 LoadLevel21, LoadLevel22, LoadLevel23, LoadLevel24, LoadLevel25,
				 LoadLevel26;
	ToggleButton Page1, Page2, Page3, Page4;
	std::vector<Button *> MainMenus, StoryMenus, LoadMenus, EditorMenus, OptionsMenus;
	std::vector<ToggleButton *> PageMenus;
};

inline MainMenu::MenuState MainMenu::GetState() const { return state; }
inline MainMenu::GameMode MainMenu::GetMode() const { return mode; }

#include "LevelUI.hpp"
#include "LevelEditor.hpp"

#endif /* INCLUDED_MAINMENU_H */
// vim: set ts=4 sts=4 sw=4 noet:
