/// \file Millipede.cpp Source file for the Millipede class

#include <sstream>

#include "SDL_gfxPrimitives.h"

#include "Millipede.hpp"
#include "LevelWriter.hpp"

SDL_Surface *Millipede::millipedeImage;
SDL_Surface *Millipede::rolledUpImage;
Mix_Chunk *Millipede::millipedeBite;
unsigned int Millipede::millipedecounter = 0;

Millipede::Millipede(Level *l, Vec2f p) :
   	GameObject(l, p, MILLIPEDE_DEFAULT_MAX_SPEED, MILLIPEDE_DEFAULT_RADIUS),
	waypoints(1,p),
   	wp_index(0),
	anim_timer(0)
{
	if (millipedecounter++ == 0) {
		millipedeImage = LoadSprite(PKGDATADIR"Artwork/creatures/centipede/centipede.png",
									CREATURE_ROTATIONS,
									level->GetTileWidth(),
									level->GetTileHeight());
		rolledUpImage = LoadSprite(PKGDATADIR"Artwork/creatures/centipede/centipede_rolledup.png",
								   CREATURE_ROTATIONS,
								   level->GetTileWidth(),
								   level->GetTileHeight());
		millipedeBite = LoadSound(PKGDATADIR"Sounds/MillipedePushed.ogg");
	}

	image = rolledUpImage;

	state = MILLIPEDE_STATE_ROLLEDUP;
}

Millipede::~Millipede()
{
	if (--millipedecounter == 0) {
		SDL_FreeSurface(millipedeImage);
		SDL_FreeSurface(rolledUpImage);
		Mix_FreeChunk(millipedeBite);
	}
}

void Millipede::Run(void)
{
	if (disabled)
	   	return;

	Charlie *c = level->GetCharlie();

	//decide if I change state
	switch(state) {
		case MILLIPEDE_STATE_AGGRO:
			if(level->IsItDay())
			{
				image = rolledUpImage;
				state = MILLIPEDE_STATE_ROLLEDUP;
			}
			else if((c->GetPosition() - position).Length() > MILLIPEDE_AGGRO_RADIUS || c->GetInvisible())
			{
				image = millipedeImage;
				state = MILLIPEDE_STATE_DEAGGRO;
				max_speed = MILLIPEDE_AGGRO_MAX_SPEED;
			}
			break;
		case MILLIPEDE_STATE_DEAGGRO:
			if(level->IsItDay())
			{
				image = rolledUpImage;
				state = MILLIPEDE_STATE_ROLLEDUP;
			}
			else if((c->GetPosition() - position).Length() < MILLIPEDE_AGGRO_RADIUS && !c->GetInvisible() && level->HasLineOfSight(c->GetPosition(),position))
			{
				image = millipedeImage;
				state = MILLIPEDE_STATE_AGGRO;
				max_speed = MILLIPEDE_AGGRO_MAX_SPEED;
			}
			else if(anchorpoint.x - max_speed - 1 <= position.x && position.x <= anchorpoint.x + max_speed + 1
				 && anchorpoint.y - max_speed - 1 <= position.y && position.y <= anchorpoint.y + max_speed + 1)
			{
				image = millipedeImage;
				state = MILLIPEDE_STATE_PATHING;
			}
			break;
		case MILLIPEDE_STATE_PATHING:
			if(level->IsItDay())
			{
				image = rolledUpImage;
				state = MILLIPEDE_STATE_ROLLEDUP;
				anchorpoint = position;
			}
			else if((c->GetPosition() - position).Length() < MILLIPEDE_AGGRO_RADIUS && !c->GetInvisible() && level->HasLineOfSight(c->GetPosition(),position))
			{
				image = millipedeImage;
				state = MILLIPEDE_STATE_AGGRO;
				anchorpoint = position;
				max_speed = MILLIPEDE_AGGRO_MAX_SPEED;
			}
			else if ((waypoints[wp_index] - position).Length() < max_speed)
				wp_index = (wp_index + 1)%waypoints.size();
			break;
		case MILLIPEDE_STATE_ROLLEDUP:
			if(!level->IsItDay())
			{
				image = millipedeImage;
				max_speed = MILLIPEDE_DEFAULT_MAX_SPEED;
				state = MILLIPEDE_STATE_PATHING;
			}
			break;
		default: assert(0);

	}

	Vec2f goal;

	//Set goal based on state
	switch(state) {
		case MILLIPEDE_STATE_AGGRO: goal = c->GetPosition(); break;
		case MILLIPEDE_STATE_DEAGGRO: goal = anchorpoint; break;
		case MILLIPEDE_STATE_PATHING: goal = waypoints.at(wp_index); break;
		case MILLIPEDE_STATE_ROLLEDUP: return;
		default: assert(0);
	}

	anim_timer = (anim_timer + 1) % MILLIPEDE_ANIMATION_SPEED;
	if(anim_timer == 0)
		anim_index = (anim_index + 1) % 2;

	velocity = level->GetShortestPath(GetPosition(), goal, max_speed);
	position += velocity;
}

void Millipede::Reset(void)
{
	this->GameObject::Reset();

	image = rolledUpImage;

	max_speed = MILLIPEDE_DEFAULT_MAX_SPEED;
	radius = MILLIPEDE_DEFAULT_RADIUS;

	state = MILLIPEDE_STATE_ROLLEDUP;

	anim_index = 0;
	anim_timer = 0;
}

void Millipede::SetRolledUp(bool ru)
{
	if (ru) {
		image = rolledUpImage;
		state = MILLIPEDE_STATE_ROLLEDUP;
	} else {
		image = millipedeImage;
		state = MILLIPEDE_STATE_PATHING;
	}
}

std::string Millipede::GetXMLRepresentation(const std::string& indent) const
{
	std::ostringstream xml;
	xml << indent << "<creature x=\"" << level->GetTilePos(position).x + 0.5f
		<< "\" y=\"" << level->GetTilePos(position).y + 0.5f
		<< "\" type=\"millipede\"";

	if (HasColor())
		xml << " color=\"" << GetColor().name() << "\"";
	
	std::string wptstring = LevelWriter::waypointsToXML(indent+"  ", position,
														waypoints.begin(),
														waypoints.end());
	if (wptstring.size() == 0)
		xml << "/>\n";
	else
		xml << ">\n" << wptstring << indent << "</creature>\n";

	return xml.str();
}

bool Millipede::IsPatroller(){
	return true;
}

void Millipede::DrawWaypoints(SDL_Surface *screen, ColorRGBA color)
{
	Vec2f first = waypoints.front();
	Vec2f prev;
	for (std::vector<Vec2f>::iterator i = waypoints.begin();
		 i != waypoints.end(); i++) {
		if (*i != first)
			aalineRGBA(screen, prev.x, prev.y, (*i).x, (*i).y,
					   color.r, color.g, color.b, color.a);

		prev = *i;
	}
	aalineRGBA(screen, prev.x, prev.y, first.x, first.y,
			   color.r, color.g, color.b, color.a);
}

void Millipede::ResetWaypoints(Vec2f pos){
	waypoints.clear();
	waypoints.push_back(pos);
}
// vim: set ts=4 sts=4 sw=4 noet:
