/// \file Millipede.hpp Header file for the Millipede class

/// Include guard for Millipede.hpp
#ifndef INCLUDED_MILLIPEDE_H
#define INCLUDED_MILLIPEDE_H 1

#include "SoundManager.hpp"
#include "GameObject.hpp"
#include "Patroller.hpp"
#include "Saveable.hpp"

/**
 * \brief Class representing a millipede.
 */
class Millipede : public GameObject, public Patroller {
public:
	/** \brief Constructs a Millipede in Level l at position p.
	* \param l a pointer to the Level this Millipede exists in
	* \param p the Vec2f start_position of this Millipede
	*/
	Millipede(Level *l, Vec2f p);

	/** \brief Destructs a Millipede */
	~Millipede();

	/** \brief Runs one cycle of this Millipede.
	*/
	void Run(void);

	/** \brief Resets this Millipede to its original state in its Level.
	*/
	void Reset(void);

	//SET methods

	/** \brief Sets whether or not this Millipede is rolled up.
	 * \param ru whether or not this Millipede is rolled up
	 */
	void SetRolledUp(bool ru);

	/** \brief Adds a waypoint to this TigerCub.
	 * 
	 * Adds a waypoint to this Tigercub. While awake, the TigerCub will
	 * attempt to visit all its waypoints in the order they were added.
	 * When it completes the circuit, it will begin again.
	 * \param wp the Vec2f waypoint to add
	 * \param duringday Should the waypoint be visited during the day?
	 * \param duringnight Should the waypoint be visited at night?
	 */
	inline void AddWaypoint(const Vec2f wp,
							bool duringday = true, bool duringnight = true) 
		{ waypoints.push_back(wp); }

	//GET methods
	std::string GetXMLRepresentation(const std::string& indent) const;

	/** \brief Gets whether or not this Millipede is rolled up.
	 * \return whether or not this Millipede is rolled up
	 */
	inline bool GetRolledUp(void) const { return state == MILLIPEDE_STATE_ROLLEDUP; }

	/** \brief Gets the GameObjectType of this Millipede.
	 * \return the GameObjectType of this Millipede
	 */
	inline GameObject::GameObjectType GetType(void) const { return GAME_OBJECT_MILLIPEDE; }

	void PlayCollisionSound() const;

	/** \brief draws the creature's waypoints
	 *
	 * Draws the creature's waypoints as dots and path between them as lines
	 */
	void DrawWaypoints(SDL_Surface *screen, ColorRGBA color);

	/** \brief resets creature's waypoints
	 *
	 * Clears all the creature's waypoints except for the origin
	 */
	void ResetWaypoints(Vec2f pos);

	bool IsPatroller();

private:
	static SDL_Surface *millipedeImage;		///< The default image of a Millipede
	static SDL_Surface *rolledUpImage;		///< The rolled-up image of a Millipede
	static Mix_Chunk *millipedeBite;		///< The sound of the Millipede attacking
	static unsigned int millipedecounter;	///< Number of Millipedes

	enum MillipedeState{			///< Enum describing the mental state of a Millipede
		MILLIPEDE_STATE_AGGRO,
		MILLIPEDE_STATE_DEAGGRO,
		MILLIPEDE_STATE_PATHING,
		MILLIPEDE_STATE_ROLLEDUP
	};

	MillipedeState state;			///< The mental state of this Millipede
	unsigned int anim_index;		///< The animation frame index of this Millipede
	unsigned int anim_timer;		///< Time until this Millipede changes to its next frame

	std::vector<Vec2f> waypoints;	///< The waypoints of this Millipede
	int wp_index;					///< The current index of this Millipede's waypoints
	Vec2f anchorpoint;
	Vec2f pt1, pt2, pt3, pt4;
};

inline void Millipede::PlayCollisionSound() const
{
	if (level->IsItDay() && GetRolledUp())
		SoundManager::PlayChunk(millipedeBite);
}

#endif /* INCLUDED_MILLIPEDE_H */
// vim: set ts=4 sts=4 sw=4 noet:
