/// \file Snake.hpp Header file for the Snake class

/// Include guard for Snake.hpp
#ifndef INCLUDED_SNAKE_H
#define INCLUDED_SNAKE_H 1

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif

#include "GameObject.hpp"
#include "Patroller.hpp"

/**
 * \brief Class representing a snake.
 */
class Snake : public GameObject, public Patroller {
public:
	/** \brief Constructs a Snake in Level l at position p with Color c.
	* \param l a pointer to the Level this Snake exists in
	* \param p the Vec2f start_position of this Snake
	* \param c the Color of this Snake
	*/
	Snake(Level *l, Vec2f p, ColorRGBA c);

	/** \brief Destructs a Snake */
	~Snake();

	/** \brief Runs one cycle of this Snake.
	*/
	void Run(void);

	/** \brief Resets this Snake to its original state in its Level.
	*/
	void Reset(void);

	void AddWaypoint(const Vec2f wp,
					 bool duringday = true, bool duringnight = true);

	/** \brief draws the creature's waypoints
	 *
	 * Draws the creature's waypoints as dots and path between them as lines
	 */
	void DrawWaypoints(SDL_Surface *screen, ColorRGBA color);

	/** \brief resets creature's waypoints
	 *
	 * Clears all the creature's waypoints except for the origin
	 */
	void ResetWaypoints(Vec2f pos);

	bool IsPatroller();

	//GET methods
	std::string GetXMLRepresentation(const std::string& indent) const;

	/** \brief Gets the GameObjectType of this Snake.
	 * \return the GameObjectType of this Snake
	 */
	inline GameObjectType GetType(void) const { return GAME_OBJECT_SNAKE; }

	void PlayCollisionSound() const;

private:
	void UpdateImage(void);

	static SDL_Surface *snake_black;			///< Image representing this Snake.
	static SDL_Surface *snake_blue;
	static SDL_Surface *snake_brown;
	static SDL_Surface *snake_green;
	static SDL_Surface *snake_orange;
	static SDL_Surface *snake_red;
	static SDL_Surface *snake_violet;
	static SDL_Surface *snake_white;
	static SDL_Surface *snake_yellow;

	static SDL_Surface *snake_moving_black;
	static SDL_Surface *snake_moving_blue;
	static SDL_Surface *snake_moving_brown;
	static SDL_Surface *snake_moving_green;
	static SDL_Surface *snake_moving_orange;
	static SDL_Surface *snake_moving_red;
	static SDL_Surface *snake_moving_violet;
	static SDL_Surface *snake_moving_white;
	static SDL_Surface *snake_moving_yellow;

	SDL_Surface *still_image;				///< Image of this Snake sitting still
	SDL_Surface *moving_image;				///< Image of this Snake moving

	static Mix_Chunk *snakeBite;			///< Sound of the Snake biting
	static unsigned int snakecount;		///< Number of Snakes created

	std::vector<Vec2f> day_waypoints;	///< The day waypoints of this Snake.
	int day_wp_index;					///< The current index of this Snake's day waypoints.

	std::vector<Vec2f> night_waypoints;	///< The night waypoints of this Snake.
	int night_wp_index;					///< The current index of this Snake's night waypoints.

	Vec2f anchorpoint;					///< The anchorpoint of this Snake; set when it aggros, returns here when it deaggros.

	/// Describes the mental state of a Snake.
	enum SnakeState {
		SNAKE_STATE_AGGRO,		///< Snake is chasing Charlie
		SNAKE_STATE_DAY,		///< Snake is operating during the day
		SNAKE_STATE_DEAGGRO,	///< Snake is not chasing Charlie
		SNAKE_STATE_NIGHT		///< Snake is operating at night
	};

	SnakeState state;					///< The mental state of this Snake.
	unsigned int anim_timer;			///< How long this Snake has before moving on to the next animation frame.
	Vec2f pt1, pt2, pt3, pt4;
};

inline void Snake::AddWaypoint(const Vec2f wp,
							   bool duringday, bool duringnight)
{
	if (duringday)
		day_waypoints.push_back(wp);

	if (duringnight)
		night_waypoints.push_back(wp);
}

inline void Snake::PlayCollisionSound() const
{
	SoundManager::PlayChunk(snakeBite);
}

#endif /* INCLUDED_SNAKE_H */
// vim: set ts=4 sts=4 sw=4 noet:
