/// \file SoundManager.hpp Header file for the SoundManager class

/// Include guard for SoundManager.hpp
#ifndef INCLUDED_SOUNDMANAGER_H
#define INCLUDED_SOUNDMANAGER_H 1

#include "SDL_mixer.h"

#include "Utility.hpp"

const unsigned int SOUND_NUM_CHANNELS = 4;

/**
 * \brief Class for coordinating sound effects and music.
 *
 * This class is basically a wrapper over the GameX sound effects with the
 * difference that this implementation does what it says it does.
 * 
 * When multiple fades are applied to a single music file, the lowest of the
 * fades is applied rather than the multiple.  This is desirable especially
 * for when multiple sound effects should be playing over the music.
 */
class SoundManager
{
public:
	/** \brief Update the SoundManager
	 *
	 * Call this function for every tick of the game.  It handles gradual fading
	 * and making sure everything is running smoothly.
	 */
	static void Update();

	/** \name Sound FX Functions
	 * Functions to control sound effects
	 */
	//@{
	/** \brief Play a sound file
	 * \param sound The sound to play
	 * \param vol Volume to play the sound at (0.0 - 1.0)
	 * \param loops Number of loops (-1 = infinite, 0 = play once)
	 * \param delay Length of time before clip starts playing
	 * \param length Length of time to play (-1 = play until end)
	 * \param musicvolume What volume should the music be faded to while this
	 *					  sound is playing?
	 */
	static void PlayChunk(Mix_Chunk *sound, int vol = 100,
						  int loops = 0, int delay = 0, int length = -1,
						  int musicvolume = 100);

	/** \brief Stop a sound if it is playing
	 * \param sound The sound to stop
	 * \param time Time until sound halts (in ms)
	 */
	static void StopChunk(Mix_Chunk *sound, int time = 0);

	/** \brief Is a given sound playing?
	 * \param The sound to check the status of
	 * \return Is the given sound playing?
	 */
	static bool IsChunkPlaying(const Mix_Chunk *sound);
	//@}

	/** \name Music Functions
	 * Functions to control music files
	 */
	//@{
	/** \brief Play a given music file
	 *
	 * \param music The music file to play
	 * \param times The number of times to loop the music file (0 = unlimited)
	 * \param volume The volume at which to play the file (1.0 = full volume)
	 * \param fadetime The length of time over which to fade in the music
	 *				   (in seconds)
	 * \param startvol The volume at which to start the fade in
	 */
	static void PlayMusic(Mix_Chunk *music, int vol = 100, int loops = -1,
						  int delay = 0, int length = -1, int fadetime = 0);

	/** \brief Pause a music file
	 * \param music The piece of music to pause (or all if NULL)
	 */
	static void PauseMusic(Mix_Chunk *music);

	/** \brief Resume a paused music file
	 * \param music The piece of music to resume (or all if NULL)
	 */
	static void ResumeMusic(Mix_Chunk *music);

	/** \brief Stop a music file if it is playing
	 * \param music The music file to stop (or all if NULL)
	 * \param time The time over which to fade the music (in ms)
	 */
	static void StopMusic(Mix_Chunk *music, int time = 0);

	/** \brief Is a given music file playing?
	 * \param music The file to examine
	 * \return True if the file is playing, false otherwise
	 */
	static bool IsMusicPlaying(const Mix_Chunk *music);

	/** \brief Fade a given music file
	 * \param music The music file to fade
	 * \param endvolume The volume at which to end the fade
	 * \param fadetime The time over which to fade the music
	 */
	static void FadeMusic(Mix_Chunk *music, float endvolume = 0.0f,
						  float fadetime = 2.0f);

	/** \brief Get the current volume for a music file
	 * \param music The file to get the current volume for
	 * \return The current volume of the music file (0.0 if not playing)
	 */
	static int GetMusicVolume(const Mix_Chunk *music);
	//@}

	/** \brief Get the current master volume
	 * \return The current master volume
	 */
	static int GetMasterVolume();

	/** \brief Set the current master volume
	 * \param volume The target volume level
	 */
	static void SetMasterVolume(float volume);

private:
	/** \brief SoundManager constructor (never to be called)
	 *
	 * The SoundManager class is intended to be used to coordinate audio
	 * for the program.  Since there is only one audio output environment,
	 * there will never need to be multiple instances of a SoundManager.
	 */
	SoundManager();

	static unsigned long currtime;
	static Mix_Chunk *playing[SOUND_NUM_CHANNELS];
};

#endif /* INCLUDED_SOUNDMANAGER_H */
// vim: set ts=4 sts=4 sw=4 noet:
