/// \file Switch.hpp Header file for the Switch class

/// Include guard for Switch.hpp
#ifndef INCLUDED_SWITCH_H
#define INCLUDED_SWITCH_H 1

#include "GameObject.hpp"
#include "Switchable.hpp"

/**
 * \brief Class representing a Switch.
 */
class Switch : public GameObject {
public:
	/** \brief Constructs a Switch in Level l at position p that isToggleable if it = true
	 * \param l a pointer to the Level this Switch exists in
	 * \param p the Vec2f position of this Switch
	 * \param group number of the switch group to which this Switch belongs
	 * \param toggle is this a toggle switch?  (one that stays on after being triggered)
	 */
	Switch(Level *l, Vec2f p, bool toggle = true, int group = 0);

	/** \brief Destructs this Switch. */
	~Switch();

	/** \brief Add an object to be toggled by this switch
	 *  \param s Object to be changed when this switch is toggled
	 */
	inline void AddSwitchable(Switchable *s) { mySwitchables.push_back(s); };

	/** \brief Runs one cycle of this Switch. */
	void Run(void);

	/** \brief Resets this Switch to its original state in this Level. */
	void Reset(void);

	/** \brief Trigger this switch (press the red button)
	 *
	 * This function should be called any time there is a collision between
	 * a game object and this switch to indicate that it is being pressed.
	 */
	inline void Trigger() { triggered = true; };

	std::string GetXMLRepresentation(const std::string& indent) const;
	inline int GetSwitchGroup() const { return switchgroup; }
	inline void SetSwitchGroup(const int group)
		{ if (group > 0) switchgroup = group;
		  else switchgroup = 0;}

	/** \brief Gets the GameObjectType of this Switch.
	 * \return the GameObjectType of this Switch
	 */
	inline GameObjectType GetType(void) const { return GAME_OBJECT_SWITCH; };
private:
	static SDL_Surface *switchImage;		///< The default image of a Switch
	static Mix_Chunk *switchPressed;	///< Sound of the Switch being pressed
	static Mix_Chunk *switchReleased;	///< Sound of the Switch being released
	static unsigned int switchcount;	///< Number of Switches created

	std::vector<Switchable*> mySwitchables;	///< The Switchable this Switch toggles
	int switchgroup;			///< Number of the switch group for this Switch
	bool state;					///< The on/off state of this Switch
	bool triggered;				///< True if a GameObject rests on this Switch, false otherwise
	bool wastriggered;			///< True if the Switch was toggled last round
	bool toggle;			///< True if this switch is a toggle switch
};

#endif /* INCLUDED_SWITCH_H */
