/// \file Switchable.hpp Header file for the Switchable interface

/// Include guard for Switchable.hpp
#ifndef INCLUDED_SWITCHABLE_H
#define INCLUDED_SWITCHABLE_H 1

/** \brief Interface to objects that can be triggered by a switch */
class Switchable {
public:
	/** \brief Set the state of this object
	 *
	 * This sets the state of whatever is being controlled by the switch
	 * to "true" however that is defined for the specific object.
	 * \param s The state to be set
	 */
	virtual void SetState(bool s) = 0;

	/** \brief Get the state of this object
	 *
	 * This returns the state of the parameter which is controlled by the
	 * switch, whatever that may be.
	 * \return The current value of the switched state
	 */
	virtual bool GetState() const = 0;

	/** \brief Get the switch group number for this object
	 *
	 * All switchable objects that can trigger or be triggered by another
	 * object are associated based on the switch group of which they
	 * are a member.  These switch groups are each identified by a number.
	 * \return The number of the switch group for this object, or 0 if it has none
	 */
	virtual int GetSwitchGroup() const = 0;

	/** \brief Set the switch group number for this object
	 *
	 * See the description for GetSwitchGroup for an explanation of switch
	 * groups.  Note that switch groups are positive integers, passing a
	 * 0 or negative group to SetSwitchGroup will disassociate the object
	 * with the group to which it currently belongs.
	 * \param group number of the group to associate this object with
	 */
	virtual void SetSwitchGroup(const int group) = 0;
};

#endif /* INCLUDED_SWITCHABLE_H */
