/// \file TigerCub.cpp Source file for the TigerCub class

#include <sstream>

#include "SDL_gfxPrimitives.h"

#include "TigerCub.hpp"
#include "LevelWriter.hpp"

SDL_Surface *TigerCub::tigerCub;
SDL_Surface *TigerCub::tigerCubAsleep;
Mix_Chunk *TigerCub::tigerCubRoar;
unsigned int TigerCub::tigercubcount = 0;

TigerCub::TigerCub(Level *l, Vec2f p) :
	GameObject(l, p, TIGERCUB_DEFAULT_MAX_SPEED, TIGERCUB_DEFAULT_RADIUS),
   	waypoints(1, p),
	wp_index(0),
	anchorpoint(p),
	state(TIGERCUB_STATE_AWAKE),
	anim_timer(0)
{
	if (tigercubcount++ == 0) {
		tigerCub = LoadSprite(PKGDATADIR"Artwork/creatures/tiger/tiger.png",
							  CREATURE_ROTATIONS,
							  level->GetTileWidth(),
							  level->GetTileHeight());
		tigerCubAsleep = LoadSprite(PKGDATADIR"Artwork/creatures/tiger/tiger_asleep.png",
									CREATURE_ROTATIONS,
									level->GetTileWidth(),
									level->GetTileHeight());
		tigerCubRoar = LoadSound(PKGDATADIR"Sounds/TigerRoar1.ogg");
	}

	SetOrientation(0.0f);
	image = tigerCub;
}

TigerCub::~TigerCub()
{
	if (--tigercubcount == 0) {
		SDL_FreeSurface(tigerCub);
		SDL_FreeSurface(tigerCubAsleep);
		Mix_FreeChunk(tigerCubRoar);
	}
}

void TigerCub::Run()
{
	if (disabled)
	   	return;

	Charlie *c = level->GetCharlie();

	//Decide if my state/wp_index has changed
	switch(state) {
		case TIGERCUB_STATE_AGGRO:
			if((anchorpoint - position).Length() > TIGERCUB_DAY_AGGRO_RADIUS ||
				c->GetColor() == COLOR_ORANGE || c->GetInvisible())
			{
				state = TIGERCUB_STATE_DEAGGRO;
				max_speed = TIGERCUB_DEFAULT_MAX_SPEED;
			}
			break;
		case TIGERCUB_STATE_ASLEEP:
			if((c->GetPosition() - position).Length() < TIGERCUB_NIGHT_AGGRO_RADIUS &&
				c->GetColor() != COLOR_ORANGE &&
				!c->GetInvisible() && level->HasLineOfSight(position, c->GetPosition()))
			{
				state = TIGERCUB_STATE_AGGRO;
				max_speed = TIGERCUB_AGGRO_MAX_SPEED;
				anchorpoint = position;
			}
			else if(level->IsItDay())
			{
				state = TIGERCUB_STATE_AWAKE;
				wp_index = 0;
			}
			break;
		case TIGERCUB_STATE_AWAKE:
			if((c->GetPosition() - position).Length() < TIGERCUB_DAY_AGGRO_RADIUS &&
				c->GetColor() != COLOR_ORANGE &&
				!c->GetInvisible() && level->HasLineOfSight(position, c->GetPosition()) )
			{
				state = TIGERCUB_STATE_AGGRO;
				max_speed = TIGERCUB_AGGRO_MAX_SPEED;
				anchorpoint = position;
			}
			else if(!level->IsItDay())
				state = TIGERCUB_STATE_ASLEEP;
			else if(waypoints.at(wp_index).x - max_speed <= position.x && position.x <= waypoints.at(wp_index).x + max_speed
				 && waypoints.at(wp_index).y - max_speed <= position.y && position.y <= waypoints.at(wp_index).y + max_speed)
				wp_index = (wp_index + 1)%waypoints.size();
			break;
		case TIGERCUB_STATE_DEAGGRO:
			if((c->GetPosition() - anchorpoint).Length() < TIGERCUB_DAY_AGGRO_RADIUS &&
				c->GetColor() != COLOR_ORANGE &&
				!c->GetInvisible() && level->HasLineOfSight(position, c->GetPosition()))
			{
				state = TIGERCUB_STATE_AGGRO;
				max_speed = TIGERCUB_AGGRO_MAX_SPEED;
			}
			else if (anchorpoint.x - max_speed - 1 <= position.x &&
					 position.x <= anchorpoint.x + max_speed + 1 &&
					 anchorpoint.y - max_speed - 1 <= position.y &&
					 position.y <= anchorpoint.y + max_speed + 1)
				if (level->IsItDay())
				{
					state = TIGERCUB_STATE_AWAKE;
					wp_index = 0;
				}
				else
					state = TIGERCUB_STATE_ASLEEP;
			break;
		default:
			assert(0);
	}

	Vec2f goal;

	//Set goal based on my state
	switch (state) {
		case TIGERCUB_STATE_AGGRO: goal = c->GetPosition(); break;
		case TIGERCUB_STATE_ASLEEP: goal = start_position; break;
		case TIGERCUB_STATE_AWAKE: goal = waypoints.at(wp_index); break;
		case TIGERCUB_STATE_DEAGGRO: goal = anchorpoint; break;
		default: assert(0);
	}

	//Move

	velocity = level->GetShortestPath(GetPosition(), goal, max_speed);
	position += velocity;

	UpdateImage();
}

void TigerCub::Reset()
{
	this->GameObject::Reset();

	image = tigerCub;

	max_speed = TIGERCUB_DEFAULT_MAX_SPEED;
	radius = TIGERCUB_DEFAULT_RADIUS;

	wp_index = 0;
	anchorpoint = start_position;
	state = TIGERCUB_STATE_AWAKE;
	anim_index = 0;
	anim_timer = 0;
}

std::string TigerCub::GetXMLRepresentation(const std::string& indent) const
{
	std::ostringstream xml;
	xml << indent << "<creature x=\"" << level->GetTilePos(position).x + 0.5f
		<< "\" y=\"" << level->GetTilePos(position).y + 0.5f
		<< "\" type=\"tigercub\"";

	if (HasColor())
		xml << " color=\"" << GetColor().name() << "\"";
	
	std::string wptstring = LevelWriter::waypointsToXML(indent+"  ", position,
														waypoints.begin(),
														waypoints.end());
	if (wptstring.size() == 0)
		xml << "/>\n";
	else
		xml << ">\n" << wptstring << indent << "</creature>\n";

	return xml.str();
}

void TigerCub::UpdateImage()
{
	if (anim_timer++ > TIGERCUB_ANIMATION_SPEED) {
		anim_timer = 0;
		anim_index = (anim_index + 1) % 4;
	}

	if (state == TIGERCUB_STATE_ASLEEP && velocity == Vec2f(0,0))
		image = tigerCubAsleep;
	else
		image = tigerCub;
}

bool TigerCub::IsPatroller() {
	return true;
}

void TigerCub::DrawWaypoints(SDL_Surface *screen, ColorRGBA color)
{
	Vec2f first = waypoints.front();
	Vec2f prev;
	for (std::vector<Vec2f>::iterator i = waypoints.begin();
		 i != waypoints.end(); i++) {
		if (*i != first)
			aalineRGBA(screen, prev.x, prev.y, (*i).x, (*i).y,
					   color.r, color.g, color.b, color.a);

		prev = *i;
	}
	aalineRGBA(screen, prev.x, prev.y, first.x, first.y,
			   color.r, color.g, color.b, color.a);
}

void TigerCub::ResetWaypoints(Vec2f pos){
	waypoints.clear();
	waypoints.push_back(pos);
}
// vim: set ts=4 sts=4 sw=4 noet:
