/// \file TigerCub.hpp Header file for the TigerCub class

/// Include guard for TigerCub.hpp
#ifndef INCLUDED_TIGERCUB_H
#define INCLUDED_TIGERCUB_H 1

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif

#include "SoundManager.hpp"
#include "GameObject.hpp"
#include "Patroller.hpp"

/**
 * \brief Class representing a tiger cub
 */
class TigerCub : public GameObject, public Patroller {
public:
	/** \brief Constructs a TigerCub in Level l at position p.
	* \param l a pointer to the Level this TigerCub exists in
	* \param p the Vec2f start_position of this TigerCub
	*/
	TigerCub(Level *l, Vec2f p);

	/** \brief Destructs a TigerCub */
	~TigerCub();

	/** \brief Runs one cycle of this TigerCub.
	*/
	void Run(void);

	/** \brief Resets this TigerCub to its original state in its Level.
	*/
	void Reset(void);

	inline void AddWaypoint(const Vec2f waypoint,
							bool duringday = true, bool duringnight = true) 
	{ waypoints.push_back(waypoint); }

	std::string GetXMLRepresentation(const std::string& indent) const;

	/** \brief Gets the GameObjectType of this TigerCub.
	 * \return the GameObjectType of this TigerCub
	*/
	inline GameObjectType GetType(void) const { return GAME_OBJECT_TIGERCUB; };

	void PlayCollisionSound() const;

	/** \brief draws the creature's waypoints
	 *
	 * Draws the creature's waypoints as dots and path between them as lines
	 */
	void DrawWaypoints(SDL_Surface *screen, ColorRGBA color);

	/** \brief resets creature's waypoints
	 *
	 * Clears all the creature's waypoints except for the origin
	 */
	void ResetWaypoints(Vec2f pos);

	bool IsPatroller();

private:
	/** \brief Updates the image of this TigerCub.
	*/
	void UpdateImage(void);

	static SDL_Surface *tigerCub;	///< Front-view image of this TigerCub
	static SDL_Surface *tigerCubAsleep;	///< sleeping image of this TigerCub
	static Mix_Chunk *tigerCubRoar;	///< Sound of the TigerCub roaring
	static unsigned int tigercubcount;	///< Number of TigerCubs created

	std::vector<Vec2f> waypoints;	///< The waypoints of this TigerCub
	int wp_index;					///< The current index of this TigerCub's waypoints
	Vec2f anchorpoint;				///< The anchorpoint of the Tiger; set when it aggros, returns here when it deaggros.
	/// \brief Describes the mental state of a TigerCub
	enum TigerCubState {
		TIGERCUB_STATE_AGGRO,
		TIGERCUB_STATE_ASLEEP,
		TIGERCUB_STATE_AWAKE,
		TIGERCUB_STATE_DEAGGRO
	};

	TigerCubState state;			///< The mental state of this TigerCub
	unsigned int anim_timer;		///< How long the Tiger has to pause before moving to the next animation frame
};

inline void TigerCub::PlayCollisionSound() const
{
	SoundManager::PlayChunk(tigerCubRoar);
}

#endif /* INCLUDED_TIGERCUB_H */
// vim: set ts=4 sts=4 sw=4 noet:
