/// \file Tiles.cpp Source file for the Tile class and sub-classes

#include <fstream>
#include <sstream>
#include <typeinfo>

#include "Tiles.hpp"

/** The alpha value that should be applied to the color of Tiles as they are
 * drawn */
static const Uint8 TILE_SHADING_ALPHA = 100;

Tile::Tile(const int nx, const int ny, const Vec2i dim) :
	x(nx),
	y(ny),
	dimension(dim),
	hascolor(false),
	colorshading(NULL)
{
}

Tile::Tile(const int nx, const int ny, const Vec2i dim, const ColorRGBA col) :
	x(nx),
	y(ny),
	dimension(dim),
	hascolor(true),
	color(col)
{
	SDL_Surface *screen = SDL_GetVideoSurface();
	colorshading = SDL_CreateRGBSurface(screen->flags, dim.x, dim.y,
										screen->format->BitsPerPixel,
										screen->format->Rmask,
										screen->format->Gmask,
										screen->format->Bmask,
										screen->format->Amask);
	SDL_FillRect(colorshading, NULL, SDL_MapRGB(colorshading->format,
												color.r, color.g, color.b));
	SDL_SetAlpha(colorshading, SDL_SRCALPHA|SDL_RLEACCEL, TILE_SHADING_ALPHA);
}

std::string Tile::GetXMLRepresentation(const std::string& indent) const
{
	std::string tiletypeattr;
	if (typeid(*this) == typeid(BridgeTile))
		tiletypeattr = "bridge";
	else if (typeid(*this) == typeid(LandTile))
		tiletypeattr = "land";
	else if (typeid(*this) == typeid(RockTile))
		tiletypeattr = "rock";
	else if (typeid(*this) == typeid(TreeTile))
		tiletypeattr = "tree";
	else if (typeid(*this) == typeid(WaterTile))
		tiletypeattr = "water";

	if (tiletypeattr.length() == 0)
		return indent + "<!-- " + typeid(*this).name() + " not saved -->\n";
	else {
		std::ostringstream xml(indent);
		xml << indent
			<< "<tile row=\"" << y << "\" col=\"" << x << "\" type=\""
			<< tiletypeattr << "\"";
		if (hascolor)
			xml << " color=\"" << color.name() << "\"";
		xml << "/>\n";
		return xml.str();
	}
}

SDL_Surface *EditorTile::charlieTileImage;		
SDL_Surface *EditorTile::beeTileImage;			
SDL_Surface *EditorTile::tigercubTileImage;		
SDL_Surface *EditorTile::snakeTileImage;			
SDL_Surface *EditorTile::centipedeTileImage;		
SDL_Surface *EditorTile::batTileImage;
SDL_Surface *EditorTile::butterflyTileImage;
SDL_Surface *EditorTile::scorpionTileImage;
SDL_Surface *EditorTile::millipedeTileImage;
SDL_Surface *EditorTile::landTileImage;			
SDL_Surface *EditorTile::waterTileImage;			
SDL_Surface *EditorTile::rockTileImage;			
SDL_Surface *EditorTile::treeTileImage;			
SDL_Surface *EditorTile::bridgeTileImage;			
SDL_Surface *EditorTile::flowerTileImage;			
SDL_Surface *EditorTile::switchTileImage;			
SDL_Surface *EditorTile::doorTileImage;			
SDL_Surface *EditorTile::goalTileImage;			
unsigned int EditorTile::editortilecount = 0;

EditorTile::EditorTile(const unsigned int xn, const unsigned int yn, const Vec2i dim): Tile(xn, yn, dim)
{
	// Initialize the images if we have not yet created a BridgeTile
	if (editortilecount++ == 0) {
		charlieTileImage = LoadImageSDL(PKGDATADIR"Artwork/creatures/charlie/charlie-green.png");
		beeTileImage = LoadImageSDL(PKGDATADIR"Artwork/creatures/bee/bee-yellow.png");
		tigercubTileImage = LoadImageSDL(PKGDATADIR"Artwork/creatures/tiger/tiger.png");
		snakeTileImage = LoadImageSDL(PKGDATADIR"Artwork/creatures/snake/snake_asleep-green.png");
		centipedeTileImage = LoadImageSDL(PKGDATADIR"Artwork/creatures/rolly_polly/rolly_polly.png");
		batTileImage = LoadImageSDL(PKGDATADIR"Artwork/creatures/bat/bat.png");
		butterflyTileImage = LoadImageSDL(PKGDATADIR"Artwork/objects/butterfly/butterfly_day.png");
		scorpionTileImage = LoadImageSDL(PKGDATADIR"Artwork/creatures/scorpion/scorpion.png");
		millipedeTileImage = LoadImageSDL(PKGDATADIR"Artwork/creatures/centipede/centipede_rolledup.png");
		landTileImage = LoadImageSDL(PKGDATADIR"Artwork/tiles/land/land1.png");
		waterTileImage = LoadImageSDL(PKGDATADIR"Artwork/tiles/lake/lake_cc.png");
		rockTileImage = LoadImageSDL(PKGDATADIR"Artwork/tiles/rock/rock.png");
		treeTileImage = LoadImageSDL(PKGDATADIR"Artwork/tiles/tree/tree.png");
		bridgeTileImage = LoadImageSDL(PKGDATADIR"Artwork/tiles/bridge/bridge_center.png");
		flowerTileImage = LoadImageSDL(PKGDATADIR"Artwork/objects/flower/flower-orange.png");
		switchTileImage = LoadImageSDL(PKGDATADIR"Artwork/objects/switches/switch.png");
		doorTileImage = LoadImageSDL(PKGDATADIR"Artwork/objects/leafdoor/leafdoor.png");
		goalTileImage = LoadImageSDL(PKGDATADIR"Artwork/creatures/goal/goal.png");
	}

	image = landTileImage;
	hascolor = false;
	passable = true;
	rotation = 0;
}

EditorTile::~EditorTile()
{
	if (--editortilecount == 0) {
		SDL_FreeSurface(charlieTileImage);
		SDL_FreeSurface(beeTileImage);
		SDL_FreeSurface(tigercubTileImage);
		SDL_FreeSurface(snakeTileImage);
		SDL_FreeSurface(centipedeTileImage);
		SDL_FreeSurface(batTileImage);
		SDL_FreeSurface(butterflyTileImage);
		SDL_FreeSurface(scorpionTileImage);
		SDL_FreeSurface(millipedeTileImage);
		SDL_FreeSurface(landTileImage);
		SDL_FreeSurface(waterTileImage);
		SDL_FreeSurface(rockTileImage);
		SDL_FreeSurface(treeTileImage);
		SDL_FreeSurface(bridgeTileImage);
		SDL_FreeSurface(flowerTileImage);
		SDL_FreeSurface(switchTileImage);
		SDL_FreeSurface(doorTileImage);
		SDL_FreeSurface(goalTileImage);
	}
}

void EditorTile::SetSurroundingTiles(const Tile *above, const Tile *right,
								   const Tile *below, const Tile *left)
{
	// No change to land tiles
}

bool EditorTile::IsCollidingWith(const GameObject *const go) const
{
	return false;
}

SDL_Surface *BridgeTile::bridgeConnectorVert;
SDL_Surface *BridgeTile::bridgeEndTop;
unsigned int BridgeTile::bridgetilecount = 0;

BridgeTile::BridgeTile(const unsigned int xn, const unsigned int yn, const Vec2i dim): Tile(xn, yn, dim)
{
	// Initialize the images if we have not yet created a BridgeTile
	if (bridgetilecount++ == 0) {
		bridgeConnectorVert = LoadSprite(PKGDATADIR"Artwork/tiles/bridge/bridge_center.png",
										 TILE_ROTATIONS,
										 dim.x,
										 dim.y);
		bridgeEndTop = LoadSprite(PKGDATADIR"Artwork/tiles/bridge/bridge_edge.png",
								  TILE_ROTATIONS,
								  dim.x,
								  dim.y);
	}

	image = bridgeConnectorVert;
	passable = true;
	seethrough = true;
}

BridgeTile::BridgeTile(const unsigned int xn, const unsigned int yn, const Vec2i dim, ColorRGBA c): Tile(xn, yn, dim, c)
{
	// Initialize the images if we have not yet created a BridgeTile
	if (bridgetilecount++ == 0) {
		bridgeConnectorVert = LoadSprite(PKGDATADIR"Artwork/tiles/bridge/bridge_center.png",
										 1,
										 dim.x,
										 dim.y);
		bridgeEndTop = LoadSprite(PKGDATADIR"Artwork/tiles/bridge/bridge_edge.png",
								  1,
								  dim.x,
								  dim.y);
	}

	image = bridgeConnectorVert;
	passable = true;
	seethrough = true;
}

BridgeTile::~BridgeTile()
{
	if (--bridgetilecount == 0) {
		SDL_FreeSurface(bridgeConnectorVert);
		SDL_FreeSurface(bridgeEndTop);
	}
}

void BridgeTile::SetSurroundingTiles(const Tile *above, const Tile *right,
									 const Tile *below, const Tile *left)
{
	if (above &&
		typeid(*above) == typeid(LandTile) ||
		typeid(*above) == typeid(TreeTile) ||
		typeid(*above) == typeid(RockTile)) {
		image = bridgeEndTop;
		rotation = 0;
	} else if (right &&
			   typeid(*right) == typeid(LandTile) ||
			   typeid(*right) == typeid(TreeTile) ||
			   typeid(*right) == typeid(RockTile)) {
		image = bridgeEndTop;
		rotation = 3;
	} else if (below &&
			   typeid(*below) == typeid(LandTile) ||
			   typeid(*below) == typeid(TreeTile) ||
			   typeid(*below) == typeid(RockTile)) {
		image = bridgeEndTop;
		rotation = 2;
	} else if (left &&
			   typeid(*left) == typeid(LandTile) ||
			   typeid(*left) == typeid(TreeTile) ||
			   typeid(*left) == typeid(RockTile)) {
		image = bridgeEndTop;
		rotation = 1;
	} else if ((above && typeid(*above) == typeid(BridgeTile)) ||
			   (below && typeid(*below) == typeid(BridgeTile))) {
		image = bridgeConnectorVert;
		rotation = 0;
	} else {
		image = bridgeConnectorVert;
		rotation = 1;
	}
}

bool BridgeTile::IsCollidingWith(const GameObject *const go) const
{
	return false;
}

SDL_Surface *LandTile::land1;
SDL_Surface *LandTile::land2;
SDL_Surface *LandTile::land3;
SDL_Surface *LandTile::land4;
unsigned int LandTile::landtilecount = 0;

LandTile::LandTile(const unsigned int xn, const unsigned int yn, const Vec2i dim): Tile(xn, yn, dim)
{
	// Initialize the images if we have not yet created a LandTile
	if (landtilecount++ == 0) {
		land1 = LoadSprite(PKGDATADIR"Artwork/tiles/land/land1.png",
						   1,
						   dim.x,
						   dim.y);
		land2 = LoadSprite(PKGDATADIR"Artwork/tiles/land/land2.png",
						   1,
						   dim.x,
						   dim.y);
		land3 = LoadSprite(PKGDATADIR"Artwork/tiles/land/land3.png",
						   1,
						   dim.x,
						   dim.y);
		land4 = LoadSprite(PKGDATADIR"Artwork/tiles/land/land4.png",
						   1,
						   dim.x,
						   dim.y);
	}

	switch (GetRandomInt(1,4)) {
		case 1: image = land1; break;
		case 2: image = land2; break;
		case 3: image = land3; break;
		case 4: image = land4; break;
		default: assert(0);
	}
	rotation = 0;
	passable = true;
	seethrough = true;
}

LandTile::LandTile(const unsigned int xn, const unsigned int yn, const Vec2i dim, ColorRGBA c): Tile(xn, yn, dim, c)
{
	// Initialize the images if we have not yet created a LandTile
	if (landtilecount++ == 0) {
		land1 = LoadSprite(PKGDATADIR"Artwork/tiles/land/land1.png",
						   1,
						   dim.x,
						   dim.y);
		land2 = LoadSprite(PKGDATADIR"Artwork/tiles/land/land2.png",
						   1,
						   dim.x,
						   dim.y);
		land3 = LoadSprite(PKGDATADIR"Artwork/tiles/land/land3.png",
						   1,
						   dim.x,
						   dim.y);
		land4 = LoadSprite(PKGDATADIR"Artwork/tiles/land/land4.png",
						   1,
						   dim.x,
						   dim.y);
	}

	switch (GetRandomInt(1,4)) {
		case 1: image = land1; break;
		case 2: image = land2; break;
		case 3: image = land3; break;
		case 4: image = land4; break;
		default: assert(0);
	}
	rotation = 0;
	passable = true;
	seethrough = true;
}

LandTile::~LandTile()
{
	if (--landtilecount == 0) {
		SDL_FreeSurface(land1);
		SDL_FreeSurface(land2);
		SDL_FreeSurface(land3);
		SDL_FreeSurface(land4);
	}
}

void LandTile::SetSurroundingTiles(const Tile *above, const Tile *right,
								   const Tile *below, const Tile *left)
{
	// No change to land tiles
}

bool LandTile::IsCollidingWith(const GameObject *const go) const
{
	return false;
}

SDL_Surface *TreeTile::tree;
unsigned int TreeTile::treetilecount = 0;

TreeTile::TreeTile(const unsigned int xn, const unsigned int yn, const Vec2i dim): Tile(xn, yn, dim)
{
	// Initialize the images if we have not yet created a TreeTile
	if (treetilecount++ == 0) {
		tree = LoadSprite(PKGDATADIR"Artwork/tiles/tree/tree.png",
						  1,
						  dim.x,
						  dim.y);
	}

	image = tree;
	passable = false;
	rotation = 0;
	seethrough = false;
}

TreeTile::TreeTile(const unsigned int xn, const unsigned int yn, const Vec2i dim, ColorRGBA c): Tile(xn, yn, dim, c)
{
	// Initialize the images if we have not yet created a TreeTile
	if (treetilecount++ == 0) {
		tree = LoadSprite(PKGDATADIR"Artwork/tiles/tree/tree.png",
						  1,
						  dim.x,
						  dim.y);
	}

	image = tree;
	passable = false;
	hascolor = true;
	rotation = 0;
}

TreeTile::~TreeTile()
{
	if (--treetilecount == 0)
		SDL_FreeSurface(tree);
}

void TreeTile::SetSurroundingTiles(const Tile *above, const Tile *right,
									 const Tile *below, const Tile *left)
{
	// No change
}

bool TreeTile::IsCollidingWith(const GameObject *const go) const
{
	return true;
}

SDL_Surface *RockTile::rock;
unsigned int RockTile::rocktilecount = 0;

RockTile::RockTile(const unsigned int xn, const unsigned int yn, const Vec2i dim): Tile(xn, yn, dim)
{
	// Initialize the images if we have not yet created a RockTile
	if (rocktilecount++ == 0) {
		rock = LoadSprite(PKGDATADIR"Artwork/tiles/rock/rock.png",
						  1,
						  dim.x,
						  dim.y);
	}

	image = rock;
	rotation = 0;
	passable = false;
	seethrough = false;
}

RockTile::RockTile(const unsigned int xn, const unsigned int yn, const Vec2i dim, ColorRGBA c): Tile(xn, yn, dim, c)
{
	// Initialize the images if we have not yet created a RockTile
	if (rocktilecount++ == 0) {
		rock = LoadSprite(PKGDATADIR"Artwork/tiles/rock/rock.png",
						  1,
						  dim.x,
						  dim.y);
	}

	image = rock;
	rotation = 0;
	passable = false;
	seethrough = false;
}

RockTile::~RockTile()
{
	if (--rocktilecount == 0)
		SDL_FreeSurface(rock);
}

void RockTile::SetSurroundingTiles(const Tile *above, const Tile *right,
								   const Tile *below, const Tile *left)
{
	// No change to rock tiles
}

bool RockTile::IsCollidingWith(const GameObject *const go) const
{
	return true;
}

// FIXME:  Get new rotation sheets in here
SDL_Surface *WaterTile::lakeNoEdge;
SDL_Surface *WaterTile::lakeTopEdge;
SDL_Surface *WaterTile::lakeTopRightEdge;
SDL_Surface *WaterTile::lakeTopBottomEdge;
SDL_Surface *WaterTile::lakeRightLeftEdge;
SDL_Surface *WaterTile::lakeTopRightBottomEdge;
SDL_Surface *WaterTile::lakeRightBottomLeftEdge;
SDL_Surface *WaterTile::lakeAllEdge;
unsigned int WaterTile::watertilecount = 0;

WaterTile::ImagePair WaterTile::imagepairs[2][2][2][2] =
	{{{{{&lakeAllEdge, 0},					// Top, Right, below, Left
	    {&lakeTopRightBottomEdge, 0}},		// Top, Right, below
	   {{&lakeRightBottomLeftEdge, 2},		// Top, Right, Left
	    {&lakeTopRightEdge, 0}}},			// Top, Right
	  {{{&lakeTopRightBottomEdge, 2},		// Top, below, Left
	    {&lakeTopBottomEdge, 0}},			// Top, below
	   {{&lakeTopRightEdge, 3},				// Top, Left
	    {&lakeTopEdge, 0}}}},				// Top
	 {{{{&lakeRightBottomLeftEdge, 0},		// Right, below, Left
	    {&lakeTopRightEdge, 1}},			// Right, below
	   {{&lakeRightLeftEdge, 0},			// Right, Left
	    {&lakeTopEdge, 1}}},				// Right
	  {{{&lakeTopRightEdge, 2},				// below, Left
	    {&lakeTopEdge, 2}},					// below
	   {{&lakeTopEdge, 3},					// Left
		{&lakeNoEdge, 0}}}}};				// None

WaterTile::WaterTile(const unsigned int xn, const unsigned int yn,
					 const Vec2i dim) :
	Tile(xn, yn, dim)
{
	// Initialize the images if we have not yet created a WaterTile
	if (watertilecount++ == 0) {
		lakeNoEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_cc.png",
							    TILE_ROTATIONS,
							    dim.x,
							    dim.y);
		lakeTopEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_tc.png",
								 TILE_ROTATIONS,
								 dim.x,
								 dim.y);
		lakeTopRightEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_tr.png",
									  TILE_ROTATIONS,
									  dim.x,
									  dim.y);
		lakeTopBottomEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_2c.png",
									   TILE_ROTATIONS,
									   dim.x,
									   dim.y);
		lakeRightLeftEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_3c.png",
									   TILE_ROTATIONS,
									   dim.x,
									   dim.y);
		lakeTopRightBottomEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_2r.png",
											TILE_ROTATIONS,
											dim.x,
											dim.y);
		lakeRightBottomLeftEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_3b.png",
											 TILE_ROTATIONS,
											 dim.x,
											 dim.y);
		lakeAllEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_1.png",
							     TILE_ROTATIONS,
								 dim.x,
								 dim.y);
	}

	image = lakeNoEdge;
	rotation = 0;
	passable = false;
	seethrough = true;
}

WaterTile::WaterTile(const unsigned int xn, const unsigned int yn,
					 const Vec2i dim, ColorRGBA c) :
	Tile(xn, yn, dim, c)
{
	// Initialize the images if we have not yet created a WaterTile
	if (watertilecount++ == 0) {
		lakeNoEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_cc.png",
							    TILE_ROTATIONS,
							    dim.x,
							    dim.y);
		lakeTopEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_tc.png",
								 TILE_ROTATIONS,
								 dim.x,
								 dim.y);
		lakeTopRightEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_tr.png",
									  TILE_ROTATIONS,
									  dim.x,
									  dim.y);
		lakeTopBottomEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_2c.png",
									   TILE_ROTATIONS,
									   dim.x,
									   dim.y);
		lakeRightLeftEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_3c.png",
									   TILE_ROTATIONS,
									   dim.x,
									   dim.y);
		lakeTopRightBottomEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_2r.png",
											TILE_ROTATIONS,
											dim.x,
											dim.y);
		lakeRightBottomLeftEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_3b.png",
											 TILE_ROTATIONS,
											 dim.x,
											 dim.y);
		lakeAllEdge = LoadSprite(PKGDATADIR"Artwork/tiles/lake/lake_1.png",
							     TILE_ROTATIONS,
								 dim.x,
								 dim.y);
	}

	image = lakeNoEdge;
	rotation = 0;
	passable = false;
	seethrough = true;
}

WaterTile::~WaterTile()
{
	if (--watertilecount == 0) {
		SDL_FreeSurface(lakeNoEdge);
		SDL_FreeSurface(lakeTopEdge);
		SDL_FreeSurface(lakeTopRightEdge);
		SDL_FreeSurface(lakeTopBottomEdge);
		SDL_FreeSurface(lakeRightLeftEdge);
		SDL_FreeSurface(lakeTopRightBottomEdge);
		SDL_FreeSurface(lakeRightBottomLeftEdge);
		SDL_FreeSurface(lakeAllEdge);
	}
}

void WaterTile::SetSurroundingTiles(const Tile *above, const Tile *right,
									const Tile *below, const Tile *left)
{
	int aboveis = !above ||
				  typeid(*above) == typeid(WaterTile) ||
				  typeid(*above) == typeid(BridgeTile);
	int rightis = !right ||
				  typeid(*right) == typeid(WaterTile) ||
				  typeid(*right) == typeid(BridgeTile);
	int belowis = !below ||
				  typeid(*below) == typeid(WaterTile) ||
				  typeid(*below) == typeid(BridgeTile);
	int leftis = !left ||
				 typeid(*left) == typeid(WaterTile) ||
				 typeid(*left) == typeid(BridgeTile);

	image = *imagepairs[aboveis][rightis][belowis][leftis].img;
	rotation = imagepairs[aboveis][rightis][belowis][leftis].rotation;
}

bool WaterTile::IsCollidingWith(const GameObject *const go) const
{
	if (image == lakeTopRightEdge) {
		Vec2f rel_pos = Vec2f( go->GetPosition().x - x, go->GetPosition().y - y);
		switch(rotation){
			case 0:
				return rel_pos.Distance(Vec2f(0,1)) < 1;
			case 1:
				return rel_pos.Distance(Vec2f(1,1)) < 1;
			case 2:
				return rel_pos.Distance(Vec2f(1,0)) < 1;
			case 3:
				return rel_pos.Distance(Vec2f(0,0)) < 1;
		}
	}
	return true;
}

// vim: set ts=4 sts=4 sw=4 noet:
