/// \file Tiles.hpp Header file for the Tile class and sub-classes

/// Include guard for Tiles.hpp
#ifndef INCLUDED_TILES_H
#define INCLUDED_TILES_H 1

#include "Saveable.hpp"
#include "Button.hpp"

class GameObject;

/**
 * \brief Representation of a single tile on the game level
 *	
 * Each tile will contain a vector of pointers to all the
 * gameobjects that is on the tile, the image representing
 * the tile's background, and wether this tile is passable.
 */
class Tile : public Saveable {
public:
	/** \brief Destroy a tile */
	virtual ~Tile() {};

	/** \brief Set the tiles surrounding this Tile
	 *
	 * This function is called whenever an adjacent Tile is changed.  It allows
	 * this Tile to change or reorient its image as necessary.
	 *
	 * \param above The Tile above this tile (smaller y/row value)
	 * \param right The Tile to the right of this tile (larger x/col value)
	 * \param below The Tile below this tile (larger y/row value)
	 * \param left The Tile to the left of this tile (smaller x/col value)
	 */
	virtual void SetSurroundingTiles(const Tile *above, const Tile *right,
									 const Tile *below, const Tile *left) =0;

	/** \brief Add a GameObject to this Tile
	 * This function is called whenever a GameObject enters this Tile.
	 * \param go The GameObject entering this Tile
	 */
	void AddObject(GameObject *go);

	/** \brief Remove a GameObject from this Tile
	 * This function is called whenever a GameObject leaves this Tile.
	 * \param go The GameObject leaving this Tile
	 */
	void RemoveObject(const GameObject *go);

	/** \brief Is a certain GameObject on this Tile
	 * \param go The GameObject to check for
	 * \return Is the GameObject on this tile?
	 */
	bool ContainsObject(GameObject *go) const;

	/** \brief Remove all GameObjects from this Tile
	 * This function is used to reset this Tile to an empty state
	 */
	void RemoveAllObjects();

	/** \brief Does this Tile have an associated color?
	 * \return true if this Tile has a color, false otherwise
	 */
	inline bool HasColor() const { return hascolor; }

	/** \brief Get the canonical color of this Tile
	 * \return Color of this Tile with alpha removed
	 *         (or black with full alpha if no color for this tile)
	 */
	inline ColorRGBA GetColor() const;

	virtual std::string GetXMLRepresentation(const std::string& indent = "") const;

	/** \brief Draw this tile
	 * \param x The x coordinate of this Tile's upper-left corner (in pixels)
	 * \param y The y coordinate of this Tile's upper-left corner (in pixels)
	 * \param dimension The width and height of this tile (in pixels)
	 */
	void Draw(SDL_Surface *screen, const int x, const int y) const;

	/** \brief Is a certain GameObject colliding with this Tile?
	 *
	 * This function provides Tile-specific collision checking
	 * \param go The GameObject to check for collision
	 * \return Is the GameObject on a forbidden part of the Tile?
	 */
	virtual bool IsCollidingWith(const GameObject *const go) const =0;

	// Temporarily public, to ease integration
    std::vector<GameObject *> objects;	///< GameObjects currently on this Tile
	bool passable;						///< Is this Tile (mostly) passable?
	bool seethrough;					///< Is this Tile see-through?

protected:
	/** \brief Create a Tile at location (nx,ny) with dimension dim
	 * \param nx The x coordinate of the left edge of this Tile
	 * \param ny The y coordinate of the top edge of this Tile
	 * \param dim The width and height of this Tile
	 */
	Tile(const int nx, const int ny, const Vec2i dim);

	/** \brief Create a Tile at location (nx,ny) with dimension dim and color
	 * \param nx The x coordinate of the left edge of this Tile
	 * \param ny The y coordinate of the top edge of this Tile
	 * \param dim The width and height of this Tile
	 * \param color The color of this Tile
	 */
	Tile(const int nx, const int ny, const Vec2i dim, const ColorRGBA color);

	int x;				///< The x-coordinate of the left edge of this Tile
	int y;				///< The y-coordinate of the top edge of this Tile
	Vec2i dimension;	///< The width and height of this Tile

    mutable SDL_Surface *image;		///< image representing this Tile
	SDL_Surface *colorshading;		///< shading for tile color (NULL if none)
	ColorRGBA color;				///< color of this Tile
	bool hascolor;					///< Does this Tile have a color?

	int rotation;					///< rotation angle for the image / 90
};

inline void Tile::Draw(SDL_Surface *screen, const int x, const int y) const
{
	SDL_Rect clip, offset;
	clip.x = 0;
	clip.y = rotation*dimension.y;
	clip.w = dimension.x;
	clip.h = dimension.y;
	offset.x = x;
	offset.y = y;
	SDL_BlitSurface(image, &clip, screen, &offset);

	if (hascolor)
		SDL_BlitSurface(colorshading, &clip, screen, &offset);
}

inline void Tile::AddObject(GameObject *go)
{
	objects.push_back(go);
}

inline void Tile::RemoveObject(const GameObject *go)
{
	for (std::vector<GameObject*>::iterator k = objects.begin();
		 k != objects.end(); k++){
		if((*k) == go){
			objects.erase(k);
			break;
		}
	}
}

inline void Tile::RemoveAllObjects()
{
	objects.clear();
}

inline ColorRGBA Tile::GetColor() const
{
	if (colorshading == NULL)
		return ColorRGBA(0,0,0,0);

	ColorRGBA c = color;
	c.a = 0;
	return c;
}

class EditorTile : public Tile {
public:
	/** \brief Create a EditorTile
	 *
	 * \param x The x position of the left edge of this EditorTile (in px)
	 * \param y The y position of the top edge of this EditorTile (in px)
	 * \param dim The width and height of this EditorTile (in px)
	 * \param type The type of the editor tile
	 */
	EditorTile(const unsigned int x, const unsigned int y, const Vec2i dim);

	/** \brief Destroy this EditorTile */
	~EditorTile();

	enum TileState {
		TILE_STATE_DEFAULT,		/**< Default state of the tile */
		TILE_STATE_HOVER,		/**< When mouse is hovered over tile */
		TILE_STATE_HOVER_SEL,	/**< When mouse is hovered over a selected tile */
		TILE_STATE_SELECTED		/**< When mouse is pressed over tile */
	};

	bool IsTileStateDefault() const;
	bool IsTileStateHover() const;
	bool IsTileStateSelected() const;
	bool IsTileStateHoverSel() const;
	void SetTileStateDefault();
	void SetTileStateHover();
	void SetTileStateSelected();
	void SetTileType(Button::ButtonType type);
	bool OnTile(const int mx, const int my) const;

	void SetSurroundingTiles(const Tile *above, const Tile *right,
							 const Tile *below, const Tile *left);

	bool IsCollidingWith(const GameObject *const go) const;

	/** \brief Draw this tile
	 * \param x The x coordinate of this Tile's upper-left corner (in pixels)
	 * \param y The y coordinate of this Tile's upper-left corner (in pixels)
	 * \param dimension The width and height of this tile (in pixels)
	 */
	void Draw(SDL_Surface *screen, const int x, const int y) const;

private:
	static SDL_Surface *charlieTileImage;		///< Image of the EditorTile
	static SDL_Surface *beeTileImage;			///< Image of the EditorTile
	static SDL_Surface *tigercubTileImage;		///< Image of the EditorTile
	static SDL_Surface *snakeTileImage;			///< Image of the EditorTile
	static SDL_Surface *centipedeTileImage;		///< Image of the EditorTile
	static SDL_Surface *batTileImage;			///< Image of the EditorTile
	static SDL_Surface *butterflyTileImage;		///< Image of the EditorTile
	static SDL_Surface *scorpionTileImage;		///< Image of the EditorTile
	static SDL_Surface *millipedeTileImage;		///< Image of the EditorTile
	static SDL_Surface *landTileImage;			///< Image of the EditorTile
	static SDL_Surface *waterTileImage;			///< Image of the EditorTile
	static SDL_Surface *rockTileImage;			///< Image of the EditorTile
	static SDL_Surface *treeTileImage;			///< Image of the EditorTile
	static SDL_Surface *bridgeTileImage;			///< Image of the EditorTile
	static SDL_Surface *flowerTileImage;			///< Image of the EditorTile
	static SDL_Surface *switchTileImage;			///< Image of the EditorTile
	static SDL_Surface *doorTileImage;			///< Image of the EditorTile
	static SDL_Surface *goalTileImage;			///< Image of the EditorTile
	static unsigned int editortilecount;	///< Number of EditorTiles created
	mutable TileState tstate;
	mutable Button::ButtonType btype;
};

inline bool EditorTile::IsTileStateDefault() const { return tstate == TILE_STATE_DEFAULT; }
inline bool EditorTile::IsTileStateHover() const  { return tstate == TILE_STATE_HOVER; }
inline bool EditorTile::IsTileStateHoverSel() const  { return tstate == TILE_STATE_HOVER_SEL; }
inline bool EditorTile::IsTileStateSelected() const { return tstate == TILE_STATE_SELECTED; }
inline void EditorTile::SetTileStateDefault() { if (!IsTileStateSelected() && !IsTileStateHoverSel()) tstate = TILE_STATE_DEFAULT; else tstate = TILE_STATE_SELECTED; }
inline void EditorTile::SetTileStateHover() { if (!IsTileStateSelected()) tstate = TILE_STATE_HOVER; else tstate = TILE_STATE_HOVER_SEL;}
inline void EditorTile::SetTileStateSelected() { tstate = TILE_STATE_SELECTED; }
inline void EditorTile::SetTileType(Button::ButtonType type) { btype = type; }
inline bool EditorTile::OnTile(const int mx, const int my) const
{
	return mx >= x*dimension.x && mx <= (x+1)*dimension.x &&
		   my >= y*dimension.y && my <= (y+1)*dimension.y;
}

inline void EditorTile::Draw(SDL_Surface *screen,
							 const int x, const int y) const
{
	switch (btype) {
		case Button::GAME_OBJECT_BAT:
			image = batTileImage;
			break;
		case Button::GAME_OBJECT_BEE:
			image = beeTileImage;
			break;
		case Button::GAME_OBJECT_BUTTERFLY:
			image = butterflyTileImage;
			break;
		case Button::GAME_OBJECT_CENTIPEDE:
			image = centipedeTileImage;
			break;
		case Button::GAME_OBJECT_CHARLIE:
			image = charlieTileImage;
			break;
		case Button::GAME_OBJECT_DOOR:
			image = doorTileImage;
			break;
		case Button::GAME_OBJECT_FLOWER:
			image = flowerTileImage;
			break;
		case Button::GAME_OBJECT_GOAL:
			image = goalTileImage;
			break;
		case Button::GAME_OBJECT_MILLIPEDE:
			image = millipedeTileImage;
			break;
		case Button::GAME_OBJECT_SCORPION:
			image = scorpionTileImage;
			break;
		case Button::GAME_OBJECT_SNAKE:
			image = snakeTileImage;
			break;
		case Button::GAME_OBJECT_SWITCH:
			image = switchTileImage;
			break;
		case Button::GAME_OBJECT_TIGERCUB:
			image = tigercubTileImage;
			break;
		case Button::TILE_BRIDGE:
			image = bridgeTileImage;
			break;
		case Button::TILE_LAND:
			image = landTileImage;
			break;
		case Button::TILE_ROCK:
			image = rockTileImage;
			break;
		case Button::TILE_TREE:
			image = treeTileImage;
			break;
		case Button::TILE_WATER:
			image = waterTileImage;
			break;
		default:
			image = landTileImage;
			break;
	}

	SDL_Rect clip, offset;
	clip.x = 0;
	clip.y = 0;
	clip.w = 32;
	clip.h = 32;
	offset.x = x;
	offset.y = y;
	SDL_BlitSurface(image, &clip, screen, &offset);

	switch (tstate) {
		case TILE_STATE_DEFAULT:
			break;
		case TILE_STATE_HOVER:
		case TILE_STATE_HOVER_SEL:
			{
				SDL_Surface *hilight;
				hilight = SDL_CreateRGBSurface(screen->flags, 32, 32,
											   screen->format->BitsPerPixel,
											   screen->format->Rmask,
											   screen->format->Gmask,
											   screen->format->Bmask,
											   screen->format->Amask);
				SDL_FillRect(hilight, NULL,
							 SDL_MapRGB(hilight->format, 0, 0, 0));
				SDL_Rect interior;
				interior.x = 2;
				interior.y = 2;
				interior.w = 60;
				interior.h = 60;
				SDL_FillRect(hilight, &interior,
							 SDL_MapRGBA(hilight->format, 0, 0, 0,
										 SDL_ALPHA_TRANSPARENT));
				SDL_BlitSurface(hilight, NULL, screen, &offset);
				SDL_FreeSurface(hilight);
			}
			break;
		case TILE_STATE_SELECTED:
			{
				SDL_Surface *hilight;
				hilight = SDL_CreateRGBSurface(screen->flags, 32, 32,
											   screen->format->BitsPerPixel,
											   screen->format->Rmask,
											   screen->format->Gmask,
											   screen->format->Bmask,
											   screen->format->Amask);
				SDL_FillRect(hilight, NULL,
							 SDL_MapRGB(hilight->format, 100, 0, 50));
				SDL_Rect interior;
				interior.x = 2;
				interior.y = 2;
				interior.w = 60;
				interior.h = 60;
				SDL_FillRect(hilight, &interior,
							 SDL_MapRGBA(hilight->format, 0, 0, 0,
										 SDL_ALPHA_TRANSPARENT));
				SDL_BlitSurface(hilight, NULL, screen, &offset);
				SDL_FreeSurface(hilight);
			}
			break;
	}

	if (tstate == TILE_STATE_HOVER)
		tstate = TILE_STATE_DEFAULT;
	btype = Button::TILE_LAND;
}

/** \brief Representation of a Tile of a bridge */
class BridgeTile : public Tile {
public:
	/** \brief Create a BridgeTile
	 *
	 * \param x The x position of the left edge of this BridgeTile (in px)
	 * \param y The y position of the top edge of this BridgeTile (in px)
	 * \param dim The width and height of this BridgeTile (in px)
	 */
	BridgeTile(const unsigned int x, const unsigned int y, const Vec2i dim);

	/** \brief Create a BridgeTile with an associated color
	 *
	 * \param x The x position of the left edge of this BridgeTile (in px)
	 * \param y The y position of the top edge of this BridgeTile (in px)
	 * \param dim The width and height of this BridgeTile (in px)
	 * \param c The color of this BridgeTile
	 */
	BridgeTile(const unsigned int x, const unsigned int y, const Vec2i dim, ColorRGBA c);

	/** \brief Destroy this BridgeTile */
	~BridgeTile();

	void SetSurroundingTiles(const Tile *above, const Tile *right,
							 const Tile *below, const Tile *left);

	bool IsCollidingWith(const GameObject *const go) const;

private:
	static SDL_Surface *bridgeConnectorVert;	///< Image of the bridge over open water
	static SDL_Surface *bridgeEndTop;		///< Image of the bridge ending on land
	static unsigned int bridgetilecount;	///< Number of BridgeTiles created
};

/** \brief Representation of a land (open grass land) Tile */
class LandTile : public Tile {
public:
	/** \brief Create a LandTile
	 *
	 * \param x The x position of the left edge of this LandTile (in px)
	 * \param y The y position of the top edge of this LandTile (in px)
	 * \param dim The width and height of this LandTile (in px)
	 */
	LandTile(const unsigned int x, const unsigned int y, const Vec2i dim);

	/** \brief Create a LandTile with an associated color
	 *
	 * \param x The x position of the left edge of this LandTile (in px)
	 * \param y The y position of the top edge of this LandTile (in px)
	 * \param dim The width and height of this LandTile (in px)
	 * \param c The color of this LandTile
	 */
	LandTile(const unsigned int x, const unsigned int y, const Vec2i dim, ColorRGBA c);

	/** \brief Destroy this LandTile */
	~LandTile();

	void SetSurroundingTiles(const Tile *above, const Tile *right,
							 const Tile *below, const Tile *left);

	bool IsCollidingWith(const GameObject *const go) const;

private:
	static SDL_Surface *land1;		///< Image of land with no grass
	static SDL_Surface *land2;		///< Image of land with some grass
	static SDL_Surface *land3;		///< Image of land with more grass
	static SDL_Surface *land4;		///< Image of land with lots of grass
	static unsigned int landtilecount;	///< Number of LandTiles created
};

/** \brief Representation of a Tile containing a large rock */
class RockTile : public Tile {
public:
	/** \brief Create a RockTile
	 *
	 * \param x The x position of the left edge of this RockTile (in px)
	 * \param y The y position of the top edge of this RockTile (in px)
	 * \param dim The width and height of this RockTile (in px)
	 */
	RockTile(const unsigned int x, const unsigned int y, const Vec2i dim);

	/** \brief Create a RockTile with an associated color
	 *
	 * \param x The x position of the left edge of this RockTile (in px)
	 * \param y The y position of the top edge of this RockTile (in px)
	 * \param dim The width and height of this RockTile (in px)
	 * \param c The color of this RockTile
	 */
	RockTile(const unsigned int x, const unsigned int y, const Vec2i dim, ColorRGBA c);
	~RockTile();

	void SetSurroundingTiles(const Tile *above, const Tile *right,
							 const Tile *below, const Tile *left);

	bool IsCollidingWith(const GameObject *const go) const;

private:
	static SDL_Surface *rock;		///< Image of the rock tile
	static unsigned int rocktilecount;	///< Number of RockTiles created
};

/** \brief Representation of a Tile of a tree */
class TreeTile : public Tile {
public:
	/** \brief Create a TreeTile
	 *
	 * \param x The x position of the left edge of this TreeTile (in px)
	 * \param y The y position of the top edge of this TreeTile (in px)
	 * \param dim The width and height of this TreeTile (in px)
	 */
	TreeTile(const unsigned int x, const unsigned int y, const Vec2i dim);

	/** \brief Create a TreeTile with an associated color
	 *
	 * \param x The x position of the left edge of this TreeTile (in px)
	 * \param y The y position of the top edge of this TreeTile (in px)
	 * \param dim The width and height of this TreeTile (in px)
	 * \param c The color of this TreeTile
	 */
	TreeTile(const unsigned int x, const unsigned int y, const Vec2i dim, ColorRGBA c);

	/** \brief Destroy this TreeTile */
	~TreeTile();

	void SetSurroundingTiles(const Tile *above, const Tile *right,
							 const Tile *below, const Tile *left);

	bool IsCollidingWith(const GameObject *const go) const;

private:
	static SDL_Surface *tree;	///< Image of the tree tile
	static unsigned int treetilecount;	///< Number of TreeTiles created
};

/** \brief Representation of a Tile containing (mostly) water */
class WaterTile : public Tile {
public:
	/** \brief Create a WaterTile
	 *
	 * \param x The x position of the left edge of this WaterTile (in px)
	 * \param y The y position of the top edge of this WaterTile (in px)
	 * \param dim The width and height of this WaterTile (in px)
	 */
	WaterTile(const unsigned int x, const unsigned int y, const Vec2i dim);

	/** \brief Create a WaterTile with an associated color
	 *
	 * \param x The x position of the left edge of this WaterTile (in px)
	 * \param y The y position of the top edge of this WaterTile (in px)
	 * \param dim The width and height of this WaterTile (in px)
	 * \param c The color of this WaterTile
	 */
	WaterTile(const unsigned int x, const unsigned int y, const Vec2i dim, ColorRGBA c);

	/** \brief Destroy this WaterTile */
	~WaterTile();

	void SetSurroundingTiles(const Tile *above, const Tile *right,
							 const Tile *below, const Tile *left);

	bool IsCollidingWith(const GameObject *const go) const;

private:
	/** \brief Representation of an associated pair of images and rotations
	 * 
	 * This is used in a lookup table for quick selection of appropriate images
	 * and rotations.
	 */
	struct ImagePair {
		SDL_Surface **img;		///< The image of the pair
		float rotation;		///< The rotation associated with the image
	};

	static ImagePair imagepairs[2][2][2][2];	///< Lookup table for images
	static SDL_Surface *lakeNoEdge;			///< open water
	static SDL_Surface *lakeTopEdge;			///< water with land above
	static SDL_Surface *lakeRightEdge;		///< water with land to the right
	static SDL_Surface *lakeTopRightEdge;		///< water with land above and right
	static SDL_Surface *lakeBottomRightEdge;		///< land below and to the right
	static SDL_Surface *lakeTopBottomEdge;	///< land above and below
	static SDL_Surface *lakeRightLeftEdge;	///< land right and left
	static SDL_Surface *lakeTopRightBottomEdge;	///< land above, right, below
	static SDL_Surface *lakeRightBottomLeftEdge;	///< land right, below, left
	static SDL_Surface *lakeAllEdge;			///< water with all edges bordered by land

	static unsigned int watertilecount;		///< Number of WaterTiles created
};

#include "GameObject.hpp"

#endif /* INCLUDED_TILES_H */
// vim: set ts=4 sts=4 sw=4 noet:
