// \file Utility.cpp Source flie for utility functions

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/stat.h>

#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef _MSC_VER
# define WIN32_LEAN_AND_MEAN 1
#include <direct.h>
#include <windows.h>
#endif

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif
#include "SDL_rotozoom.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"

#include "Utility.hpp"

Configuration *config;
volatile bool quit = 0;
unsigned long ticks = 0;
TTF_Font *gamefont;
TTF_Font *introfont;
DialogBoxes *dialogs;
Vec2i screensize;

const char *GetWorkingDirectory()
{
#if defined(__GLIBC__) || defined(__GNU_LIBRARY__)
	return get_current_dir_name();
#elif defined(HAVE_GETCWD)
	static char path[PATH_MAX];
	getcwd(path, PATH_MAX);
	return path;
#elif defined(_MSC_VER)
	static char path[MAX_PATH] = {0};
	_getcwd(path, MAX_PATH);
	return path;
#else
# error Define a method to get the path to the game.
#endif
}

int ChangeDirectory(const char *newdir)
{
#if defined(_POSIX_VERSION) || defined(__unix__) || defined(__unix)
	return chdir(newdir);
#elif defined(_MSC_VER)
	return _chdir(newdir);
#else
# error Define a method to change the working directory
#endif
}

bool FileExists(const char *filename)
{
	struct stat unused;
	return stat(filename, &unused) == 0;
}

SDL_Surface *LoadImageSDL(const char *filename, int width, int height)
{
	SDL_Surface *image = IMG_Load(filename);

	if (!image) {
		fprintf(stderr, "Error loading %s: %s\n", filename, IMG_GetError());
		return NULL;
	}

	if (width <= 0)
		height = static_cast<int>(static_cast<double>(width)/image->w*image->h);
	else if (height <= 0)
		width = static_cast<int>(static_cast<double>(height)/image->h*image->w);

	SDL_Surface *resized;
	if (width > 0 && height > 0) {
		resized = zoomSurface(image,
							  width/static_cast<double>(image->w),
							  height/static_cast<double>(image->h), 1);
		if (!resized) {
			fprintf(stderr, "Unable to resize image: %s\n", SDL_GetError());
			resized = image;
		} else {
			SDL_FreeSurface(image);
		}
	} else {
		resized = image;
	}

	SDL_Surface *displayimage;
	if (resized->flags & SDL_SRCALPHA)
		displayimage = SDL_DisplayFormatAlpha(resized);
	else
		displayimage = SDL_DisplayFormat(resized);

	if (!displayimage) {
		fprintf(stderr, "Error converting image: %s\n",
				SDL_GetError());
		return resized;
	}

	SDL_FreeSurface(resized);
	return displayimage;
}

SDL_Surface *LoadSprite(const char *filename, int rotations,
						int width, int height)
{
	SDL_Surface *image = IMG_Load(filename);

	if (!image) {
		fprintf(stderr, "Error loading %s: %s\n", filename, IMG_GetError());
		return NULL;
	}

	int origheight = image->h/rotations;
	int origwidth = origheight;

#ifdef DEBUG
	if (origheight * rotations != image->h ||
			image->w / origwidth * origwidth != image->w)
		fprintf(stderr,
				"Warning: %s (%d,%d) doesn't divide evenly into %d rotations\n",
				filename, image->w, image->h, rotations);
#endif

	/* Note:  The resizing process is a bit tedious here.
	 * We resize each sprite individually to prevent bleeding between sprites
	 * TODO:  Find a way to resize the whole sheet without bleeding
	 */
	SDL_Surface *resized;
	if (origwidth != width || origheight != height) {
		resized = SDL_CreateRGBSurface(image->flags,
				image->w / origwidth * width,
				image->h / origheight * height,
				image->format->BitsPerPixel,
				image->format->Rmask,
				image->format->Gmask,
				image->format->Bmask,
				image->format->Amask);

		SDL_Surface *sprite = SDL_CreateRGBSurface(image->flags,
				origwidth, origheight,
				image->format->BitsPerPixel,
				image->format->Rmask,
				image->format->Gmask,
				image->format->Bmask,
				image->format->Amask);

		if (resized == NULL || sprite == NULL) {
			fprintf(stderr, "Unable to allocate surface\n");
			if (resized != NULL)
				SDL_FreeSurface(resized);
			if (sprite != NULL)
				SDL_FreeSurface(sprite);
			SDL_FreeSurface(image);
			return NULL;
		}

		// Turn off SRCALPHA so that transparent pixels are copied
		image->flags &= ~SDL_SRCALPHA;
		sprite->flags &= ~SDL_SRCALPHA;

		for (int row=0; row < image->h/origheight; row++) {
			for (int col=0; col < image->w/origwidth; col++) {
				SDL_Rect dim;
				dim.x = origwidth * col;
				dim.y = origheight * row;
				dim.w = origwidth;
				dim.h = origheight;
				if (SDL_BlitSurface(image, &dim, sprite, NULL) < 0)
				{
					fprintf(stderr, "Error resizing %s: %s\n",
							filename, SDL_GetError());
					SDL_FreeSurface(sprite);
					SDL_FreeSurface(resized);
					SDL_FreeSurface(image);
					return NULL;
				}

				SDL_Surface *resizedsprite = zoomSurface(sprite,
							  static_cast<double>(width)/origwidth,
							  static_cast<double>(height)/origheight,
							  1);

				if (resizedsprite == NULL)
				{
					fprintf(stderr, "Error resizing %s: %s\n",
							filename, SDL_GetError());
					SDL_FreeSurface(sprite);
					SDL_FreeSurface(resized);
					SDL_FreeSurface(image);
					return NULL;
				}
				
				resizedsprite->flags &= ~SDL_SRCALPHA;
				assert(resizedsprite->w == width && resizedsprite->h == height);
				dim.x = width * col;
				dim.y = height * row;
				if (resizedsprite == NULL ||
						SDL_BlitSurface(resizedsprite, NULL, resized, &dim) < 0)
				{
					fprintf(stderr, "Error resizing %s: %s\n",
							filename, SDL_GetError());
					SDL_FreeSurface(resizedsprite);
					SDL_FreeSurface(sprite);
					SDL_FreeSurface(resized);
					SDL_FreeSurface(image);
					return NULL;
				}

				SDL_FreeSurface(resizedsprite);
			}
		}

		SDL_FreeSurface(sprite);
		SDL_FreeSurface(image);
	} else {
		resized = image;
	}

	SDL_Surface *displayimage;
	if (resized->flags & SDL_SRCALPHA)
		displayimage = SDL_DisplayFormatAlpha(resized);
	else
		displayimage = SDL_DisplayFormat(resized);

	if (!displayimage) {
		fprintf(stderr, "Error converting image: %s\n",
				SDL_GetError());
		return resized;
	}

	SDL_FreeSurface(resized);
	return displayimage;
}

Mix_Chunk *LoadSound(const char *filename)
{
	Mix_Chunk *sound = Mix_LoadWAV(filename);
	if (sound == NULL)
		fprintf(stderr, "Error loading %s: %s\n", filename, Mix_GetError());

	return sound;
}

int LoadFonts()
{
	gamefont = TTF_OpenFont(GAME_FONT, screensize.y/42);
	if (gamefont == NULL) {
		fprintf(stderr, "Error opening game font (%s): %s\n", GAME_FONT,
				TTF_GetError());
		return -1;
	}
	//TTF_SetFontStyle(gamefont, TTF_STYLE_BOLD);

	introfont = TTF_OpenFont(INTRO_FONT, screensize.y/27);
	if (gamefont == NULL) {
		fprintf(stderr, "Error opening intro font (%s): %s\n", INTRO_FONT,
				TTF_GetError());
		return -1;
	}

	return 0;
}

int BlitSurfaceWithAlpha(SDL_Surface *src, SDL_Rect *srcrect,
						 SDL_Surface *dst, SDL_Rect *dstrect,
						 Uint8 alpha)
{
	if (alpha == 0)
		return SDL_BlitSurface(src, srcrect, dst, dstrect);

	// FIXME:  Fix backwards definition in the code, then change this
	alpha = 255 - alpha;

	// At the moment, we are only worrying about 32bpp
	assert(src->format->BitsPerPixel == 32);

	void *oldpixels = src->pixels;
	Uint32 *newpixels = (Uint32 *)malloc(src->w*src->h*src->format->BytesPerPixel);
	if (newpixels == NULL)
		return -1;

	memcpy(newpixels, oldpixels, src->w*src->h*src->format->BytesPerPixel);

	size_t start, end;
	if (srcrect == NULL) {
		start = 0;
		end = src->w*src->h;
	} else {
		start = srcrect->y * src->w;
		end = (srcrect->y + srcrect->h) * src->w;
	}

	for (size_t p=start; p<end; p++) {
		Uint32 pixel = *(newpixels+p);
		Uint32 temp = pixel & src->format->Amask;
		temp >>= src->format->Ashift;
		temp <<= src->format->Aloss;
		Uint8 pixela = static_cast<Uint8>(temp);

		if (pixela == 255)
			continue;
		else if (pixela >= 255 - alpha)
			pixela = 255;
		else
			pixela += alpha;

		temp = pixela;
		temp >>= src->format->Aloss;
		temp <<= src->format->Ashift;
		*(newpixels+p) &= ~src->format->Amask;
		*(newpixels+p) &= temp & src->format->Amask;
	}

	src->pixels = newpixels;
	int status = SDL_BlitSurface(src, srcrect, dst, dstrect);
	src->pixels = oldpixels;

	free(newpixels);
	return status;
}

int BlitText(TTF_Font *font, const char *text, const SDL_Color *color,
			 SDL_Surface *dst, SDL_Rect *offset)
{
	// If the length of the text is 0, there is nothing to do
	if (text[0] == '\0')
		return 0;

	SDL_Surface *rendered = TTF_RenderText_Solid(font, text, *color);

	if (rendered == NULL) {
		fprintf(stderr, "Error rendering text \"%s\": %s\n",
				text, TTF_GetError());
		return -1;
	}

	int status = SDL_BlitSurface(rendered, NULL, dst, offset);

	SDL_FreeSurface(rendered);
	return status;
}

// vim: set ts=4 sts=4 sw=4 noet:
