/// \file Utility.hpp Header file for utility definitions and functions

/// Include guard for Utility.hpp
#ifndef INCLUDED_UTILITY_H
#define INCLUDED_UTILITY_H 1

#include <cstdlib>

#include "SDL_mixer.h"
#include "SDL_ttf.h"

#include "ColorRGBA.hpp"
#include "Configuration.hpp"
#include "DialogBoxes.hpp"
#include "Vec.hpp"

const char *const GAME_FONT = PKGDATADIR"Fonts/FreeSansBold.ttf";
const char *const INTRO_FONT = PKGDATADIR"Fonts/FreeSansBold.ttf";

/** \brief The mathematical constant pi (as a float) */
#ifndef M_PIf
# ifdef M_PI
const float M_PIf = static_cast<float>(M_PI);
# else
const float M_PIf = 3.14159265358979323846f;
# endif
#endif

/** \brief Game configuration information */
extern Configuration *config;

/** \brief Should the game exit? */
volatile extern bool quit;

/** \brief Global tick counter */
extern unsigned long ticks;

/** \brief Font for rendering game text */
extern TTF_Font *gamefont;

/** \brief Font for rendering the introductory text */
extern TTF_Font *introfont;

/** \brief Dialogs for user interaction */
extern DialogBoxes *dialogs;

/** \brief Dimensions of the screen (drawing surface) */
extern Vec2i screensize;

/**
 * \brief Enumeration of all of the movement directions
 *
 * Directions are defined in the absolute sence, the direction that a creature
 * is facing when it returns a Direction is not relevant.  Directions are
 * defined so that for creature X:
 *
 * 3 2 1
 * 4 X 0
 * 5 6 7
 */
enum Direction {
	DIRECTION_RIGHT = 0,	///< The next larger column (right)
	DIRECTION_UP = 2,		///< The next smaller row (up)
	DIRECTION_LEFT = 4,		///< The next smaller column (left)
	DIRECTION_DOWN = 6,		///< The next larger row (down)
	DIRECTION_NONE
};

/**
 * Convenience arrays for direction computations.
 *
 * This allows things like
 * for (int i=0; i<8; i+=2)
 *	 check_horizontally(x+dx8[i], y+dy8[i]);
 */
/** \brief Convenience array for X */
const short dx8[] = {1,  1,  0, -1, -1, -1,  0,  1};
/** \brief Convenience array for Y */
const short dy8[] = {0, -1, -1, -1,  0,  1,  1,  1};

/// The number of calls to Run per second
const Uint32 UPDATES_PER_SECOND = 60;
const Uint32 TICK_INTERVAL = 1000/UPDATES_PER_SECOND; // In ms

/// The number of rotations for creature sprite sheets
const int CREATURE_ROTATIONS = 8;
/// The number of rotations for tile sprite sheets
const int TILE_ROTATIONS = 4;

/// The length of the fade-in/fade-out time for music (for day/night)
const int MUSIC_FADE_TIME = 3000;
const int MUSIC_FADE_TIME_TICKS = MUSIC_FADE_TIME*UPDATES_PER_SECOND/1000;

/// The length of the fade-in/fade-out time for music as a result of SoundFX
const int MUSIC_FADE_TIME_SFX = 4*UPDATES_PER_SECOND;

/** \name CreatureParameters
 * Tunable parameters for creatures
 */
//@{
const float CHARLIE_DEFAULT_MAX_SPEED		= 0.04f;
const float BAT_DEFAULT_MAX_SPEED			= CHARLIE_DEFAULT_MAX_SPEED*1.3f;
const float BEE_DEFAULT_MAX_SPEED			= CHARLIE_DEFAULT_MAX_SPEED*0.6f;
const float BUTTERFLY_DEFAULT_MAX_SPEED		= 0.0f;
const float CENTIPEDE_DEFAULT_MAX_SPEED		= CHARLIE_DEFAULT_MAX_SPEED*0.5f;
const float DOOR_DEFAULT_MAX_SPEED			= 0.0f;
const float FLOWER_DEFAULT_MAX_SPEED		= 0.0f;
const float GOAL_DEFAULT_MAX_SPEED			= CHARLIE_DEFAULT_MAX_SPEED*2.0f;
const float MILLIPEDE_DEFAULT_MAX_SPEED		= CHARLIE_DEFAULT_MAX_SPEED*0.5f;
const float SCORPION_DEFAULT_MAX_SPEED		= CHARLIE_DEFAULT_MAX_SPEED*0.6f;
const float SNAKE_DEFAULT_MAX_SPEED			= CHARLIE_DEFAULT_MAX_SPEED*1.0f;
const float SWITCH_DEFAULT_MAX_SPEED		= 0.0f;
const float TIGERCUB_DEFAULT_MAX_SPEED		= CHARLIE_DEFAULT_MAX_SPEED*1.1f;

const float BEE_AGGRO_MAX_SPEED				= CHARLIE_DEFAULT_MAX_SPEED*0.7f;
const float TIGERCUB_AGGRO_MAX_SPEED		= CHARLIE_DEFAULT_MAX_SPEED*1.4f;
const float SNAKE_AGGRO_MAX_SPEED			= CHARLIE_DEFAULT_MAX_SPEED*1.3f;
const float SCORPION_AGGRO_MAX_SPEED		= CHARLIE_DEFAULT_MAX_SPEED*0.8f;
const float MILLIPEDE_AGGRO_MAX_SPEED		= CHARLIE_DEFAULT_MAX_SPEED*0.6f;

const float SCORPION_ATTACK_SPEED			= CHARLIE_DEFAULT_MAX_SPEED*2.0f;

const float BAT_DEFAULT_RADIUS				= 0.5f;
const float BEE_DEFAULT_RADIUS				= 0.3f;
const float BUTTERFLY_DEFAULT_RADIUS		= 0.5f;
const float CENTIPEDE_DEFAULT_RADIUS		= 0.25f;
const float CHARLIE_DEFAULT_RADIUS			= 0.25f;
const float DOOR_DEFAULT_RADIUS				= 0.5f;
const float FLOWER_DEFAULT_RADIUS			= 0.4f;
const float GOAL_DEFAULT_RADIUS				= 0.5f;
const float MILLIPEDE_DEFAULT_RADIUS		= 0.25f;
const float SCORPION_DEFAULT_RADIUS			= 0.5f;
const float SNAKE_DEFAULT_RADIUS			= 0.5f;
const float SWITCH_DEFAULT_RADIUS			= 0.5f;
const float TIGERCUB_DEFAULT_RADIUS			= 0.5f;

const float MILLIPEDE_AGGRO_RADIUS			= 5.0f;
const float SCORPION_DAY_AGGRO_RADIUS		= 2.0f;
const float SCORPION_NIGHT_AGGRO_RADIUS		= 5.0f;
const float SNAKE_AGGRO_RADIUS				= 4.0f;
const float TIGERCUB_DAY_AGGRO_RADIUS		= 4.0f;
const float TIGERCUB_NIGHT_AGGRO_RADIUS		= 1.0f;

const float SCORPION_ATTACK_RADIUS			= 1.0f;

const int BAT_ANIMATION_SPEED				= 10;
const int BEE_ANIMATION_SPEED				= 10;
const int FLOWER_ANIMATION_TIME				= 10;
const int SCORPION_ANIMATION_SPEED			= 10;
const int TIGERCUB_ANIMATION_SPEED			= 10;
const int CENTIPEDE_ANIMATION_SPEED			= 10;
const int MILLIPEDE_ANIMATION_SPEED			= 10;
const int GOAL_ANIMATION_SPEED				= 10;
const int SNAKE_ANIMATION_SPEED				= 10;
const int CHARLIE_ANIMATION_SPEED			= 10;

/// Distance at which Charlie begins to influence the Scorpion alpha
static const float SCORPION_ALPHA_DIST = 10.0f;
/// Alpha of Scorpion when Charlie is not near
static const int SCORPION_BASE_ALPHA = 50;
//@}

#undef max
template<typename type>
inline type max(type obj1, type obj2) { return (obj1 > obj2) ? obj1 : obj2; }

#undef min
template<typename type>
inline type min(type obj1, type obj2) { return (obj1 < obj2) ? obj1 : obj2; }

/** \brief Get the path to the current working directory
 * \return The current working directory
 */
const char *GetWorkingDirectory();

/** \brief Change the current working directory
 * \param newdir Directory to move to
 */
int ChangeDirectory(const char *newdir);

/** \brief Does this file exist?
 * \param The filename to test for existence
 * \return true if the file exists, false otherwise
 */
bool FileExists(const char *filename);

#undef LoadImageSDL
/** \brief Load an image from a file
 *
 * This function loads an image into a surface that is ready to be efficiently
 * blitted to the screen.  It must be released using SDL_FreeSurface().
 * \param filename The image to load
 * \param width The preferred image width (<=0 disables resizing)
 * \param height The preferred image height (<=0 disables resizing)
 * \return Pointer to the loaded image (surface) or NULL on error
 */
SDL_Surface *LoadImageSDL(const char *filename, int width = 0, int height = 0);

/** \brief Load and resizes a sprite sheet
 *
 * This function loads a sprite sheet from a file, resizes it so that the
 * sprites will have the given dimension and prepares the surface to be
 * efficiently blitted to the screen.  It must be released using
 * SDL_FreeSurface().
 * \param filename The image to load
 * \param rotations The number of rotations (rows) in the sprite sheet
 * \param width The width of a single sprite
 * \param height The height of a single sprite
 * \return Pointer to the loaded image (surface) or NULL on error
 */
SDL_Surface *LoadSprite(const char *filename, int rotations,
						int width, int height);

/** \brief Load a sound from a file
 * \param filename The sound file to load
 * \return Pointer to the sound loaded or NULL on error
 */
Mix_Chunk *LoadSound(const char *filename);

/** \brief Prepare fonts for use */
int LoadFonts();

/** \brief Blit a surface with additional alpha
 *
 * Similar to SDL_BlitSurface() with additional alpha added to the source
 * \brief src Source  surface (surface to be blitted)
 * \brief srcrect Dimensions of source to blit (all if NULL)
 * \brief dst Surface to blit to
 * \brief dstrect Offset for the blit (only x and y used)
 * \brief alpha The additional alpha to add to the source
 * \return 0 on success, -1 otherwise
 */
int BlitSurfaceWithAlpha(SDL_Surface *src, SDL_Rect *srcrect,
						 SDL_Surface *dst, SDL_Rect *dstrect,
						 Uint8 alpha);

/** \brief Render and blit text to a surface
 *
 * \param font The font to render the text with
 * \param text The text to render
 * \param color The color to render the text
 * \param dst The surface to render the text to
 * \param offset The location on the surface to render to
 * \return 0 on success, -1 on failure
 */
int BlitText(TTF_Font *font, const char *text, const SDL_Color *color,
			 SDL_Surface *dst, SDL_Rect *offset);

/** \brief Get a value that sinusoidally oscillates over time
 * \param min The minimum value of the oscillation
 * \param max The maximum value of the oscillation
 * \param period The period of the oscillation (in ticks)
 * \param phase Phase angle to be added to the sinusoid
 */
template <typename type>
type GetOscillatingValue(type min, type max,
						 float period = 200*M_PIf, float phase = 0.0f)
{
	float amp = (max - min)/2.0f;
	float theta = ticks*2.0f*M_PIf/period;
	return static_cast<type>(min + amp*(1.0f+sinf(theta + phase)));
}

/** \brief Get a random floating-point value between two bounds (inclusive) */
template <typename type>
inline type GetRandomFloat(type min, type max)
{
	return rand()/(RAND_MAX/(max-min)) + min;
}

/** \brief Get a random integral value between two bounds (inclusive) */
template <typename type>
inline type GetRandomInt(type min, type max)
{
	if (min > max) {
		int temp = max;
		max = min;
		min = temp;
	}

	if (max - min > RAND_MAX)
		// To spread values across range use floating version & floor
		return static_cast<type>(GetRandomFloat(static_cast<double>(min),
												static_cast<double>(max)));
	else if (max - min == RAND_MAX)
		// To prevent overflow of max-min+1, special case this
		return rand() + min;

	// Prevent favoring some numbers (think [0-3]%3, 0 is favored 2:1)
	int range = static_cast<int>(max - min) + 1;
	int excess = (static_cast<unsigned>(RAND_MAX)+1)%range;

	type generated;
	do {
		generated = rand();
	} while (generated > RAND_MAX - excess);

	// Note:  Division instead of mod because linear congruential random number
	//		  generators are more random in upper bits
	return generated/(RAND_MAX/range) + min;
}

#endif /* INCLUDED_UTILITY_H */
// vim: set ts=4 sts=4 sw=4 noet:
