/// \file Vec.hpp Header file for the Vec class

/// Include guard for Vec.hpp
#ifndef INCLUDED_VEC_H
#define INCLUDED_VEC_H 1

#include <vector>
#include <cmath>

/**
 * \brief Class representing a mathematical vector
 */
template<unsigned int N, typename type>
struct Vec {
	Vec(type x = 0, type y = 0);
	Vec(const std::vector<type>& values);
	Vec(const type *values);

	/** \brief Destructs this Vec */
	~Vec() {};

	type Length() const;
	
	type& operator[](size_t i) { return values[i]; }
	const type& operator[](size_t i) const { return values[i]; }

	Vec<N,type>& operator+=(const Vec<N,type>& vec);
	Vec<N,type>& operator-=(const Vec<N,type>& vec);

	Vec<N,type>& operator+=(type val);
	Vec<N,type>& operator-=(type val);
	Vec<N,type>& operator*=(type val);
	Vec<N,type>& operator/=(type val);

	Vec<N,type> Rescaled(type scaledlength) const;
	Vec<N,type> PerpVec() const;

	type Distance(const Vec<N,type>& vec) const;
#pragma pack(push)
#pragma pack(1)
	union {
		struct { type x, y; };
		type values[N];
	};
#pragma pack(pop)
};

template<unsigned int N, typename type>
Vec<N,type>::Vec(type xv, type yv)
{
	x = xv;
	y = yv;
	for (size_t i=2; i < N; i++)
		values[i] = 0;
}

template<unsigned int N, typename type>
Vec<N,type>::Vec(const std::vector<type>& vals)
{
	for (size_t i=0; i < N; i++)
		values[i] = vals[i];
}

template<unsigned int N, typename type>
Vec<N,type>::Vec(const type *vals)
{
	for (size_t i=0; i < N; i++)
		values[i] = *(vals+i);
}

template<unsigned int N, typename type>
type Vec<N,type>::Length() const
{
	type squaresum = 0;
	for (size_t i=0; i<N; i++)
		squaresum += values[i] * values[i];
	return static_cast<type>(sqrtf(static_cast<float>(squaresum)));
}

template<unsigned int N, typename type>
Vec<N,type>& Vec<N,type>::operator+=(const Vec<N,type>& vec)
{
	for (size_t i=0; i<N; i++)
		values[i] += vec.values[i];
	return *this;
}

template<unsigned int N, typename type>
Vec<N,type>& Vec<N,type>::operator-=(const Vec<N,type>& vec)
{
	for (size_t i=0; i<N; i++)
		values[i] -= vec.values[i];
	return *this;
}

template<unsigned int N, typename type>
Vec<N,type>& Vec<N,type>::operator+=(type val)
{
	for (size_t i=0; i<N; i++)
		values[i] += val;
	return *this;
}

template<unsigned int N, typename type>
Vec<N,type>& Vec<N,type>::operator-=(type val)
{
	for (size_t i=0; i<N; i++)
		values[i] -= val;
	return *this;
}

template<unsigned int N, typename type>
Vec<N,type>& Vec<N,type>::operator*=(type val)
{
	for (size_t i=0; i<N; i++)
		values[i] *= val;
	return *this;
}

template<unsigned int N, typename type>
Vec<N,type>& Vec<N,type>::operator/=(type val)
{
	for (size_t i=0; i<N; i++)
		values[i] /= val;
	return *this;
}

template<unsigned int N, typename type>
Vec<N,type> Vec<N,type>::PerpVec() const
{
	// FIXME:  Only valid for Vec2
	return Vec<N,type>(-y,x);
}

template<unsigned int N, typename type>
Vec<N,type> Vec<N,type>::Rescaled(type scaledlength) const
{
	if (Length() == 0)
		return *this;

	Vec<N,type> result = *this;
	return result *= scaledlength/Length();
}

template<unsigned int N, typename type>
type Vec<N,type>::Distance(const Vec<N,type>& vec) const
{
	type squaredist = values[0]*values[0];
	for (size_t i=1; i<N; i++)
		squaredist += values[i]*values[i];

	return static_cast<type>(sqrtf(static_cast<float>(squaredist)));
}

template<unsigned int N, typename type>
bool operator==(const Vec<N,type>& vec1, const Vec<N,type>& vec2)
{
	for (size_t i=0; i<N; i++)
		if (vec1[i] != vec2[i])
			return false;
	return true;
}

template<unsigned int N, typename type>
bool operator!=(const Vec<N,type>& vec1, const Vec<N,type>& vec2)
{
	return !(vec1 == vec2);
}

template<unsigned int N, typename type>
Vec<N,type> operator+(const Vec<N,type>& vec1, const Vec<N,type>& vec2)
{
	Vec<N,type> result = vec1;
	result += vec2;
	return result;
}

template<unsigned int N, typename type>
Vec<N,type> operator-(const Vec<N,type>& vec1, const Vec<N,type>& vec2)
{
	Vec<N,type> result = vec1;
	result -= vec2;
	return result;
}

template<unsigned int N, typename type>
Vec<N,type> operator+(const Vec<N,type>& vec, type val)
{
	Vec<N,type> result = vec;
	result += val;
	return result;
}

template<unsigned int N, typename type>
Vec<N,type> operator-(const Vec<N,type>& vec, type val)
{
	Vec<N,type> result = vec;
	result -= val;
	return result;
}

template<unsigned int N, typename type>
Vec<N,type> operator*(const Vec<N,type>& vec, type val)
{
	Vec<N,type> result = vec;
	result *= val;
	return result;
}

template<unsigned int N, typename type>
Vec<N,type> operator/(const Vec<N,type>& vec, type val)
{
	Vec<N,type> result = vec;
	result /= val;
	return result;
}

typedef Vec<2,float> Vec2f;
typedef Vec<2,int> Vec2i;

#endif /* INCLUDED_VEC_H */
