/// \file Version.hpp Header file for the Version class

/// Include guard for Version.hpp
#ifndef INCLUDED_VERSION_H
#define INCLUDED_VERSION_H 1

#include <sstream>

/**
 * \brief Class representing version information (major,minor,patch)
 */
struct Version {
	/** \brief Constructs a Version of the given values
	* \param major Major version number
	* \param minor Minor version number
	* \param patch Patch level
	*/
	Version(int major, int minor = 0, int patch = 0);

	/** \brief Construct a Version from SDL version information
	 * \param sdlversion The version of SDL to represent
	 */
	Version(const SDL_version *sdlversion);

	/** \brief Destructs this Version */
	~Version() {};

	/** \brief String representation of this Version
	 * \return String representing this Version as MM.mm.pp
	 */
	const char *c_str() const;

	int major;
	int minor;
	int patch;
};

inline Version::Version(int maj, int min, int pat)
{
	// Can't use constructor syntax since glibc defines major() & minor() macros
	major = maj;
	minor = min;
	patch = pat;
}

inline Version::Version(const SDL_version *sdlversion)
{
	// Can't use constructor syntax since glibc defines major() & minor() macros
	major = sdlversion->major;
	minor = sdlversion->minor;
	patch = sdlversion->patch;
}

inline const char *Version::c_str() const
{
	std::ostringstream out;
	out << major << "." << minor << "." << patch;
	return out.str().c_str();
}

inline bool operator==(const Version& ver1, const Version& ver2)
{
	return ver1.major == ver2.major &&
		   ver1.minor == ver2.minor &&
		   ver1.patch == ver2.patch;
}

inline bool operator!=(const Version& ver1, const Version& ver2)
{
	return !(ver1 == ver2);
}

inline bool operator>=(const Version& ver1, const Version& ver2)
{
	return ver1.major > ver2.major ||
		   (ver1.major == ver2.major &&
			(ver1.minor > ver2.minor ||
			 (ver1.minor == ver2.minor &&
			  ver1.patch >= ver2.patch)));
}

#endif /* INCLUDED_VERSION_H */
// vim: set ts=4 sts=4 sw=4 noet:
