/// \file WinRegConfiguration.cpp Source file for the WinRegConfiguration class (Windows)

#define WIN32_LEAN_AND_MEAN 1
#include <windows.h>

#include "WinRegConfiguration.hpp"
#include "Utility.hpp"

static void PrintRegistryError(LONG status)
{
	LPSTR message;
	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
				  NULL, status, 0, reinterpret_cast<LPSTR>(&message), 0, NULL);
	dialogs->ErrorMessage("Error loading WinRegConfiguration:\n%s", message);
	LocalFree(message);
}

static bool GetRegistryValue(LPCSTR value, LPDWORD data)
{
	HKEY softwarekey, geukakey;
	DWORD disposition;
	LONG status;

	status = RegCreateKeyEx(HKEY_CURRENT_USER, "Software",
							0, 0, 0, KEY_READ, NULL, &softwarekey, &disposition);
	if (status != ERROR_SUCCESS) {
		PrintRegistryError(status);
		return false;
	}

	status = RegCreateKeyEx(softwarekey, "Geuka",
							0, 0, 0, KEY_READ, NULL, &geukakey, &disposition);
	RegCloseKey(softwarekey);

	if (status == ERROR_SUCCESS) {
		if (disposition == REG_OPENED_EXISTING_KEY) {
			DWORD dat;
			DWORD datsize = sizeof(data);
			status = RegQueryValueEx(geukakey, value, 0, NULL,
									 reinterpret_cast<LPBYTE>(&dat), &datsize);
	
			if (status == ERROR_SUCCESS) {
				*data = dat;
				return true;
			}
		}

		status = RegCloseKey(geukakey);
		if (status != ERROR_SUCCESS)
			PrintRegistryError(status);
	} else {
		PrintRegistryError(status);
	}

	return false;
}

static void SetRegistryValue(LPCSTR value, DWORD data)
{
	HKEY softwarekey, geukakey;
	DWORD disposition;
	LONG status;

	status = RegCreateKeyEx(HKEY_CURRENT_USER, "Software",
							0, 0, 0, KEY_SET_VALUE, NULL, &softwarekey, &disposition);
	if (status != ERROR_SUCCESS) {
		PrintRegistryError(status);
		return;
	}

	status = RegCreateKeyEx(softwarekey, "Geuka",
							0, 0, 0, KEY_SET_VALUE, NULL, &geukakey, &disposition);
	RegCloseKey(softwarekey);

	if (status == ERROR_SUCCESS) {
		DWORD datasize = sizeof(data);
		status = RegSetValueEx(geukakey, value, 0, REG_DWORD,
							   reinterpret_cast<LPBYTE>(&data), datasize);
	
		if (status != ERROR_SUCCESS)
			PrintRegistryError(status);

		status = RegCloseKey(geukakey);
		if (status != ERROR_SUCCESS)
			PrintRegistryError(status);
	} else {
		PrintRegistryError(status);
	}
}

WinRegConfiguration::WinRegConfiguration() :
	Configuration()
{
	DWORD keyvalue;

	if (GetRegistryValue("Levels Completed", &keyvalue))
		if (keyvalue > 0)
			levelscompleted = keyvalue;

	if (GetRegistryValue("Fullscreen", &keyvalue))
		fullscreen = keyvalue != 0;

	if (GetRegistryValue("Window Width", &keyvalue))
		if (keyvalue > 0)
		{
			windowwidth = keyvalue;
			forcevideo = true;
		}

	if (GetRegistryValue("Window Height", &keyvalue))
		if (keyvalue > 0)
		{
			windowheight = keyvalue;
			forcevideo = true;
		}

	if (GetRegistryValue("Color Mode", &keyvalue))
		if (keyvalue > 0)
		{
			colordepth = keyvalue;
			forcevideo = true;
		}

	if (GetRegistryValue("Sound Mode", &keyvalue))
		soundon = keyvalue != 0;
}

WinRegConfiguration::~WinRegConfiguration()
{
}

void WinRegConfiguration::Save() const
{
	SetRegistryValue("Levels Completed", levelscompleted);
	SetRegistryValue("Fullscreen", fullscreen);
	SetRegistryValue("Window Width", windowwidth);
	SetRegistryValue("Window Height", windowheight);
	SetRegistryValue("Color Mode", colordepth);
	SetRegistryValue("Sound Mode", soundon);
}

// vim: set ts=4 sts=4 sw=4 noet:
