/// \file main.cpp Source file containing the GameX-called control functions

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iostream>
#include <cstdlib>

#ifdef WITH_GLSDL
# include "glSDL.h"
#else
# include "SDL.h"
#endif

#include "SDL_mixer.h"
#include "SDL_ttf.h"

#include "Version.hpp"
#include "Utility.hpp"
#include "Level.hpp"
#include "MainMenu.hpp"

#ifdef WITH_WINGUI
#include "DialogBoxesWindows.hpp"
#elif defined(WITH_GTK)
#include "DialogBoxesGTK.hpp"
#else
# error Windowing system not selected
#endif

#ifdef WITH_WINREGCONFIG
# include "WinRegConfiguration.hpp"
#else
# include "FileConfiguration.hpp"
#endif

/// \brief Number of consecutive slow frames before reducing framerate
const unsigned int SLOW_THRESHOLD = 5;
/// \brief Number of consecutive fast frames before increasing framerate
const unsigned int FAST_THRESHOLD = 300;

/** \brief Screen surface for drawing */
static SDL_Surface *screen;

/** \brief Window icon */
static SDL_Surface *icon;

static MainMenu *menuSystem;

/** \brief Initialize the game */
void GameInit(int *argc, char ***argv)
{
	screensize = Vec2i(screen->w, screen->h);
	icon = LoadImageSDL(PKGDATADIR"Artwork/icons/32x32.png");

	SDL_WM_SetCaption("Geuka", "Geuka");
	SDL_WM_SetIcon(icon, NULL);

	LoadFonts();

#ifdef WITH_WINGUI
	dialogs = new DialogBoxesWindows();
#elif defined(WITH_GTK)
	dialogs = new DialogBoxesGTK(argc, argv);
#else
# error Windowing system not selected
#endif

	menuSystem = new MainMenu(screen->w, screen->h);
}

/** \brief Run one cycle of this game
 *
 * This function is called by GameX once per game cycle */
void GameRun(void)
{
	ticks++;
	menuSystem->Update();
	SoundManager::Update();
}

/** \brief Draw the current state of the game
 *
 * This function is called by GameX whenever it needs to update the screen
 */
void GameDraw(void)
{
	menuSystem->Draw(screen);
	SDL_Flip(screen);
}

/** \brief Cleanup the game and prepare for termination
 *
 * This function is called by GameX after the game is closed
 */
void GameCleanup(void)
{
	delete menuSystem;
	delete dialogs;
	SDL_FreeSurface(icon);
}


int main(int argc, char **argv)
{
	if (SDL_Init(SDL_INIT_AUDIO | SDL_INIT_VIDEO | SDL_INIT_TIMER) == -1) {
		std::cerr << "Error initializing SDL: " << SDL_GetError() << '\n';
		exit(EXIT_FAILURE);
	}

	// Enable UNICODE so that the keysym is set for keyboard input
	SDL_EnableUNICODE(1);

	// Cleanup SDL when we exit
	atexit(SDL_Quit);

	if (TTF_Init() == -1) {
		std::cerr << "Error initializing SDL_ttf: " << TTF_GetError() << '\n';
		exit(EXIT_FAILURE);
	}

	// Cleanup SDL_ttf when we exit
	atexit(TTF_Quit);

	if (Mix_OpenAudio(44100, MIX_DEFAULT_FORMAT, 2, 2048) == -1) {
		std::cerr << "Error initializing SDL_mixer: " << Mix_GetError() << '\n';
		exit(EXIT_FAILURE);
	}

	// Cleanup SDL_mixer when we exit
	atexit(Mix_CloseAudio);

	Mix_AllocateChannels(SOUND_NUM_CHANNELS);

#ifdef WITH_WINREGCONFIG
	config = new WinRegConfiguration();
#else
	try {
		config = new FileConfiguration(FileConfiguration::DefaultLocation());
	} catch (...) {
		config = new FileConfiguration();
	}
#endif

	Uint32 flags = SDL_ANYFORMAT | SDL_HWSURFACE;

#ifdef WITH_GLSDL
	flags |= SDL_GLSDL | SDL_DOUBLEBUF;
#endif

	if (config->GetFullscreen())
		flags |= SDL_FULLSCREEN;

	// 0 is interpreted as "no change" in SDL > 1.2.10
	if (!config->GetForceVideoMode() &&
		Version(SDL_Linked_Version()) >= Version(1,2,10))
		screen = SDL_SetVideoMode(0, 0, 0, flags);
	else
		screen = SDL_SetVideoMode(config->GetPreferredWindowWidth(), 
								  config->GetPreferredWindowHeight(),
								  config->GetColorDepth(), flags);

	if (!screen) {
		std::cerr << "Unable to set video mode: " << SDL_GetError() << '\n';
		exit(EXIT_FAILURE);
	}

	GameInit(&argc, &argv);

	Uint32 next_tick = SDL_GetTicks() + TICK_INTERVAL;
	bool drawframe = true;
	bool runningslow = false;
	unsigned int slowcount = 0;		// Number of consecutive frames w/o delay
	unsigned int fastcount = 0;		// Number of consecutive frames w/ delay

	SDL_Event event;
	while (!quit) {
		// Handle pending events
		while (SDL_PollEvent(&event)) {
			switch (event.type) {
				case SDL_KEYDOWN:
				case SDL_KEYUP:
					menuSystem->KeyboardEvent(&event.key);
					break;
				case SDL_MOUSEMOTION:
					menuSystem->MouseMotionEvent(&event.motion);
					break;
				case SDL_MOUSEBUTTONDOWN:
				case SDL_MOUSEBUTTONUP:
					menuSystem->MouseButtonEvent(&event.button);
					break;
				case SDL_QUIT:
					quit = true;
					break;
				default:
					// Other events not currently handled
					break;
			}
		}

		// Do it
		GameRun();

		// Draw it
		if (slowcount > SLOW_THRESHOLD)
			runningslow = true;
		else if (fastcount > FAST_THRESHOLD)
			runningslow = false;

		if (!runningslow || drawframe)
			GameDraw();
		drawframe = !drawframe;

		// Wait
		Uint32 now = SDL_GetTicks();
		if (now >= next_tick)
		{
			fastcount = 0;
			slowcount++;

			if (now > next_tick + TICK_INTERVAL)
				// Over 1 tick behind, catch up
				next_tick = now + TICK_INTERVAL;
		}
		else if (now < next_tick)
		{
			fastcount++;
			slowcount = 0;

			SDL_Delay(next_tick - now);
		}
		next_tick += TICK_INTERVAL;
	}

	GameCleanup();

	return EXIT_SUCCESS;
}

// vim: set ts=4 sts=4 sw=4 noet:
