// \file levelvalidator.cpp Simple utility to check if a level file is valid

#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <cstdlib>

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>
#include <xercesc/util/OutOfMemoryException.hpp>

#include <xercesc/sax2/Attributes.hpp>
#include <xercesc/sax2/DefaultHandler.hpp>

XERCES_CPP_NAMESPACE_USE

class LevelValidatorHandler : public DefaultHandler
{
public:
	enum TileType {
		TILE_TYPE_BRIDGE,
		TILE_TYPE_LAND,
		TILE_TYPE_ROCK,
		TILE_TYPE_WATER
	};

	enum VictoryType {
		VICTORY_UNSET,
		VICTORY_GOALTILE,
		VICTORY_SURVIVAL,
		VICTORY_COLLECTION
	};


	LevelValidatorHandler();
	~LevelValidatorHandler();

	void printResult();


    // -----------------------------------------------------------------------
    //  Handlers for the SAX ContentHandler interface
    // -----------------------------------------------------------------------
    void startElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname, const Attributes& attrs);
    void characters(const XMLCh* const chars, const unsigned int length);
    void ignorableWhitespace(const XMLCh* const chars, const unsigned int length);
    void resetDocument();


    // -----------------------------------------------------------------------
    //  Handlers for the SAX ErrorHandler interface
    // -----------------------------------------------------------------------
	void warning(const SAXParseException& exc);
    void error(const SAXParseException& exc);
    void fatalError(const SAXParseException& exc);
    void resetErrors();

private:
	void checkLevelElement(const Attributes& attrs);
	void checkTilesElement(const Attributes& attrs);
	void checkTileElement(const Attributes& attrs);
	void checkCreaturesElement(const Attributes& attrs);
	void checkCreatureElement(const Attributes& attrs);
	void checkObjectsElement(const Attributes& attrs);
	void checkObjectElement(const Attributes& attrs);
	void parseColor(const char *color);

	std::string errors;
	std::string warnings;

	std::vector<std::vector<bool> > tiledefined;
	std::vector<std::vector<TileType> > tiletype;
	bool havechameleon;
	VictoryType victorytype;
	bool victorysatisfied;
};

LevelValidatorHandler::LevelValidatorHandler() : havechameleon(false),
												 victorytype(VICTORY_UNSET),
												 victorysatisfied(false)
{
}

LevelValidatorHandler::~LevelValidatorHandler() {}

void LevelValidatorHandler::printResult()
{
	// First, do some final checks to make sure we have everything
	if (!havechameleon)
		errors += "Charlie not found on map!\n";

	for (unsigned int row=0; row<tiledefined.size(); row++)
		for (unsigned int col=0; col<tiledefined[0].size(); col++)
			if (!tiledefined[row][col]) {
				std::stringstream warn;
				warn << "Tile (" << row << "," << col << ") not defined.\n";
				warnings += warn.str(); 
			}

	if (!victorysatisfied)
		switch (victorytype) {
			case VICTORY_UNSET:
				errors += "No victory type set!\n";
				break;
			case VICTORY_GOALTILE:
				errors += "Goal tile victory condition selected, but no "
						  "goal tile placed on the board!\n";
				break;
			case VICTORY_SURVIVAL:
				errors += "Survival victory condition selected, but no "
						  "victory time specified.\n";
				break;
			case VICTORY_COLLECTION:
				errors += "Collection victory condition selected, but no "
						  "butterflies placed on the board!\n";
				break;
			default:
				warnings += "Validator error, unrecognized victory "
							"condition.\n";
				break;
		}

	if (errors.length() > 0)
		std::cout << "Errors:\n" << errors << "\n";

	if (warnings.length() > 0)
		std::cout << "Warnings:\n" << warnings << "\n";

	if (errors.length() == 0 && warnings.length() == 0)
		std::cout << "This level file is valid!\n";
}

void LevelValidatorHandler::checkLevelElement(const Attributes& attrs)
{
	int daylength = 0;
	int dayremaining = 0;
	int survivaltime = 0;

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));
		char *uri = XMLString::transcode(attrs.getURI(i));

		// Ignore xsi attributes
		if (!XMLString::compareIString(uri, "http://www.w3.org/2001/XMLSchema-instance")) {
			XMLString::release(&uri);
			continue;
		}

		if (!XMLString::compareIString(attr, "daylength")) {
			daylength = atoi(attrval);
		} else if (!XMLString::compareIString(attr, "dayremaining")) {
			dayremaining = atoi(attrval);
		} else if (!XMLString::compareIString(attr, "name")) {
			if (!XMLString::stringLen(attrs.getValue(i)))
				warnings += "The level name is empty.\n";
		} else if (!XMLString::compareIString(attr, "victorytype")) {
			if (!XMLString::compareIString(attrval, "goaltile")) {
				victorytype = VICTORY_GOALTILE;
			} else if (!XMLString::compareIString(attrval, "survival")) {
				victorytype = VICTORY_SURVIVAL;
			} else if (!XMLString::compareIString(attrval, "collection")) {
				victorytype = VICTORY_COLLECTION;
			} else
				warnings += "Unknown victory condition.\n";
		} else if (!XMLString::compareIString(attr, "survivaltime")) {
			survivaltime = atoi(attrval);
		} else {
			warnings += "Unrecognized attribute \"";
			warnings += attr;
			warnings += "\" of level element ignored.\n";
		}

		if (daylength < 1)
			errors += "The day length for this level should be > 0\n";
		if (daylength > 60*5)
			warnings += "The day length is > 5 minutes.\n";
		if (dayremaining > daylength || -dayremaining > daylength)
			warnings += "The day remaining is longer than the day length.\n";
		if (victorytype == VICTORY_SURVIVAL) {
			if (survivaltime > 0)
				victorysatisfied = true;
			else
				errors += "Survival victory type selected, but no time "
						  "specified!\n";
		} else if (survivaltime > 0)
			errors += "Survival time specified, but survival victory type"
					  "not selected!\n";

		XMLString::release(&attr);
		XMLString::release(&attrval);
		XMLString::release(&uri);
	}
}

void LevelValidatorHandler::checkTilesElement(const Attributes& attrs)
{
	int cols = 0;
	int rows = 0;

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		if (!XMLString::compareIString(attr, "cols")) {
			cols = atoi(attrval);
		} else if (!XMLString::compareIString(attr, "rows")) {
			rows = atoi(attrval);
		} else {
			warnings += "Unrecognized attribute \"";
			warnings += attr;
			warnings += "\" of tiles element ignored.\n";
		}

		XMLString::release(&attr);
		XMLString::release(&attrval);
	}

	if (rows <= 0) {
		std::stringstream err;
		err << "The \"rows\" attribute of the tiles element must be > 0, " <<
			   "got " << rows << "!\n";
		errors += err.str();
		rows = 1;
	}

	if (cols <= 0) {
		std::stringstream err;
		err << "The \"cols\" attribute of the tiles element must be > 0, " <<
			   "got " << cols << "!\n";
		errors += err.str();
		cols = 1;
	}

	tiledefined.resize(rows);
	for (int row=0; row<rows; row++) {
		tiledefined[row].resize(cols);
		for (int col=0; col<cols; col++)
			tiledefined[row][col] = false;
	}

	tiletype.resize(rows);
	for (int row=0; row<rows; row++) {
		tiletype[row].resize(cols);
		for (int col=0; col<cols; col++)
			tiletype[row][col] = TILE_TYPE_LAND;
	}
}

void LevelValidatorHandler::checkTileElement(const Attributes& attrs)
{
	int row = -1;
	int col = -1;
	bool seencolor = false;
	std::string type;

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		if (!XMLString::compareIString(attr, "row")) {
			row = atoi(attrval);
		} else if (!XMLString::compareIString(attr, "col")) {
			col = atoi(attrval);
		} else if (!XMLString::compareIString(attr, "type")) {
			/* Already handled by DTD and Schema checking */
			type = attrval;
		} else if (!XMLString::compareIString(attr, "color")) {
			seencolor = true;
			parseColor(attrval);
		} else {
			warnings += "Unrecognized attribute \"";
			warnings += attr;
			warnings += "\" of the tiles element ignored.\n";
		}

		XMLString::release(&attrval);
		XMLString::release(&attr);
	}

	if (row < 0 || row >= static_cast<signed>(tiledefined.size())) {
		std::ostringstream err;
		err << "The \"row\" attribute of the tile element must be between "
			   "0 and the number of rows-1 (" << tiledefined.size()-1 <<
			   ").  Got " << row << ".\n";
		errors += err.str();
		row = (row<0)?0:tiledefined.size()-1;
	}

	if (col < 0 || col >= static_cast<signed int>(tiledefined[0].size())) {
		std::ostringstream err;
		err << "The \"col\" attribute of the tile element must be between "
			   "0 and the number of columns-1 (" << tiledefined[0].size()-1 <<
			   ").  Got " << col << ".\n";
		errors += err.str();
		col = (col<0)?0:tiledefined[0].size()-1;
	}

	if (tiledefined[row][col]) {
		std::stringstream warn;
		warn << "Tile (" << row << "," << col << ") defined more than once.\n";
		warnings += warn.str();
	}
	tiledefined[row][col] = true;

	if (!XMLString::compareIString(type.c_str(), "bridge")) {
		tiletype[row][col] = TILE_TYPE_BRIDGE;
	} else if (!XMLString::compareIString(type.c_str(), "land")) {
		tiletype[row][col] = TILE_TYPE_LAND;
	} else if (!XMLString::compareIString(type.c_str(), "rock")) {
		tiletype[row][col] = TILE_TYPE_ROCK;
	} else if (!XMLString::compareIString(type.c_str(), "water")) {
		tiletype[row][col] = TILE_TYPE_WATER;
	} else {
		warnings += "Unrecognized type \"" + type + "\"\n";
	}
}

void LevelValidatorHandler::checkCreaturesElement(const Attributes& attrs)
{
	/* This is a current limitation of the level loading code... */
	if (tiledefined.size() == 0)
		warnings += "The tiles element should precede the creatures element.";

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		warnings += "Unrecognized attribute \"";
		warnings += attr;
		warnings += "\" of the creatures element ignored.\n";

		XMLString::release(&attr);
		XMLString::release(&attrval);
	}
}

void LevelValidatorHandler::checkCreatureElement(const Attributes& attrs)
{
	float x = -1.0f;
	float y = -1.0f;
	std::string type;
	bool seencolor = false;

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		if (!XMLString::compareIString(attr, "x")) {
			x = atof(attrval);
		} else if (!XMLString::compareIString(attr, "y")) {
			y = atof(attrval);
		} else if (!XMLString::compareIString(attr, "type")) {
			/* Mostly handled by Schema and DTD checking */
			type = attrval;
			if (!XMLString::compareIString(attrval, "chameleon"))
				havechameleon = true;
		} else if (!XMLString::compareIString(attr, "color")) {
			seencolor = true;
			parseColor(attrval);
		} else {
			warnings += "Unrecognized attribute \"";
			warnings += attr;
			warnings += "\" of the creature element ignored.\n";
		}

		XMLString::release(&attr);
		XMLString::release(&attrval);
	}

	if (x < 0.0f || x >= tiledefined.size()) {
		std::ostringstream err;
		err << "The \"x\" attribute of the creature element must be between "
			   "0 and the number of rows-1 (" << tiledefined.size()-1 <<
			   ").  Got " << x << ".\n";
		errors += err.str();
		x = (x<0.0f)?0.0f:tiledefined.size()-1;
	}

	if (y < 0.0f || y >= tiledefined[0].size()) {
		std::ostringstream err;
		err << "The \"y\" attribute of the creature element must be between "
			   "0 and the number of columns-1 (" << tiledefined[0].size()-1 <<
			   ").  Got " << y << ".\n";
		errors += err.str();
		y = (y<0.0f)?0.0f:tiledefined[0].size()-1;
	}

	unsigned int row = static_cast<unsigned int>(x);
	unsigned int col = static_cast<unsigned int>(y);
	switch (tiletype[row][col]) {
		case TILE_TYPE_WATER:
			{
				std::stringstream err;
				err << type << " can not be defined on a water tile at (" <<
					   row << "," << col << ").\n";
				errors += err.str();
			}
			break;
		case TILE_TYPE_ROCK:
			{
				std::stringstream err;
				err << type << " can not be defined on a rock tile at (" <<
					row << "," << col << ").\n";
				errors += err.str();
			}
			break;
		default:
			break;
	}

	if (!XMLString::compareIString(type.c_str(), "butterfly") && !seencolor)
		warnings += "butterfly defined without any color.\n";

	if (!XMLString::compareIString(type.c_str(), "butterfly") &&
		victorytype == VICTORY_COLLECTION)
		victorysatisfied = true;
}

void LevelValidatorHandler::checkObjectsElement(const Attributes& attrs)
{
	/* This is a current limitation of the level loading code... */
	if (tiledefined.size() == 0)
		warnings += "The tiles element should precede the creatures element.";

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		warnings += "Unrecognized attribute \"";
		warnings += attr;
		warnings += "\" of the objects element ignored.\n";

		XMLString::release(&attr);
		XMLString::release(&attrval);
	}
}

void LevelValidatorHandler::checkObjectElement(const Attributes& attrs)
{
	float x = -1.0f;
	float y = -1.0f;
	std::string type;
	bool seencolor = false;

	for (unsigned int i=0; i<attrs.getLength(); i++) {
		char *attr = XMLString::transcode(attrs.getLocalName(i));
		char *attrval = XMLString::transcode(attrs.getValue(i));

		if (!XMLString::compareIString(attr, "x")) {
			x = atof(attrval);
		} else if (!XMLString::compareIString(attr, "y")) {
			y = atof(attrval);
		} else if (!XMLString::compareIString(attr, "type")) {
			type = attrval;
			/* Mostly Handled by Schema and DTD checking */
		} else if (!XMLString::compareIString(attr, "color")) {
			seencolor = true;
			parseColor(attrval);
		} else {
			warnings += "Unrecognized attribute \"";
			warnings += attr;
			warnings += "\" of object element ignored.\n";
		}

		XMLString::release(&attr);
		XMLString::release(&attrval);
	}

	if (x < 0.0f || x >= tiledefined.size()) {
		std::ostringstream err;
		err << "The \"x\" attribute of the object element must be between "
			   "0 and the number of rows-1 (" << tiledefined.size()-1 <<
			   ").  Got " << x << ".\n";
		errors += err.str();
		x = (x<0.0f)?0.0f:tiledefined.size()-1;
	}

	if (y < 0.0f || y >= tiledefined[0].size()) {
		std::ostringstream err;
		err << "The \"y\" attribute of the object element must be between "
			   "0 and the number of columns-1 (" << tiledefined[0].size()-1 <<
			   ").  Got " << y << ".\n";
		errors += err.str();
		y = (y<0.0f)?0.0f:tiledefined[0].size()-1;
	}

	unsigned int row = static_cast<unsigned int>(x);
	unsigned int col = static_cast<unsigned int>(y);
	switch (tiletype[row][col]) {
		case TILE_TYPE_WATER:
			{
				std::stringstream err;
				err << type << " can not be defined on a water tile at (" <<
					   row << "," << col << ").\n";
				errors += err.str();
			}
			break;
		case TILE_TYPE_ROCK:
			{
				std::stringstream err;
				err << type << " can not be defined on a rock tile at (" <<
					row << "," << col << ").\n";
				errors += err.str();
			}
			break;
		default:
			break;
	}

	if (!XMLString::compareIString(type.c_str(), "flower") && !seencolor) {
		warnings += "flower defined without any color.\n";
	}
	
	if (!XMLString::compareIString(type.c_str(), "goal")) {
		if (victorytype == VICTORY_GOALTILE)
				victorysatisfied = true;
		else
			warnings += "Goal tile added, but victory condition is not "
						"goal tile.\n";
	}
}

void LevelValidatorHandler::parseColor(const char *color)
{
	if (!XMLString::compareIString(color, "black")) {
		return;
	} else if (!XMLString::compareIString(color, "blue")) {
		return;
	} else if (!XMLString::compareIString(color, "brown")) {
		return;
	} else if (!XMLString::compareIString(color, "green")) {
		return;
	} else if (!XMLString::compareIString(color, "orange")) {
		return;
	} else if (!XMLString::compareIString(color, "red")) {
		return;
	} else if (!XMLString::compareIString(color, "violet")) {
		return;
	} else if (!XMLString::compareIString(color, "white")) {
		return;
	} else if (!XMLString::compareIString(color, "yellow")) {
		return;
	} else {
		std::ostringstream err;
		err << "Color \"" << color << "\" not recognized.\n";
		errors += err.str();
		return;
	}
}

void LevelValidatorHandler::startElement(const XMLCh* const uri,
										 const XMLCh* const localname,
										 const XMLCh* const qname,
										 const Attributes& attrs)
{
	char *element = XMLString::transcode(localname);
	if (!XMLString::compareIString(element, "level")) {
		checkLevelElement(attrs);
	} else if (!XMLString::compareIString(element, "tiles")) {
		checkTilesElement(attrs);
	} else if (!XMLString::compareIString(element, "tile")) {
		checkTileElement(attrs);
	} else if (!XMLString::compareIString(element, "creatures")) {
		checkCreaturesElement(attrs);
	} else if (!XMLString::compareIString(element, "creature")) {
		checkCreatureElement(attrs);
	} else if (!XMLString::compareIString(element, "objects")) {
		checkObjectsElement(attrs);
	} else if (!XMLString::compareIString(element, "object")) {
		checkObjectElement(attrs);
	} else {
		warnings += "Unrecognized element \"";
		warnings += element;
		warnings += "\" ignored.\n";
	}

	XMLString::release(&element);
}

void LevelValidatorHandler::characters(const XMLCh *const chars,
									   const unsigned int length)
{
	char *mchars = XMLString::transcode(chars);
	warnings += "String \"";
	warnings += mchars;
	warnings += "\" ignored.";
	XMLString::release(&mchars);
}

void LevelValidatorHandler::ignorableWhitespace(const XMLCh *const chars,
												const unsigned int length)
{
}

void LevelValidatorHandler::resetDocument()
{
	tiledefined.clear();
	warnings.clear();
	errors.clear();
}

void LevelValidatorHandler::error(const SAXParseException& e)
{
	char *msg = XMLString::transcode(e.getMessage());
	char *file = XMLString::transcode(e.getSystemId());
	std::stringstream err;
	err << "Error in file " << file << " at line " << e.getLineNumber() <<
		   ":\n" << msg << "\n";
	errors += err.str();
	XMLString::release(&msg);
	XMLString::release(&file);
}

void LevelValidatorHandler::fatalError(const SAXParseException& e)
{
	char *msg = XMLString::transcode(e.getMessage());
	char *file = XMLString::transcode(e.getSystemId());
	std::cerr << "Fatal Error parsing file " << file << " at line " <<
				 e.getLineNumber() << ":\n" << msg << std::endl;
	XMLString::release(&msg);
	XMLString::release(&file);
}

void LevelValidatorHandler::warning(const SAXParseException& e)
{
	char *msg = XMLString::transcode(e.getMessage());
	char *file = XMLString::transcode(e.getSystemId());
	std::stringstream err;
	err << "Warning in file " << file << " at line " << e.getLineNumber() <<
		   ":\n" << msg << "\n";
	errors += err.str();
	XMLString::release(&msg);
	XMLString::release(&file);
}

void LevelValidatorHandler::resetErrors()
{
	warnings.clear();
	errors.clear();
}

int main(int argc, char **argv)
{
	if (argc != 2) {
		std::cerr << "Usage:  " << argv[0] << " filename" << std::endl;
		return EXIT_FAILURE;
	}

	try {
		XMLPlatformUtils::Initialize();
	} catch (const XMLException& e) {
		char* msg = XMLString::transcode(e.getMessage());
		std::cerr << "Error initializing Xerces:\n" << msg << std::endl;
		XMLString::release(&msg);
		return EXIT_FAILURE;
	}

	SAX2XMLReader* parser = XMLReaderFactory::createXMLReader();

	// Turn on validation
	parser->setFeature(XMLUni::fgSAX2CoreValidation, true);
	parser->setFeature(XMLUni::fgXercesSchema, true);
	parser->setFeature(XMLUni::fgXercesSchemaFullChecking, true);

	LevelValidatorHandler handler;
	parser->setContentHandler(&handler);
	parser->setErrorHandler(&handler);

	try {
		parser->parse(argv[1]);
	} catch (const OutOfMemoryException&) {
		std::cerr << "Error:  Out of memory!" << std::endl;
		return EXIT_FAILURE;
	} catch (const XMLException& e) {
		char *msg = XMLString::transcode(e.getMessage());
		std::cerr << "Error parsing " << argv[1] << ":\n" << msg <<
			std::endl;
		XMLString::release(&msg);
		return EXIT_FAILURE;
	} catch (...) {
		std::cerr << "Unexpected exception during parsing." <<
			std::endl;
		return EXIT_FAILURE;
	}

	std::cout << "Parsing complete:  ";
	handler.printResult();

	delete parser;
	XMLPlatformUtils::Terminate();

	return EXIT_SUCCESS;
}

// vim: set ts=4 sts=4 sw=4 noet:
